/*
 * Created on 19/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command;

import java.util.Stack;

import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.MedException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;

/**
 * @author Alexander,Bennyamin,Svetlana and Valentin 
 * Created in SSDL lab - Technion Israel
 *
 */
public abstract class Command {
	protected Stack undoOpers = new Stack();

	/**
	 * Method runAndPush
	 * @param  c
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	protected void runAndPush(Command c)
		throws RetriableException, CriticalException {
		c.run();
		undoOpers.push(c);
	}

	/**
	 * Method execute
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	protected abstract void execute()
		throws RetriableException, CriticalException;

	/**
	 * Method undo
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public void undo() throws RetriableException, CriticalException {
		Stack poppedOpers = new Stack();
		while (!undoOpers.empty()) {
			Command c = (Command) undoOpers.pop();
			try {
				c.undo();
				poppedOpers.push(c);
			} catch (RetriableException e) {
				try {
					while (!poppedOpers.empty()) {
						Command c2 = (Command) undoOpers.pop();
						c2.undo();
					}
				} catch (MedException e2) {
					if (poppedOpers.empty()) {
						throw new RetriableException(
							"Undo failed " + "- system remained consistent");
					}
					throw new CriticalException(
							"Undo failed leaving system in inconsistent state");
				}
			}
		}
	}

	/**
	 * Method run
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public void run() throws RetriableException, CriticalException {
		try {
			execute();
		} catch (RetriableException e) {
			while (!undoOpers.empty()) {
				try {
					((Command) (undoOpers.pop())).undo();
				} catch (RetriableException e2) {
					throw new CriticalException(
						"Undo failed leaving system in inconsistent state");
				}
			}
			throw e;
		}

	}
}
