/*
 * Created on 23/04/2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package il.ac.technion.cs.course236308.e3.med.command;

import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.gui.datamodels.IllegalStatesTableModel;
import il.ac.technion.cs.course236308.e3.med.model.Answer;
import il.ac.technion.cs.course236308.e3.med.model.CategoriesFactory;
import il.ac.technion.cs.course236308.e3.med.model.Category;
import il.ac.technion.cs.course236308.e3.med.model.LegalObject;
import il.ac.technion.cs.course236308.e3.med.model.Question;
import il.ac.technion.cs.course236308.e3.med.model.QuestionsFactory;
import il.ac.technion.cs.course236308.e3.med.model.Test;
import il.ac.technion.cs.course236308.e3.med.model.TestTaker;
import il.ac.technion.cs.course236308.e3.med.model.TestVersion;
import il.ac.technion.cs.course236308.e3.med.model.TestVersionsFactory;
import il.ac.technion.cs.course236308.e3.med.model.TestsFactory;
import il.ac.technion.cs.course236308.e3.med.model.Version;
import il.ac.technion.cs.course236308.e3.med.model.VersionsFactory;

import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.Stack;

/**
 * Type CommandDispatcher
 * @author sybenny
 * @version 1.0
 */

public class CommandDispatcher {
	private static CommandDispatcher instance = new CommandDispatcher();
	private static Stack undoStack = new Stack();
	private static Stack redoStack = new Stack();

	private Answer currentAnswer = null;
	private Question currentQuestion = null;
	private Test currentTest = null;
	private Version currentVersion = null;
	private Category currentCategory = null;
	private TestTaker currentTestTaker = null;
	private static Set illegalObjects = new LinkedHashSet();

	private CommandDispatcher(){}

	public void init() {
        try{
            currentCategory = CategoriesFactory.getInstance().getRootCategory();
        }catch(Throwable e){}
    }

	/**
	 * Method getInstance
	 * @return CommandDispatcher
	 */
	public static synchronized CommandDispatcher getInstance() {
		return instance;
	}

	//passes up exceptions from Command::execute()
	/**
	 * Method run
	 * @param c
	 * @throws MedException
	 */
	public synchronized void run(Command c) throws RetriableException,CriticalException {
		c.run();
		redoStack.clear();
		undoStack.push(c);
	}

	//throws RetriableException if stack is empty
	//passes up exceptions from Command::undo()
	/**
	 * Method undo
	 * @throws MedException
	 */
	public synchronized void undo() throws RetriableException,CriticalException {
		if (undoStack.isEmpty())
			throw (new RetriableException("Undo failed: undo stack is empty"));
		//At first don't remove the command from stack since it can fail
		Command c = (Command) undoStack.peek();
		c.undo();
		undoStack.pop();
		redoStack.push(c);
	}

	//throws RetriableException if stack is empty
	//passes up exceptions from Command::execute()
	/**
	 * Method redo
	 * @throws MedException
	 */
	public synchronized void redo() throws RetriableException,CriticalException {
		if (redoStack.isEmpty())
			throw new RetriableException("Redo failed: redo stack is empty");
		//At first don't remove the command from stack since it can fail
		Command c = (Command) redoStack.peek();
		c.run();
		redoStack.pop();
		undoStack.push(c);
	}

	/**
	 * Method getCurrentAnswer
	 * @return Answer
	 */
	public Answer getCurrentAnswer() {
		return currentAnswer;
	}

	/**
	 * Method getCurrentCategory
	 * @return Category
	 */
	public Category getCurrentCategory() {
		return currentCategory;
	}
	/**
	 * Method getCurrentTest
	 * @return Test
	 */
	public Test getCurrentTest() {
		return currentTest;
	}

	/**
	 * Method getCurrentQuestion
	 * @return Question
	 */
	public Question getCurrentQuestion() {
		return currentQuestion;
	}

	/**
	 * Method getCurrentVersion
	 * @return Version
	 */
	public Version getCurrentVersion() {
		return currentVersion;
	}

	/**
	 * Method getCurrentTestTaker
	 * @return TestTaker
	 */
	public TestTaker getCurrentTestTaker() {
		return currentTestTaker;
	}

	/**
	 * Method setCurrentAnswer
	 * @param answer
	 */
	public void setCurrentAnswer(Answer answer) {
		currentAnswer = answer;
	}

	/**
	 * Method setCurrentCategory
	 * @param category
	 */
	public void setCurrentCategory(Category category) {
		currentCategory = category;
	}

	/**
	 * Method setCurrentTest
	 * @param test
	 */
	public void setCurrentTest(Test test) {
		currentTest = test;
	}

	/**
	 * Method setCurrentQuestion
	 * @param question
	 */
	public void setCurrentQuestion(Question question) {
		currentQuestion = question;
	}

	/**
	 * Method setCurrentVersion
	 * @param version
	 */
	public void setCurrentVersion(Version version) {
		currentVersion = version;
	}
	
	/**
	 * Method setCurrentTestTaker
	 * @param testTaker
	 */
	public void setCurrentTestTaker(TestTaker testTaker) {
		currentTestTaker = testTaker;
	}
	
	public boolean undoEnabled(){
		return !undoStack.isEmpty();
	}
	
	public boolean redoEnabled(){
		return !redoStack.isEmpty();
	}

	public static Set getIllegalObjects(){
		return illegalObjects;
	}
	
	public void removeIllegalObject(LegalObject illegalObject){
		try{
			illegalObjects.remove(illegalObject);
			IllegalStatesTableModel.getInstance().fireTableDataChanged();
			IllegalStatesTableModel.getInstance().refreshTrees();
		}catch(Throwable e){}
	}
	
	public void checkAnswerLegality(Answer answer) throws RetriableException,CriticalException{
		answer.recalculateSelfLegality();
		checkLegality(answer);
		Version version = VersionsFactory.getInstance().getVersion(answer.getVersionId());
		checkVersionLegality(version);
	}
	
	public void checkVersionLegality(Version version) throws RetriableException, CriticalException{
		version.recalculateSelfLegality();
		checkLegality(version);
		Question quest = QuestionsFactory.getInstance().getQuestion(version.getQuestionId());
		checkQuestionLegality(quest);
		checkTestsLegality(version);
	}
	
	private void checkTestsLegality(Version ver) throws RetriableException, CriticalException{
		for(Iterator it=TestVersionsFactory.getInstance().getTestVersions(ver.getId()).iterator();it.hasNext();){
			checkTestLegality(TestsFactory.getInstance().getTest(((TestVersion)it.next()).getTestId()));
		}
	}
	
	public void checkTestLegality(Test test) throws CriticalException{
		test.recalculateSelfLegality();
		checkLegality(test);		
	}
	
	public void checkQuestionLegality(Question question) throws RetriableException, CriticalException{
		question.recalculateSelfLegality();
		checkLegality(question);
		Category cat = CategoriesFactory.getInstance().getCategory(question.getMainCategoryId());
		checkCategoryLegality(cat);
	}
	
	public void checkCategoryLegality(Category cat) throws RetriableException, CriticalException{
		cat.recalculateSelfLegality();
		if(cat.getParentID() != null){
			Category parentCat = CategoriesFactory.getInstance().getCategory(cat.getParentID().intValue());
			checkCategoryLegality(parentCat);
		}
	}
	
	private void checkLegality(LegalObject obj){
		try{
			try{
				obj.isSelfLegal();
				removeIllegalObject(obj);
				return;
			}catch(RetriableException e){}
			illegalObjects.add(obj);
			IllegalStatesTableModel.getInstance().fireTableDataChanged();
			IllegalStatesTableModel.getInstance().refreshTrees();
		}catch(Throwable e){}
	}
	
	public void refreshTrees(){
		IllegalStatesTableModel.getInstance().refreshTrees();
	}
}
