/*
 * Created on 19/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.answer;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.command.version.VersionSetUpdatedDate;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Answer;
import il.ac.technion.cs.course236308.e3.med.model.AnswersFactory;
import il.ac.technion.cs.course236308.e3.med.model.Version;

/**
 * @author  Alexander, Bennyamin, Svetlana and Valentin 
 * Created in SSDL lab - Technion Israel
 *
 */
public class AnswerClone extends Command {
	private Version version = null;
	private Answer answer = null;
	private Answer newAnswer = null;

	/**
	 * Method AnswerClone
	 * @param version
	 * @param answer
	 */
	public AnswerClone(Version version, Answer answer) {
		this.version = version;
		this.answer = answer;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws RetriableException, CriticalException {
		if (version == null) {
			throw new RetriableException
				("Can't clone answer - no version defined");
		}
		if (answer == null) {
			throw new RetriableException
				("Can't clone answer - no answer defined");
		}
		newAnswer = AnswersFactory.getInstance().cloneAnswer(version, answer);
		if (answer.getVersionId() != newAnswer.getVersionId() &&
				version.getCorrectAnswerId() != 0 &&
				AnswersFactory.getInstance().getAnswer(version.getCorrectAnswerId()).equals(answer)) {
			runAndPush(new AnswerMarkAsCorrect(newAnswer));
		}
		runAndPush(new VersionSetUpdatedDate(version));
		version.updateDB();
		
		MedReporter.report(
			"# new answer cloned, id = "
				+ answer.getId()
				+ ", for version "
				+ version.getId());
	}
	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws RetriableException, CriticalException {
		super.undo();
		AnswersFactory.getInstance().deleteAnswer(newAnswer);
		version.updateDB();
	}

	public String toString() {
		return "Clone Answer with ordinal: " + answer.getOrdinal();
	}

}
