/*
 * Created on 19/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.answer;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.command.version.VersionSetUpdatedDate;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Answer;
import il.ac.technion.cs.course236308.e3.med.model.AnswersFactory;
import il.ac.technion.cs.course236308.e3.med.model.Test;
import il.ac.technion.cs.course236308.e3.med.model.TestsFactory;
import il.ac.technion.cs.course236308.e3.med.model.Version;
import il.ac.technion.cs.course236308.e3.med.parameters.AnswerParams;

import java.util.Iterator;

/**
 * @author Alexander, Bennyamin, Svetlana and Valentin 
 * Created in SSDL lab - Technion Israel
 *
 */
public class AnswerCreate extends Command {
	private Version version;
	private Answer answer = null;

	/**
	 * Method AnswerCreate
	 */
	public AnswerCreate() {
		this.version = CommandDispatcher.getInstance().getCurrentVersion();
	}
	/**
	 * Method AnswerCreate
	 * @param version
	 */
	public AnswerCreate(Version version) {
		this.version = version;
	}
	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws RetriableException, CriticalException {
		if (version == null) {
			throw new RetriableException
				("Can't create answer - no current version defined");
		}
		// We check if there are already 10 answers to version
		// if so we don't continue
		if (version.getAnswers().size()
			>= AnswerParams.MAX_ANSWERS_PER_VERSION) {
			throw new RetriableException
				("Can't add answer - maximum answers were already defined");
		}
		if (version.isLocked()) { throw new 
			RetriableException("Can't create answer - locked version");
		}
		answer = AnswersFactory.getInstance().createAnswer(version);
		runAndPush(new AnswerSelect(answer));
		runAndPush(new VersionSetUpdatedDate(version));
		version.updateDB();
		CommandDispatcher.getInstance().checkVersionLegality(version);
        for(Iterator it=TestsFactory.getInstance().getTestsOfVersion(version).iterator();it.hasNext();){
            Test test = (Test)it.next();
            runAndPush(new AnswerAddToTest(test,answer,true)); 
        }
		MedReporter.report(
			"# new answer created, id = "
				+ answer.getId()
				+ ", for version "
				+ version.getId());
	}
	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws RetriableException, CriticalException {
		super.undo();
		AnswersFactory.getInstance().deleteAnswer(answer);
		version.updateDB();
		CommandDispatcher.getInstance().removeIllegalObject(answer);
		CommandDispatcher.getInstance().checkVersionLegality(version);
	}

	public String toString() {
		return "Create New Answer";
	}

}
