/*
 * Created on 19/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.answer;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.command.version.VersionSetUpdatedDate;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Answer;
import il.ac.technion.cs.course236308.e3.med.model.AnswersFactory;
import il.ac.technion.cs.course236308.e3.med.model.Test;
import il.ac.technion.cs.course236308.e3.med.model.TestAnswer;
import il.ac.technion.cs.course236308.e3.med.model.TestAnswersFactory;
import il.ac.technion.cs.course236308.e3.med.model.TestsFactory;
import il.ac.technion.cs.course236308.e3.med.model.Version;
import il.ac.technion.cs.course236308.e3.med.model.VersionsFactory;

import java.util.HashMap;
import java.util.Iterator;

/**
 * @author Alexander,Bennyamin, Svetlana and Valentin 
 * Created in SSDL lab - Technion Israel
 *
 */
public class AnswerDelete extends Command {

	private Answer answer;
	private Version version = null;
	private HashMap oldRefs = new HashMap();
	/**
	 * Method AnswerDelete
	 */
	public AnswerDelete() {
		this.answer = CommandDispatcher.getInstance().getCurrentAnswer();
	}
	/**
	 * Method AnswerDelete
	 * @param answer
	 */
	public AnswerDelete(Answer answer) {
		this.answer = answer;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws RetriableException, CriticalException {
		if (answer == null) { throw new 
			RetriableException("Can't execute - no current answer defined");
		}

		version =
			VersionsFactory.getInstance().getVersion(answer.getVersionId());
		if (version.isLocked()) { throw new 
				RetriableException("Can't delete answer - locked version");
		}
        for(Iterator it=TestsFactory.getInstance().getTestsOfVersion(version).iterator();it.hasNext();){
            Test test = (Test)it.next();
            TestAnswer testAnswer = TestAnswersFactory.getInstance().getTestAnswer(test.getId(),answer.getId());
            runAndPush(new AnswerRemoveFromTest(testAnswer)); 
        }
		for (Iterator iter = version.getAnswers().iterator();iter.hasNext();){
			Answer curAns = (Answer) iter.next();
			if (curAns.getOrdinal() > answer.getOrdinal()) {
				//if ordinal is larger then ours subtract 1 from its ordinal
				oldRefs.put(String.valueOf(curAns.getOrdinal()),
						String.valueOf(curAns.getOrdinal() - 1));
				curAns.setOrdinal((byte) (curAns.getOrdinal() - 1));
			}
		}
		if (version.getCorrectAnswerId() == answer.getId()) {
			runAndPush(new UnmarkCorrectAnswer());
		}
		if (answer.equals(CommandDispatcher.getInstance().getCurrentAnswer())) {
			runAndPush(new AnswerSelect(null));
		}
		oldRefs.put(String.valueOf(answer.getOrdinal()),"0");
		runAndPush(new AnswerUpdateRefsForAll(version,oldRefs));
		AnswersFactory.getInstance().deleteAnswer(answer);
		runAndPush(new VersionSetUpdatedDate(version));
		version.updateDB();
		CommandDispatcher.getInstance().removeIllegalObject(answer);
		CommandDispatcher.getInstance().checkVersionLegality(version);
		MedReporter.report(
			"# answer deleted, id = "
				+ answer.getId()
				+ ", for version "
				+ version.getId());
	}

	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws CriticalException, RetriableException {
		for (Iterator iter = version.getAnswers().iterator();
			iter.hasNext();
			) {
			Answer curAns = (Answer) iter.next();
			if (curAns.getOrdinal() >= answer.getOrdinal()) {
				//if ordinal is larger then ours add 1 from its ordinal
				curAns.setOrdinal((byte) (curAns.getOrdinal() + 1));
			}
		}
		AnswersFactory.getInstance().storeAnswer(version, answer);
		super.undo();
		version.updateDB();
		CommandDispatcher.getInstance().checkAnswerLegality(answer);
	}

	public String toString() {
		return "Delete Answer: " + answer.getText();
	}
	
	class UnmarkCorrectAnswer extends Command {
		/**
		 * Method execute
		 * @see Command#execute()
		 */
		protected void execute() throws CriticalException {
			version.setCorrectAnswerId(0);
			version.updateDB();
		}
		/**
		 * Method undo
		 * @see Command#undo()
		 */
		public void undo() throws CriticalException {
			version.setCorrectAnswerId(answer.getId());
			version.updateDB();
		}
	}


}
