
/*
 * Created on 20/04/2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package il.ac.technion.cs.course236308.e3.med.command.answer;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.command.version.VersionSetUpdatedDate;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Answer;
import il.ac.technion.cs.course236308.e3.med.model.Version;
import il.ac.technion.cs.course236308.e3.med.model.VersionsFactory;

/**
 * @author sybenny
 *
 * 
 */
public class AnswerMarkAsCorrect extends Command {

	private Answer newAnswer;
	private Version version;
	private int oldCorrectAnswer;

	/**
	 * Method AnswerMarkAsCorrect
	 * @param newAnswer
	 */
	public AnswerMarkAsCorrect(Answer newAnswer) {
		this.newAnswer = newAnswer;
	}

	/**
	 * Method AnswerMarkAsCorrect
	 */
	public AnswerMarkAsCorrect() {
		this.newAnswer = CommandDispatcher.getInstance().getCurrentAnswer();
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws RetriableException, CriticalException {
		if (newAnswer == null) { throw new 
				RetriableException("Can't execute no current answer defined");
		}
		version =
			VersionsFactory.getInstance().getVersion(newAnswer.getVersionId());
		if (version.isLocked()) { throw new 
			RetriableException("Can't update answer - locked version");
		}
		oldCorrectAnswer = version.getCorrectAnswerId();
		version.setCorrectAnswerId(newAnswer.getId());
		runAndPush(new VersionSetUpdatedDate(version));
		version.updateDB();	
		CommandDispatcher.getInstance().checkVersionLegality(version);
		MedReporter.report(
			"# answer marked as correct, id = "
				+ newAnswer.getId()
				+ ", for version "
				+ version.getId());
	}

	/**
	 * Method undo
	 * @throws CriticalException
	 * @throws RetriableException
	 * @see Command#undo()
	 */
	public void undo() throws RetriableException, CriticalException {
		super.undo();
		version.setCorrectAnswerId(oldCorrectAnswer);
		version.updateDB();
		CommandDispatcher.getInstance().checkVersionLegality(version);
	}

	public String toString() {
		return "Mark Answer " + newAnswer.getText() + " as Correct";
	}

}
