/*
 * Created on 20/04/2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package il.ac.technion.cs.course236308.e3.med.command.answer;

import java.util.HashMap;
import java.util.Iterator;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.command.version.VersionSetUpdatedDate;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Answer;
import il.ac.technion.cs.course236308.e3.med.model.Version;
import il.ac.technion.cs.course236308.e3.med.model.VersionsFactory;
/**
 * @author sybenny
 *
 */
public class AnswerSetOrdinal extends Command {
	private Answer answer;
	private Version version;
	private byte newOrdinal;
	private byte oldOrdinal;
	private HashMap oldRefs = new HashMap();

	/**
	 * Method AnswerSetOrdinal
	 * @param answer
	 * @param newOrdinal
	 */
	public AnswerSetOrdinal(Answer answer, byte newOrdinal) {
		this.answer = answer;
		this.newOrdinal = newOrdinal;
	}

	/**
	 * Method AnswerSetOrdinal
	 * @param newOrdinal
	 */
	public AnswerSetOrdinal(byte newOrdinal) {
		this.answer = CommandDispatcher.getInstance().getCurrentAnswer();
		this.newOrdinal = newOrdinal;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws RetriableException, CriticalException {
		if (answer == null) { throw new 
			RetriableException("Can't execute - no current answer defined");
		}
		oldOrdinal = answer.getOrdinal();
		version = VersionsFactory.getInstance().getVersion(answer.getVersionId());
		if (version.isLocked()) {throw new 
			RetriableException("Can't update answer - locked version");
		}
		if (newOrdinal < 1 || newOrdinal > version.getAnswers().size()) {
			throw new RetriableException(
				"New ordinal must be between 1 and "
					+ version.getAnswers().size());
		}

		if (newOrdinal < oldOrdinal) {
			moveUp(oldOrdinal, newOrdinal, oldRefs);
		}
		if (newOrdinal > oldOrdinal) {
			moveDown(oldOrdinal, newOrdinal, oldRefs);
		}
		answer.setOrdinal(newOrdinal);
		oldRefs.put(String.valueOf(oldOrdinal), String.valueOf(newOrdinal));
		runAndPush(new AnswerUpdateRefsForAll(version,oldRefs));
		runAndPush(new VersionSetUpdatedDate(version));
		version.updateDB();
		MedReporter.report(
			"# answer ordinal changed, id = "
				+ answer.getId()
				+ ", new ordinal="
				+ newOrdinal);
	}

	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws RetriableException, CriticalException {
		HashMap dummyHash = new HashMap();
		if (newOrdinal > oldOrdinal) {
			moveUp(newOrdinal, oldOrdinal, dummyHash);
		}
		if (newOrdinal < oldOrdinal) {
			moveDown(newOrdinal, oldOrdinal, dummyHash);
		}
		answer.setOrdinal(oldOrdinal);
		super.undo();
		version.updateDB();
	}

	private void moveDown(byte newPlace, byte oldPlace, HashMap refs) {
		for (Iterator it = version.getAnswers().iterator(); it.hasNext();) {
			Answer ans = (Answer) it.next();
			if (ans.getOrdinal() > newPlace && ans.getOrdinal() <= oldPlace) {
				refs.put(String.valueOf(ans.getOrdinal()),String.valueOf(ans.getOrdinal() - 1));
				ans.setOrdinal((byte) (ans.getOrdinal() - 1));
			}
		}
	}

	private void moveUp(byte newPlace, byte oldPlace, HashMap refs) {
		for (Iterator it = version.getAnswers().iterator(); it.hasNext();) {
			Answer ans = (Answer) it.next();
			if (ans.getOrdinal() >= oldPlace && ans.getOrdinal() < newPlace) {
				refs.put(String.valueOf(ans.getOrdinal()),String.valueOf(ans.getOrdinal() + 1));
				ans.setOrdinal((byte) (ans.getOrdinal() + 1));
			}
		}
	}

	public String toString(){
		return "Set Ordinal of answer " + answer.getOrdinal();
	}


}
