/*
 * Created on 21/04/2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package il.ac.technion.cs.course236308.e3.med.command.category;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.command.question.QuestionDelete;
import il.ac.technion.cs.course236308.e3.med.command.question.QuestionRemoveSecondaryCategory;
import il.ac.technion.cs.course236308.e3.med.command.question.QuestionSetMainCategory;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.CategoriesFactory;
import il.ac.technion.cs.course236308.e3.med.model.Category;
import il.ac.technion.cs.course236308.e3.med.model.Question;
import il.ac.technion.cs.course236308.e3.med.model.QuestionsFactory;
import il.ac.technion.cs.course236308.e3.med.parameters.CategoryParams;

/**
 * @author sybenny
 *
 */
public class CategoryDelete extends Command {

	private Category parentCategory = null;
	private Category category = null;
	private int handling;

	/**
	 * Method CategoryDelete
	 * @param category
	 * @param handling
	 */
	public CategoryDelete(Category category,int handling) {
		this.category = category;
		this.handling = handling;
	}
	
	/**
	 * Method CategoryDelete
	 * @param handling
	 */
	public CategoryDelete(int handling) {
		this.category = CommandDispatcher.getInstance().getCurrentCategory();
		this.handling = handling;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws RetriableException,CriticalException {
		if (category==null){ throw new 
			RetriableException("Can't execute - no current category defined");
		}				
		if(category.equals(CategoriesFactory.getInstance().getRootCategory())){
			throw new RetriableException(
			"Can't execute - root category can not be deleted");
		}
		parentCategory = 
			CategoriesFactory.getInstance().
				getCategory(category.getParentID().intValue());
		if(category.hasQuestInDescendant() && handling==CategoryParams.CANCEL_MODE){
			throw new RetriableException( "Can't execute - " +
				"category has question(s) in it's subcategory(ies)");
		}
		while(category.getLinksToQuestionIds().size() > 0){
			int questId = ((Integer)category.getLinksToQuestionIds().get(0)).intValue();
			Question quest = QuestionsFactory.getInstance().getQuestion(questId);
			runAndPush(new QuestionRemoveSecondaryCategory(quest,category));
		}

		if(handling==CategoryParams.DELETE_QUESTIONS_MODE){
			runAndPush(new CategoryDeleteWithRemove(parentCategory,category));
		}else{
			runAndPush(new CategoryDeleteWithMove(
					parentCategory,parentCategory,category));
		}		
		runAndPush(new CategorySelect(
				CategoriesFactory.getInstance().getRootCategory()));
		runAndPush(new CategoryDecreaseOrdinals(parentCategory,
											category.getOrdinal()));
		MedReporter.report("# category deleted, id = "+category.getId()
									+ ", parent category id = "
									+ parentCategory.getId());
	}
	
	public String toString() {
		return "Delete category";
	}

	
	class CategoryDeleteWithRemove extends Command{
		private Category cat = null;
		private Category parentCat = null;
		/**
		 * Method CategoryDeleteWithRemove
		 * @param parentCategory
		 * @param category
		 */
		CategoryDeleteWithRemove(Category parentCategory,Category category){
			this.cat = category;
			this.parentCat = parentCategory;
		}
		/**
		 * Method execute
		 * @see Command#execute()
		 */
		protected void execute() throws RetriableException,CriticalException {
			while(cat.getQuestions().size() > 0){
				Question quest = (Question)cat.getQuestions().get(0);
				runAndPush(new QuestionDelete(quest));
			}
			while(cat.getCategories().size() > 0){
				Category son = (Category)cat.getCategories().get(0);
				runAndPush(new CategoryDeleteWithRemove(cat,son));
			}
			CategoriesFactory.getInstance().deleteCategory(cat);
		}
		
		/**
		 * Method undo
		 * @see Command#undo()
		 */
		public void undo() throws RetriableException, CriticalException{
			CategoriesFactory.getInstance().
				storeCategory(parentCat,cat);
			super.undo();
		}
	}
	
	static class CategoryDeleteWithMove extends Command{
		private Category parentCat = null;
		private Category cat = null;
		private Category moveTo = null;
		/**
		 * Method CategoryDeleteWithMove
		 * @param moveTo
		 * @param parentCategory
		 * @param category
		 */
		 CategoryDeleteWithMove(Category moveTo,Category parentCategory,
		 						Category category){
			this.parentCat = parentCategory;
			this.cat = category;
			this.moveTo = moveTo;
		}
		/**
		 * Method execute
		 * @see Command#execute()
		 */
		protected void execute() throws RetriableException,CriticalException {
			while(cat.getQuestions().size() > 0){
				Question quest = (Question)cat.getQuestions().get(0);
				runAndPush(new QuestionMove(moveTo,quest));
			}
			while(cat.getCategories().size() > 0){
				Category son = (Category)cat.getCategories().get(0);
				runAndPush(new CategoryDeleteWithMove(moveTo,cat,son));
			}
			CategoriesFactory.getInstance().deleteCategory(cat);
		}
		
		/**
		 * Method undo
		 * @see Command#undo()
		 */
		public void undo() throws RetriableException, CriticalException{
			CategoriesFactory.getInstance().
				storeCategory(parentCat,cat);
			super.undo();
		}
	}
	
	static class QuestionMove extends Command{
		private Category moveTo;
		private Question question;
		/**
		 * Method QuestionMove
		 * @param moveTo
		 * @param question
		 */
		QuestionMove(Category moveTo,Question question){
			this.moveTo = moveTo;
			this.question = question;
		}
		/**
		 * Method execute
		 * @see Command#execute()
		 */
		protected void execute() throws RetriableException, CriticalException{
			runAndPush(new QuestionSetMainCategory(question,moveTo));
		}
	}
}
