/*
 * QuestionSetMainCategory.java
 * 
 *  Created on 23/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.category;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.CategoriesFactory;
import il.ac.technion.cs.course236308.e3.med.model.Category;

/**
 * @author svekant
 *
 */
public class CategorySetName extends Command{

	private Category category;
	private String oldName;
	private String newName;
	
	/**
	 * Method CategorySetName
	 * @param newName
	 */
	public CategorySetName(String newName){
		this.newName = newName;
		this.category = CommandDispatcher.getInstance().getCurrentCategory();
	}

	/**
	 * Method CategorySetName
	 * @param category
	 * @param newName
	 */
	public CategorySetName(Category category,String newName){
		this.newName = newName;
		this.category = category;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws RetriableException,CriticalException{
		if(category == null){ throw new 
			RetriableException("Can't execute - no current category defined");
		}
		if(category.equals(CategoriesFactory.getInstance().getRootCategory())){
			throw new 
				RetriableException("Can't execute -" +
					" root category can not be changed");
		}
		if(newName == null || newName.equals("")){
			throw new RetriableException("New name can not be empty");
		}
		Category parent = CategoriesFactory.getInstance().
			getCategory(category.getParentID().intValue());
		if(parent.hasChildWithName(newName)){
			throw new RetriableException("Rename failed - Category \"" 
				+ parent.getName() + "\" already contains category \""+ newName +"\"");
		}
		oldName = category.getName();
		category.setName(newName);		
		category.updateDB();				
		MedReporter.report("# category updated, id = "+category.getId()
		+ ", new Name = "+ newName);
	}
	
	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws CriticalException{
		category.setName(oldName);		
		category.updateDB();
	}
	
	public String toString() {
		return "Set Name for category";
	}

}
