/*
 * Created on 29/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.category;

import java.util.Iterator;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.CategoriesFactory;
import il.ac.technion.cs.course236308.e3.med.model.Category;

/**
 * @author Alexander, Bennyamin, Svetlana and Valentin 
 * Created in SSDL lab - Technion Israel
 *
 */
public class CategorySetOrdinal extends Command {

	private int newOrdinal;
	private int oldOrdinal;
	private Category category;
	private Category parent;

	/**
	 * Method CategorySetOrdinal
	 * @param category
	 * @param newOrdinal
	 */
	public CategorySetOrdinal(Category category, int newOrdinal) {
		this.category = category;
		this.newOrdinal = newOrdinal;
	}
	/**
	 * Method CategorySetOrdinal
	 * @param newOrdinal
	 */
	public CategorySetOrdinal(byte newOrdinal) {
		this.category = CommandDispatcher.getInstance().getCurrentCategory();
		this.newOrdinal = newOrdinal;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws RetriableException, CriticalException {
		if (category == null) { throw new 
			RetriableException("Can't execute - no current category defined");
		}
		oldOrdinal = category.getOrdinal();
		if(category.equals(CategoriesFactory.getInstance().getRootCategory())){
			throw new RetriableException("Can't update ordinal of root category");
		}
		parent =
			CategoriesFactory.getInstance().getCategory(
				category.getParentID().intValue());
		if (parent==null){ throw new 
			RetriableException("Can't execute - there is no parent category");
		}
		if (newOrdinal < 1 || newOrdinal > parent.getCategories().size()) {
			throw new 
				RetriableException("New ordinal must be between 1 and "
									+ parent.getCategories().size());
		}
		if(newOrdinal < oldOrdinal ){
			moveUp(oldOrdinal,newOrdinal);
		}
		if(newOrdinal > oldOrdinal ){
			moveDown(oldOrdinal,newOrdinal);
		}
		category.setOrdinal(newOrdinal);
		category.updateDB();
		MedReporter.report("# ordinal was set to "+ newOrdinal
			+ " for category "+ category.getId());
	}
	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws CriticalException {
		if(newOrdinal > oldOrdinal ){
			moveUp(newOrdinal,oldOrdinal);
		}
		if(newOrdinal < oldOrdinal ){
			moveDown(newOrdinal,oldOrdinal);
		}
		category.setOrdinal(oldOrdinal);
		category.updateDB();
	}

	private void moveDown(int newPlace, int oldPlace) 
		throws CriticalException {
		for (Iterator it = parent.getCategories().iterator(); it.hasNext();) {
			Category cat = (Category) it.next();
			if (cat.getOrdinal() > newPlace 
				&& cat.getOrdinal() <= oldPlace) {
				cat.setOrdinal((cat.getOrdinal() - 1));
				cat.updateDB();
			}
		}
	}

	private void moveUp(int newPlace, int oldPlace) 
		throws CriticalException {
		for (Iterator it = parent.getCategories().iterator(); it.hasNext();) {
			Category cat = (Category) it.next();
			if (cat.getOrdinal() >= oldPlace && cat.getOrdinal() < newPlace) {
				cat.setOrdinal((cat.getOrdinal() + 1));
				cat.updateDB();
			}
		}
	}
	
	public String toString() {
		return "Set Ordinal for category";
	}

}
