/*
 * QuestionSetMainCategory.java
 * 
 *  Created on 23/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.category;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.CategoriesFactory;
import il.ac.technion.cs.course236308.e3.med.model.Category;
import il.ac.technion.cs.course236308.e3.med.stats.StatsUpdateDB;

/**
 * @author svekant
 *
 */
public class CategorySetParent extends Command{

	private int newParentId;
	private Category category;
	private Category oldParent;
	private Category newParent;
	private int oldOrdinal;
	private String oldName;
	
	/**
	 * Method CategorySetParent
	 * @param newParentId
	 */
	public CategorySetParent(int newParentId){
		this.newParentId = newParentId;
		this.category = CommandDispatcher.getInstance().getCurrentCategory();
	}

	/**
	 * Method CategorySetParent
	 * @param category
	 * @param newParentId
	 */
	public CategorySetParent(Category category,int newParentId){
		this.newParentId = newParentId;
		this.category = category;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws RetriableException,CriticalException{
		if(category == null){ throw new 
			RetriableException("Can't execute - no current category defined");
		}
		if(category.equals(CategoriesFactory.getInstance().getRootCategory())){
			throw new 
				RetriableException("Can't execute - " +
					"root category can't be moved");
		}
		oldParent = CategoriesFactory.getInstance().
					getCategory(category.getParentID().intValue());
		newParent = CategoriesFactory.getInstance().getCategory(newParentId);
		if(newParent.isDescendant(category)){
			throw new RetriableException("Can not move Category to it's descendant");
		}
		if(oldParent.equals(newParent)){
			throw new RetriableException("Can not move to the same Category");
		}
		
		category.setParentID(new Integer(newParent.getId()));
		oldOrdinal = category.getOrdinal();
		oldName = category.getName();
		category.setOrdinal(newParent.getCategories().size()+1);
		category.setName(newParent.newSubCatUniqueName(category.getName()));		
		newParent.getCategories().add(category);
		oldParent.getCategories().remove(category);
		category.updateDB();	
		runAndPush(new CategoryDecreaseOrdinals(oldParent,oldOrdinal));
		CommandDispatcher.getInstance().checkCategoryLegality(category);
		CommandDispatcher.getInstance().checkCategoryLegality(oldParent);
		CommandDispatcher.getInstance().refreshTrees();
		StatsUpdateDB.getInstance().updateCategoryStatistics(oldParent);
		StatsUpdateDB.getInstance().updateCategoryStatistics(category);

		MedReporter.report("# category updated, id = "+category.getId()
			+ ", new parent id= = "+ newParentId);			
	}
	
	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws RetriableException, CriticalException{
		super.undo();
		category.setParentID(new Integer(oldParent.getId()));
		category.setName(oldName);
		category.setOrdinal(oldOrdinal);	
		newParent.getCategories().remove(category);
		oldParent.getCategories().add(category);
		category.updateDB();
		CommandDispatcher.getInstance().checkCategoryLegality(category);
		CommandDispatcher.getInstance().checkCategoryLegality(newParent);
		CommandDispatcher.getInstance().refreshTrees();
		StatsUpdateDB.getInstance().updateCategoryStatistics(newParent);
		StatsUpdateDB.getInstance().updateCategoryStatistics(category);

	}
	
	public String toString() {
		return "Set Parent for category";
	}

}
