/*
 * Created on 04/05/2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package il.ac.technion.cs.course236308.e3.med.command.grading;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.command.test.TestSelect;
import il.ac.technion.cs.course236308.e3.med.control.controllers.ReplyPanelController;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Test;
import il.ac.technion.cs.course236308.e3.med.model.TestTaker;
import il.ac.technion.cs.course236308.e3.med.model.TestTakersFactory;
import il.ac.technion.cs.course236308.e3.med.model.TestsFactory;

/**
 * @author sybenny
 *
 * 
 */

public class TestImportAnswers extends Command {

	private String fileName = ".ans";
	private static final int ARGUMENT_MIN_NUM = 5;
	
	private List failedActs = new LinkedList();
	private List successAct = new LinkedList(); 
	private Set updatedTests = new HashSet();
	private Test selectedTest = null;
	/**
	 * Method TestImportAnswers
	 * @param fileName
	 */
	public TestImportAnswers(String fileName){
		if(fileName != null)
			this.fileName = fileName;
	}
	
	public TestImportAnswers(Test currTest,String fileName){
			if(fileName != null)
				this.fileName = fileName;
			if(currTest != null){
				selectedTest = currTest;
			}
	}
	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws RetriableException, CriticalException {
		// input file has the following format :
		// test id , notebookNum , student id , student name , 
		// "answersStr" [,"CheckSum"]
		BufferedReader in;
		try{
			in = new BufferedReader(new FileReader(fileName));
		}catch(IOException e){			
			throw new 
				RetriableException("Can't execute : file " + fileName + 
									"isn't found");
		}
		
		String str;
		TestsFactory tf =  TestsFactory.getInstance();
		int index = 1;
		Test old_test = CommandDispatcher.getInstance().getCurrentTest();
		try{
			while((str = in.readLine()) != null){
				// parse the line
				str = str.trim();
				if(str.equals("")){
					index ++;
					continue;
				}
				
				String[] arguments = str.split(",");
				
				if(arguments.length < ARGUMENT_MIN_NUM ){	
					failedActs.add(new String("Line " + index + " : Parsing"));
					failedActs.add(new String("The line format must be test id, notebookNum, student id, student name, \"answersStr\" [,\"CheckSum\"]"));
					index ++;
					continue;
					
				}
				
				int testId = Integer.parseInt(arguments[0].trim());
				if(selectedTest != null && selectedTest.getId() != testId){
					failedActs.add(new String("Line " + index + " : Parsing"));
					failedActs.add(new String("Test id " + testId + " is different from current selected test with id = " + selectedTest.getId()));
					index ++;
					continue;
				}
				
				Test test;
				try{
					test = tf.getTest(testId);	
				}catch(RetriableException e){
					failedActs.add(new String("Line " + index + " : Parsing"));
					failedActs.add(new String(e.getMessage()));
					index ++;
					continue;
				}
						 
				int notebookNum = Integer.parseInt(arguments[1].trim());				
				String studentId = arguments[2].trim();	
				
				if(!ReplyPanelController.isLegal(studentId)){
					failedActs.add(new String("Line " + index + " : Parsing"));
					failedActs.add(new String("Illegal student number " + studentId));
					index ++;
					continue;
				}
				
				//				add answers
				String studentName = arguments[3].trim();	
				TestSelect ts = new TestSelect(test.getId());
				runAndPush(ts);
				
				TestSelectStudent tss = 
					new TestSelectStudent(studentId, studentName, notebookNum);
				runAndPush(tss);
					
				char[] answersArray = 
					(arguments[4].trim().substring(1,arguments[4].trim().
												length()-1)).toCharArray();
				int[] checkSumArray = null;
				if(arguments.length > ARGUMENT_MIN_NUM){
					checkSumArray = new int[arguments.length-ARGUMENT_MIN_NUM];
					for(int i = ARGUMENT_MIN_NUM ;i < arguments.length;i++ ){
						checkSumArray[i-ARGUMENT_MIN_NUM] = 
							Integer.parseInt(arguments[i].replaceAll("\"","").
																		trim());
					}
				}
				TestTaker testTaker = TestTakersFactory.getInstance().getTestTaker(studentId,test.getId());	
				testTaker.setNotebookNumber(notebookNum);
				TestSetStudentAnswers tssa = new TestSetStudentAnswers(test,testTaker,answersArray,checkSumArray);
				try{
					runAndPush(tssa);
				}catch(RetriableException e){					
					MedReporter.report("# answers of student " + studentId +
						 " , exam id = " + test.getId() + " were not set : " + 
						 									e.getMessage());
						 									
					failedActs.add(new String("Line " + index + " : Add answers of student " + studentId));
					failedActs.add(new String(e.getMessage()));
					tss.undo();
					ts.undo();
					index++;
					continue;
				}
				successAct.add("Add answers of student " + studentId + ", exam " + test.getId());
				updatedTests.add(new Integer(test.getId()));
				index++;
			}
			
			in.close();
			if (old_test!=null){
				TestSelect ts = new TestSelect(old_test.getId());
				ts.run();
			}
			
		}catch(IOException e){
			try{
				in.close();
			}catch(IOException e1){
				undo();
				throw new RetriableException(e1.getMessage());
			}			
			undo();
			throw new RetriableException(e.getMessage());
		}
		
	}
	
	public List getSuccessActionsList(){
		return successAct;
	
	}
	
	public List getFailedActionsList(){
		return failedActs;
	
	}
	
	public Set getUpdatedTests(){
		return updatedTests;
	}

}
