/*
 * Created on 04/05/2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package il.ac.technion.cs.course236308.e3.med.command.grading;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Test;
import il.ac.technion.cs.course236308.e3.med.model.TestTaker;
import il.ac.technion.cs.course236308.e3.med.model.TestTakersFactory;

/**
 * @author svekant
 *
 * 
 */
public class TestSelectStudent extends Command {
	
	private String id;
	private String name;
	private int notebookNum;
	private Test test = null;	

	/**
	 * Method TestSelectStudent
	 * @param id
	 * @param name
	 * @param notebookNum
	 */
	public TestSelectStudent(String id, String name, int notebookNum){
		this.id = id;
		this.name = name;
		this.notebookNum = notebookNum;
		this.test = CommandDispatcher.getInstance().getCurrentTest();
	}
	
	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws RetriableException, CriticalException {	
		// check current test
		if(test == null){
			throw new 
				RetriableException("Can't execute - no current test defined");
		}
			 
		TestTaker newTestTaker = new TestTaker();
		
		try{
			newTestTaker = TestTakersFactory.getInstance().
									getTestTaker(id,test.getId());
		}catch(RetriableException e){
			// if the student isn't existing - add him to DB
			newTestTaker.setId(id);
			newTestTaker.setName(name);
			newTestTaker.setNotebookNumber(notebookNum);
			newTestTaker.setTestId(test.getId());
			
			Command c1 = new TestStoreStudent(newTestTaker);
			runAndPush(c1);
			
			Command c2 = new SetCurrentTestTaker(newTestTaker);
			runAndPush(c2);
			return;
		}
	
		//the student is existing , update name and notebookNum;
		Command c3 = new UpdateTestTakerDetails(newTestTaker,name,notebookNum);
		runAndPush(c3);
		
		Command c4 = new SetCurrentTestTaker(newTestTaker);
		runAndPush(c4);		
	}	

	static class TestStoreStudent extends Command {
		private TestTaker newTestTaker;
		
		/**
		 * Method TestStoreStudent
		 * @param newTestTaker
		 */
		TestStoreStudent(TestTaker newTestTaker){
			this.newTestTaker = newTestTaker;
		}
		/**
		 * Method execute
		 * @see Command#execute()
		 */
		protected void execute() throws RetriableException, CriticalException{
			TestTakersFactory.getInstance().createTestTaker(newTestTaker);
		}
		/**
		 * Method undo
		 * @see Command#undo()
		 */
		public void undo() throws RetriableException, CriticalException{
			TestTakersFactory.getInstance().deleteTestTaker(newTestTaker);
		}
	}
	
	static class SetCurrentTestTaker extends Command {
			private TestTaker newTestTaker;
			private TestTaker oldTestTaker = null;
		
			/**
			 * Method SetCurrentTestTaker
			 * @param newTestTaker
			 */
			 SetCurrentTestTaker(TestTaker newTestTaker){
				this.newTestTaker = newTestTaker;
				oldTestTaker = CommandDispatcher.getInstance().
											getCurrentTestTaker();
			}
			/**
			 * Method execute
			 * @see Command#execute()
			 */
			protected void execute() {
				CommandDispatcher.getInstance().
					setCurrentTestTaker(newTestTaker);
			}
			/**
			 * Method undo
			 * @see Command#undo()
			 */
			public void undo() {
				CommandDispatcher.getInstance().
					setCurrentTestTaker(oldTestTaker);
			}
	}
		
	static class UpdateTestTakerDetails extends Command {
		private TestTaker taker;
		private String newName,oldName;
		private int newNotebookNum ,oldNotebookNum;
		
		/**
		 * Method UpdateTestTakerDetails
		 * @param taker
		 * @param name
		 * @param notebookNum
		 */
		UpdateTestTakerDetails(TestTaker taker ,String name,int notebookNum){
			this.taker = taker;
			this.newName = name;
			this.newNotebookNum = notebookNum;
			
			oldName = taker.getName();
			oldNotebookNum = taker.getNotebookNumber();
		}
		
		/**
		 * Method execute
		 * @see Command#execute()
		 */
		protected void execute() throws CriticalException{
			taker.setName(newName);
			taker.setNotebookNumber(newNotebookNum);
			taker.updateDB();
		}
		
		/**
		 * Method undo
		 * @see Command#undo()
		 */
		public void undo() throws CriticalException{
			taker.setName(oldName);
			taker.setNotebookNumber(oldNotebookNum);
			taker.updateDB();
		}
	}
}