/*
 * QuestionAddSecondaryCategory.java
 * 
 *  Created on 23/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.question;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.CategoriesFactory;
import il.ac.technion.cs.course236308.e3.med.model.Category;
import il.ac.technion.cs.course236308.e3.med.model.Question;

/**
 * @author svekant
 *
 */
public class QuestionAddSecondaryCategory extends Command
{

	private Question question;
	private Category category;
	
	/**
	 * Method QuestionAddSecondaryCategory
	 */
	public QuestionAddSecondaryCategory(){
		this.question = CommandDispatcher.getInstance().getCurrentQuestion();
		this.category = CommandDispatcher.getInstance().getCurrentCategory();
	}
	
	/**
	 * Method QuestionAddSecondaryCategory
	 * @param category
	 * @param question
	 */
	public QuestionAddSecondaryCategory(Category category,Question question){
		this.question = question;
		this.category = category;
	}
	 
	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws CriticalException, RetriableException{
		if(question == null){ throw new 
			RetriableException("Can't execute - no current question defined");
		}
		if(category == null){ throw new 
			RetriableException("Can't execute - no current category defined");
		}
		if(category.getLinksToQuestionIds().
			contains(new Integer(question.getId()))){
			throw new RetriableException("Category " + category.getFullName() 
				+ " has link to the question " + question.getId());
		}
		CategoriesFactory cf = CategoriesFactory.getInstance();
		Category mainCategory = cf.getCategory(question.getMainCategoryId());
		while(true){
			if(mainCategory.equals(category)){ throw new 
				RetriableException("Category " + mainCategory.getFullName() 
				+ " is ancestor of main category of the question " 
				+ question.getId());		
			}
			try{
				mainCategory = cf.getCategory(mainCategory.
												getParentID().intValue());
			}catch(NullPointerException e){
				break;			
			}
		}
		category.addLinkToQuestion(question);
		MedReporter.report("# added secondary category " 
			+ category.getId() + " for question " + question.getId());					
	}
	
	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws  CriticalException{
		category.removeLinkToQuestion(question);
	}

	public String toString() {
			return "Add to question to secondary category " + category.getName();
	}
}
