/*
 * Created on 21/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.question;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.command.answer.AnswerAddToTest;
import il.ac.technion.cs.course236308.e3.med.command.test.TestSetUpdatedDate;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Answer;
import il.ac.technion.cs.course236308.e3.med.model.Question;
import il.ac.technion.cs.course236308.e3.med.model.Test;
import il.ac.technion.cs.course236308.e3.med.model.TestVersion;
import il.ac.technion.cs.course236308.e3.med.model.TestVersionsFactory;
import il.ac.technion.cs.course236308.e3.med.model.Version;
import il.ac.technion.cs.course236308.e3.med.model.VersionsFactory;
import il.ac.technion.cs.course236308.e3.med.stats.StatisticData;

import java.util.Iterator;

/**
 * @author Alexander, Bennyamin, Svetlana and Valentin
 * Created in SSDL lab - Technion Israel
 *
 */
public class QuestionAddToTest extends Command{
	private Test test;
	private Question question;
	private Version activeVersion;
	private TestVersion testVersion;

	/**
	 * Method QuestionAddToTest
	 */
	public QuestionAddToTest(){
		this.question = CommandDispatcher.getInstance().getCurrentQuestion();
		this.test = CommandDispatcher.getInstance().getCurrentTest();
	}

	/**
	 * Method QuestionAddToTest
	 * @param test
	 * @param question
	 */
	public QuestionAddToTest(Test test,Question question){
		this.question = question;
		this.test = test;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws CriticalException, RetriableException{
		if (test == null){ throw new 
			RetriableException("Can't execute - no current test defined");
		}
		if (question == null){
			throw new RetriableException("Can't execute - " +
				"no current question defined");
		}
		if(question.getActiveVersionId() == null){ throw new 
			RetriableException("Can't execute - " +
				"no active Version defined for current question");
		}
		if(question.isCancelled()){ throw new 
			RetriableException("Can't execute - " +
				"cancelled question can not be added to test");
		}
		try{
			activeVersion = VersionsFactory.getInstance().
					getVersion(question.getActiveVersionId().intValue());
		}catch(RetriableException e){ throw new 
			RetriableException("Referential integrity violation - in question " 
					+ question.getId() + " active Version is" 
					+ question.getActiveVersionId() + 
					" without such record in Version table");
		}
		if(activeVersion.getQuestionId() != question.getId()){
			throw new RetriableException("Referential integrity violation" +
				" - in question " + question.getId() + " active Version is" 
				+ question.getActiveVersionId() + 
				" however,this version does not belong to the question");
		}
		if(!test.getStatus().equals("U")){
			throw new RetriableException("Only Exams which are" +
				" \"Under Constraction\" can be edited");
		}
		if (test.containsQuestion(question)){
			throw new RetriableException("Question is already in exam");
		}
		testVersion = TestVersionsFactory.getInstance().
					createTestVersion(test,activeVersion);
		for(Iterator it=activeVersion.getAnswers().iterator();it.hasNext();){
			Answer ans = (Answer)it.next();
			runAndPush(new AnswerAddToTest(test,ans,false));
		}
		runAndPush(new TestSetUpdatedDate(test));
		test.updateDB();
		StatisticData.getInstance().updateQuestionsStatistics(test.getId());
		CommandDispatcher.getInstance().checkTestLegality(test);
		MedReporter.report("# test version created, test id = " 
				+testVersion.getTestId() + " version id ="+ 
				testVersion.getVersion().getId());
	}

	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws RetriableException, CriticalException{
		super.undo();
		TestVersionsFactory.getInstance().deleteTestVersion(testVersion);
		CommandDispatcher.getInstance().checkTestLegality(test);
		test.updateDB();
		StatisticData.getInstance().updateQuestionsStatistics(test.getId());
	}

	public String toString() {
		return "Add question " + question.getId()+ " to test " + test.getId();
	}

}
