/*
 * Created on 21/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.question;

import java.util.Iterator;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.command.version.VersionClone;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.CategoriesFactory;
import il.ac.technion.cs.course236308.e3.med.model.Category;
import il.ac.technion.cs.course236308.e3.med.model.Question;
import il.ac.technion.cs.course236308.e3.med.model.QuestionsFactory;

/**
 * @author Alexander, Bennyamin, Svetlana and Valentin 
 * Created in SSDL lab - Technion Israel
 *
 */
public class QuestionClone extends Command{

	private Question newQuestion = null;
	private Question question = null;
	private Category category = null;

	/**
	 * Method QuestionClone
	 */
	public QuestionClone(){
		this.category = CommandDispatcher.getInstance().getCurrentCategory();
		this.question = CommandDispatcher.getInstance().getCurrentQuestion();
	}

	/**
	 * Method QuestionClone
	 * @param category
	 * @param question
	 */
	public QuestionClone(Category category,Question question){
		this.question = question;
		this.category = category;
	}
	/**
	 * Create new question. Empty version is added to the question.
	 * The version will be active version.
	 */
	protected void execute() throws CriticalException, RetriableException{
		if (category == null){ throw new 
			RetriableException("Can't execute - no active category defined");
		}
		if (question == null){ throw new 
			RetriableException("Can't execute - no active question defined");
		}
		newQuestion = QuestionsFactory.getInstance().
						cloneQuestion(category,question);
		runAndPush(new QuestionSelect(newQuestion));
		
		runAndPush(new VersionClone(newQuestion,question.getUpdatedVersion()));
		for(Iterator it=question.getSecondaryCategoryIds().iterator();
					it.hasNext();){
			int secCatId = ((Integer)it.next()).intValue();
			Category secCategory = CategoriesFactory.getInstance().
												getCategory(secCatId);
			if(!category.isDescendant(secCategory)){
				runAndPush(new 
					QuestionAddSecondaryCategory(secCategory,newQuestion));
			}
		}
		CommandDispatcher.getInstance().checkQuestionLegality(question);
		MedReporter.report("# new question cloned, id = " 
							+ newQuestion.getId());
	}

	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws RetriableException, CriticalException{
		super.undo();
		QuestionsFactory.getInstance().deleteQuestion(newQuestion);
		CommandDispatcher.getInstance().removeIllegalObject(question);
		CommandDispatcher.getInstance().checkCategoryLegality(category);		
	}
	
	public String toString(){
		return "Clone question " + question.getId();
	}
	
}
