/*
 * Created on 21/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.question;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.command.version.VersionDelete;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.CategoriesFactory;
import il.ac.technion.cs.course236308.e3.med.model.Category;
import il.ac.technion.cs.course236308.e3.med.model.Question;
import il.ac.technion.cs.course236308.e3.med.model.QuestionsFactory;
import il.ac.technion.cs.course236308.e3.med.model.Version;

import java.util.List;

/**
 * @author Alexander, Bennyamin, Svetlana and Valentin 
 * Created in SSDL lab - Technion Israel
 *
 */
public class QuestionDelete extends Command{

	private Question question;
	private Category mainCategory;
	private int oldOrdinal; 
	/**
	 * Method QuestionDelete
	 */
	public QuestionDelete(){
		this.question = CommandDispatcher.getInstance().getCurrentQuestion();
	}
	/**
	 * Method QuestionDelete
	 * @param question
	 */
	public QuestionDelete(Question question){
		this.question = question;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws CriticalException, RetriableException{
		if (question == null){ throw new 
			RetriableException("Can't execute - no current question defined");
		}       
        if(question.hasVersionsInExams()){
            throw new RetriableException("Can't execute - question contains version(s) included in test(s)");
        }
		CategoriesFactory cf = CategoriesFactory.getInstance();
		mainCategory = cf.getCategory(question.getMainCategoryId());
		oldOrdinal = question.getOrdinal();
		if(question.equals(CommandDispatcher.getInstance().
					getCurrentQuestion())){
			runAndPush(new QuestionSelect(null));
		}

		List versions = question.getVersions();
		while(versions.size() > 0){
			runAndPush(new VersionDelete((Version)versions.get(0),false));
		}

		while(question.getSecondaryCategoryIds().size() > 0){
			Category cat = CategoriesFactory.getInstance().
						getCategory(((Integer)question.
						getSecondaryCategoryIds().get(0)).intValue());
			runAndPush(new QuestionRemoveSecondaryCategory(cat, question));
		}
		runAndPush(new QuestionDecreaseOrdinals(mainCategory,oldOrdinal));
		QuestionsFactory.getInstance().deleteQuestion(question);
		runAndPush(new QuestionDeleteIllegalObject(question));
		CommandDispatcher.getInstance().removeIllegalObject(question);
		CommandDispatcher.getInstance().checkCategoryLegality(mainCategory);
		MedReporter.report("# question deleted, id = " + question.getId());
	}	
	
	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws RetriableException, CriticalException{
		QuestionsFactory.getInstance().storeQuestion(mainCategory,question);
		super.undo();
		question.setOrdinal(oldOrdinal);
		question.updateDB();
		CommandDispatcher.getInstance().checkQuestionLegality(question);
	}
	
	public String toString() {
			return "Delete Question " + question.getId();
	}


	static class QuestionRemoveSecondaryCategory extends Command{
		private Category category;
		private Question quest;
		/**
		 * Method QuestionRemoveSecondaryCategory
		 * @param category
		 * @param question
		 */
		QuestionRemoveSecondaryCategory(Category category,Question question){
			this.category = category;
			this.quest = question;
		}
		
		/**
		 * Method execute
		 * @see Command#execute()
		 */
		protected void execute() throws CriticalException{
			category.removeLinkToQuestion(quest);
		}
		
		/**
		 * Method undo
		 * @see Command#undo()
		 */
		public void undo() throws CriticalException{
			category.addLinkToQuestion(quest);			
		}
	}

}
