/*
 * Created on 21/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.question;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Question;
import il.ac.technion.cs.course236308.e3.med.parameters.GeneralParams;
import il.ac.technion.cs.course236308.e3.med.parameters.QuestionParams;
import il.ac.technion.cs.course236308.e3.med.xmlTools.XMLImporter;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * @author Alexander, Bennyamin, Svetlana and Valentin 
 * Created in SSDL lab - Technion Israel
 *
 */
public class QuestionExport extends Command{
	private Question question;
	private String xmlFileName;
	private int mode;
	
	/**
	 * Method QuestionExport
	 * @param question
	 * @param fileName
	 * @param mode
	 */
	public QuestionExport(Question question,String fileName, int mode){
		this.question = question;
		this.xmlFileName = fileName;
		this.mode = mode;
	}

	/**
	 * Method QuestionExport
	 * @param fileName
	 * @param mode
	 */
	public QuestionExport(String fileName,int mode){
		this.question = CommandDispatcher.getInstance().getCurrentQuestion();
		this.xmlFileName = fileName;
		this.mode = mode;
	}

	
	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws CriticalException, RetriableException{

		if (question == null){
			throw new RetriableException("Can't execute - " +
								"no current question defined");
		}
		try{
			question.isLegal();
		}catch(RetriableException e){
			throw new RetriableException("Can't Export - Question Contains Version(s) in Illegal state");
		}
		
		if(xmlFileName.indexOf(".") < 0 ){
			xmlFileName+="." + QuestionParams.xmlQuestionFileExtension;
		}
		File file = new File(xmlFileName);
		if(file.exists() && mode==QuestionParams.ABORT_MODE){
			throw new RetriableException("File with name "+ xmlFileName + 
				" already exists, aborting command");
		}
		Element questions = new Element("questions");
		if(file.exists() && mode==QuestionParams.APPEND_MODE){
			try {
				SAXBuilder builder = GeneralParams.saxBuilder;
				Document doc = builder.build(file);
				questions = (Element) doc.getRootElement().detach();
			} catch (JDOMException e) {
				throw new RetriableException("Failed reading file "+
				xmlFileName +" error:"+e.getMessage());
			} catch (IOException e) {
				throw new RetriableException("Error occured while reading" +
					" file "+xmlFileName+ " error:"+e.getMessage());
			}
		}
		questions.addContent(question.toElement(false));
		
		Document doc = new Document(questions);
		Format format = Format.getPrettyFormat();
		format.setEncoding(GeneralParams.ENCODING);
		XMLOutputter out = new XMLOutputter(format);
		try {
			FileWriter fw = new FileWriter(file);
			out.output(doc,fw);
			fw.close();
		} catch (IOException e) {
			throw new 
			RetriableException("File export failed, error message: "+
			e.getMessage());
		}
		
		MedReporter.report("# question exported to XML, id = " +
		 question.getId());
		
		XMLImporter importer = 
		new XMLImporter(xmlFileName, QuestionParams.XSD_RELATIVE_FILE_PATH);
		try{
			importer.build();
			MedReporter.report("# XML validated and found: OK!");
		}catch(RetriableException e){
			MedReporter.report("# XML validated and found: WRONG!");
			MedReporter.report("# Reason: " + e.getMessage());
		}
		
	}
	
	public String toString() {
		return "Export Questions - undo has no effect for this command";
	}


}