/*
 * Created on 21/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.question;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.command.answer.AnswerRemoveFromTest;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Question;
import il.ac.technion.cs.course236308.e3.med.model.Test;
import il.ac.technion.cs.course236308.e3.med.model.TestAnswer;
import il.ac.technion.cs.course236308.e3.med.model.TestVersion;
import il.ac.technion.cs.course236308.e3.med.model.TestVersionsFactory;
import il.ac.technion.cs.course236308.e3.med.model.Version;
import il.ac.technion.cs.course236308.e3.med.model.VersionsFactory;
import il.ac.technion.cs.course236308.e3.med.stats.StatisticData;

/**
 * @author Alexander, Bennyamin, Svetlana and Valentin 
 * Created in SSDL lab - Technion Israel
 *
 */
public class QuestionRemoveFromTest extends Command {
	private Test test;
	private Question question;
	private Version activeVersion;
	private TestVersion testVersion;

	/**
	 * Method QuestionRemoveFromTest
	 */
	public QuestionRemoveFromTest() {
		this.question = CommandDispatcher.getInstance().getCurrentQuestion();
		this.test = CommandDispatcher.getInstance().getCurrentTest();
	}

	/**
	 * Method QuestionRemoveFromTest
	 * @param test
	 * @param question
	 */
	public QuestionRemoveFromTest(Test test, Question question) {
		this.question = question;
		this.test = test;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws CriticalException, RetriableException {
		if (test == null) { throw new 
			RetriableException("Can't execute - no current test defined");
		}
		if (question == null) { throw new 
			RetriableException("Can't execute - no current question defined");
		}
		if (question.getActiveVersionId() == null) { throw new 
			RetriableException("Can't execute - no active Version" +
				" defined for current question");
		}
		try {
			activeVersion =
				VersionsFactory.getInstance().getVersion(
					question.getActiveVersionId().intValue());
		} catch (RetriableException e) {
			throw new RetriableException(
				"Referential integrity violation - in question "
					+ question.getId()
					+ " active Version is"
					+ question.getActiveVersionId()
					+ " without such record in Version table");
		}
		if (activeVersion.getQuestionId() != question.getId()) {
			throw new RetriableException(
				"Referential integrity violation - in question "
					+ question.getId()
					+ " active Version is"
					+ question.getActiveVersionId()
					+ " however,this version does not belong to the question");
		}
		if (!test.getStatus().equals("U")) { throw new 
			RetriableException("Only Exams which are \"Under Constraction\"" +
				" can be edited");
		}
			try{		
				testVersion = TestVersionsFactory.getInstance().
						getTestVersion(test.getId(),activeVersion.getId());
			}catch(RetriableException e2){ throw new 
				RetriableException("Can't execute - active Version " +
					"not included into current test");
			}
		while (testVersion.getTestAnswers().size() > 0) {
			TestAnswer testAns =
				(TestAnswer) testVersion.getTestAnswers().get(0);
			runAndPush(new AnswerRemoveFromTest(testAns));
		}
		TestVersionsFactory.getInstance().deleteTestVersion(testVersion);
		StatisticData.getInstance().updateQuestionsStatistics(test.getId());
		MedReporter.report(
			"# question removed from test, question id="
				+ question.getId()
				+ " test id="
				+ test.getId());
	}

	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws RetriableException, CriticalException {
		super.undo();
		TestVersionsFactory.getInstance().storeTestVersion(testVersion);
		StatisticData.getInstance().updateQuestionsStatistics(test.getId());

	}
	
	public String toString() {
			return "Remove Question from Test";
	}


}
