/*
 * QuestionSetMainCategory.java
 * 
 *  Created on 23/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.question;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.CategoriesFactory;
import il.ac.technion.cs.course236308.e3.med.model.Category;
import il.ac.technion.cs.course236308.e3.med.model.Question;
import il.ac.technion.cs.course236308.e3.med.stats.StatsUpdateDB;

/**
 * @author svekant
 *
 */
public class QuestionSetMainCategory extends Command{

	private Question question = null;
	private Category category = null;
	private Category oldMainCategory;
	private int oldOrdinal;
	
	/**
	 * Method QuestionSetMainCategory
	 */
	public QuestionSetMainCategory(){
		this.question = CommandDispatcher.getInstance().getCurrentQuestion();
		this.category = CommandDispatcher.getInstance().getCurrentCategory();
	}

	/**
	 * Method QuestionSetMainCategory
	 * @param question
	 * @param category
	 */
	public QuestionSetMainCategory(Question question, Category category){
		this.question = question;
		this.category = category;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws RetriableException,CriticalException{
		if(question == null){ throw new 
			RetriableException("Can't execute - no current question defined");
		}
		if(category == null){ throw new 
			RetriableException("Can't execute - no current category defined");
		}
		if(question.hasVersionsInExams()){ throw new 
			RetriableException("Can't execute - question " +
				"contains locked versions");
		}
		runAndPush(new DeleteLinksToQuestion(category,question));
		
		oldMainCategory = CategoriesFactory.getInstance().
						getCategory(question.getMainCategoryId());
		oldOrdinal = question.getOrdinal();
		question.setOrdinal(category.getQuestions().size()+1);
		question.setMainCategoryId(category.getId());		
		category.getQuestions().add(question);
		oldMainCategory.getQuestions().remove(question);
		question.setMainCategoryId(category.getId());
		question.updateDB();	
		runAndPush(new QuestionDecreaseOrdinals(oldMainCategory,oldOrdinal));
		CommandDispatcher.getInstance().checkCategoryLegality(oldMainCategory);
		CommandDispatcher.getInstance().checkQuestionLegality(question);
		CommandDispatcher.getInstance().refreshTrees();
		StatsUpdateDB.getInstance().updateCategoryStatistics(oldMainCategory);
		StatsUpdateDB.getInstance().updateCategoryStatistics(category);
		MedReporter.report("# main category " + category.getId() + 
						" was set for question " + question.getId());
	}
	
	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws RetriableException, CriticalException{
		super.undo();
		category.getQuestions().remove(question);
		oldMainCategory.getQuestions().add(question);
		question.setMainCategoryId(oldMainCategory.getId());
		question.setOrdinal(oldOrdinal);
		question.updateDB();
		super.undo();		
		CommandDispatcher.getInstance().checkCategoryLegality(category);
		CommandDispatcher.getInstance().checkQuestionLegality(question);
		CommandDispatcher.getInstance().refreshTrees();
		StatsUpdateDB.getInstance().updateCategoryStatistics(oldMainCategory);
		StatsUpdateDB.getInstance().updateCategoryStatistics(category);
	}
	
	public String toString() {
		return "Change Main category of Question";
	}

	
	static class DeleteLinksToQuestion extends Command{
		private Question quest;
		private Category cat;
		boolean containLink = false;
		/**
		 * Method DeleteLinksToQuestion
		 * @param category
		 * @param question
		 */
		DeleteLinksToQuestion(Category category,Question question){
			this.cat = category;
			this.quest = question;
		}
		/**
		 * Method execute
		 * @see Command#execute()
		 */
	protected void execute() throws RetriableException, CriticalException{
		if(cat.getLinksToQuestionIds().contains(
				new Integer(quest.getId()))){
			containLink = true;
			cat.removeLinkToQuestion(quest);
			}			
		Integer parentId = cat.getParentID();
		if(parentId == null){
			return;
		}
		Category parent = CategoriesFactory.getInstance().
						getCategory(parentId.intValue());
		runAndPush(new DeleteLinksToQuestion(parent,quest));
		
	}
		
	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws RetriableException, CriticalException{
		if(containLink){
			cat.addLinkToQuestion(quest);
		}
		super.undo();
	}
	}
}
