package il.ac.technion.cs.course236308.e3.med.command.question;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Answer;
import il.ac.technion.cs.course236308.e3.med.model.CategoriesFactory;
import il.ac.technion.cs.course236308.e3.med.model.Question;
import il.ac.technion.cs.course236308.e3.med.model.QuestionsFactory;
import il.ac.technion.cs.course236308.e3.med.model.Version;
import il.ac.technion.cs.course236308.e3.med.parameters.GeneralParams;
import il.ac.technion.cs.course236308.e3.med.parameters.QuestionParams;
import il.ac.technion.cs.course236308.e3.med.xmlTools.XMLImporter;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
/**
 * @author shralex
 *
 * 
 */
public class QuestionsImport extends Command {
	String xmlFileName;
	
	/**
	 * Method QuestionsImport
	 * @param filename
	 */
	public QuestionsImport(String filename) {
		this.xmlFileName = filename;
	}
	
	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws RetriableException, CriticalException {
		if(xmlFileName.indexOf(".") < 0){
			xmlFileName+="." + QuestionParams.xmlQuestionFileExtension;
		}
		XMLImporter importer =
		 new XMLImporter(xmlFileName, QuestionParams.XSD_RELATIVE_FILE_PATH);
		Document doc = importer.build();
		Element questionsElem = doc.getRootElement(); 
		List questions = new ArrayList();

		int i = 1;
		for (Iterator it = questionsElem.getChildren("question").iterator();it.hasNext(); i++) {
			Question question = buildQuestion((Element) it.next(), i);
			questions.add(question);
		}
		QuestionsFactory.getInstance().importQuestions(questions);
		MedReporter.report("# Question import command completed");

	}

	/**
	 * Method buildQuestion
	 * @param questionElem
	 * @param questionId
	 * @return Question
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public Question buildQuestion(Element questionElem, int questionId)
		 throws RetriableException, CriticalException {
		String mainCat = questionElem.getChildTextTrim("mainCategory");
		String canceledText = questionElem.getAttributeValue("canceled");
		boolean isCanceled = false;
		
		//canceled is an optional attribute
		if (canceledText != null &&
			 (canceledText.equals("true") || canceledText.equals("1"))) {
			isCanceled = true;
		}
		Question question = new Question();
		question.setActiveVersionId(null);
		question.setCancelled(isCanceled);
		question.setId(questionId);
		//question.setOrdinal(questionId); 
		//ordinal is not imported from the file like in answer
		CategoriesFactory cf = CategoriesFactory.getInstance();
		int categoryId = cf.createCategoriesFromFullName(mainCat);
		question.setMainCategoryId(categoryId);
		
		List versions = question.getVersions();
		int i = 1;
		for (Iterator it = questionElem.getChildren("version").iterator(); 
			it.hasNext(); i++) {
			Version version =  buildVersion((Element) it.next(), i, question);
			versions.add(version);
		}
	
		return question;
	}
	/**
	 * Method extractDate
	 * @param xsdDateTimeString
	 * @return Calendar
	 * @throws RetriableException
	 */
	public Calendar extractDate(String xsdDateTimeString) 
		throws RetriableException
	{
		
		if (xsdDateTimeString == null) return null;
		GregorianCalendar gc = new GregorianCalendar();
		{
			if (xsdDateTimeString.startsWith("-")) {
					xsdDateTimeString = xsdDateTimeString.substring(1);
			} 
			final int datelength = 19;
			String simpleDateText = xsdDateTimeString.substring(0,datelength);
			simpleDateText = simpleDateText.replace('T', ' ');
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			Date simpleDate;
			try {
				simpleDate = sdf.parse(simpleDateText);
			} catch (java.text.ParseException e) {
				throw new RetriableException(e.getMessage());
			}
			gc.setTimeInMillis(simpleDate.getTime());			
		}
		return gc;
	}
	/**
	 * Method buildVersion
	 * @param versionElem
	 * @param versionId
	 * @param question
	 * @return Version
	 * @throws RetriableException
	 */
	public Version buildVersion(Element versionElem, int versionId, 
			Question question) throws RetriableException {
		String note = versionElem.getChildTextTrim("note");
		if (note==null) note = new String("");

		String activeText = versionElem.getAttributeValue("active");
		boolean isActive = false;
		
		//active is an optional attribute
		if (activeText != null &&
			 (activeText.equals("true") || activeText.equals("1"))) {
			isActive = true;
		}

		String createdText = versionElem.getAttributeValue("created");
		Calendar created = extractDate(createdText);

		String updatedText = versionElem.getAttributeValue("updated");
		Calendar updated = extractDate(updatedText);
		//updated is optional
		if (updated==null) updated = created;

		Element bodyElem = versionElem.getChild("body");
		
		String langText = bodyElem.getAttributeValue("lang");
		if (langText==null) langText = new String("en");
		
		XMLOutputter xo = new XMLOutputter(Format.getCompactFormat());
		xo.getFormat().setEncoding(GeneralParams.ENCODING);
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		try{
			xo.outputElementContent(bodyElem, bos);
		}catch(IOException e){
			throw new RetriableException(e.getMessage());
		}
		String body = bos.toString();
		
		Version version = new Version();
		version.setCorrectAnswerId(0);
		version.setId(versionId);
		if (isActive) question.setActiveVersionId(new Integer(versionId));
		version.setBody(body);
		version.setCreated(created);
		version.setUpdated(updated);
		version.setLanguage(langText);
		version.setNotes(note);
		version.setQuestionId(question.getId());
		
		List answers = version.getAnswers();
		byte i = 1;
		for (Iterator it = versionElem.getChildren("answer").iterator();
		 it.hasNext(); i++) {
			Answer answer = buildAnswer((Element) it.next(),i,version);
			answers.add(answer);
		}				
		
		return version;
	}
	/**
	 * Method buildAnswer
	 * @param answerElem
	 * @param answerId
	 * @param version
	 * @return Answer
	 * @throws RetriableException
	 */
	public Answer buildAnswer(Element answerElem,byte answerId,Version version)
		 throws RetriableException{
		String mixableText = answerElem.getAttributeValue("mixable");
		boolean isMixable = true;
		//mixable is an optional attribute
		if (mixableText != null && (mixableText.equals("false")
		 || mixableText.equals("0"))) {
			isMixable = false;
		}
		String correctText = answerElem.getAttributeValue("correct");
		boolean isCorrect = false;
		//mixable is an optional attribute
		if (correctText != null && (correctText.equals("true") ||
		 correctText.equals("1"))) {
			isCorrect = true;
		}
		String ordinalText = answerElem.getAttributeValue("ordinal");
		Byte ordinal = null;
		try {
			ordinal = (new Byte(ordinalText));
		} catch (NumberFormatException e){
			throw new RetriableException(e.getMessage());
		}
		Element textElem = answerElem.getChild("text");
		String langText = textElem.getAttributeValue("lang");
		if (langText==null) langText = new String("en");
		XMLOutputter xo = new XMLOutputter(Format.getCompactFormat());
		xo.getFormat().setEncoding(GeneralParams.ENCODING);
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		try{
			xo.outputElementContent(textElem, bos);
		}catch(IOException e){
			throw new RetriableException(e.getMessage());
		}
		String text = bos.toString();
		
		Answer answer = new Answer();
		answer.setLanguage(langText);
		answer.setMixable(isMixable);
		if (isCorrect) version.setCorrectAnswerId(answerId);
		answer.setOrdinal(ordinal.byteValue());
		answer.setId(answerId);
		answer.setText(text);
		answer.setVersionId(version.getId());
		return answer;	
	}
	
	public String toString() {
		return "Import Questions";
	}

}
