/*
 * Created on 21/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.test;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Test;
import il.ac.technion.cs.course236308.e3.med.parameters.GeneralParams;
import il.ac.technion.cs.course236308.e3.med.parameters.TestParams;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * @author Alexander, Bennyamin, Svetlana and Valentin 
 * Created in SSDL lab - Technion Israel
 *
 */
public class TestExport extends Command{
	private Test test;
	private String xmlFileName;
	private int mode;
	private Integer copyNum = null;
	/**
	 * Method TestExport
	 * @param test
	 * @param fileName
	 * @param mode
	 */
	public TestExport(Test test,String fileName, int mode){
		this.test = test;
		this.xmlFileName = fileName;
		this.mode = mode;
	}
	/**
	 * Method TestExport
	 * @param test
	 * @param fileName
	 * @param mode
	 * @param copyNum
	 */
	public TestExport(Test test,String fileName, int mode, int copyNum){
		this.test = test;
		this.xmlFileName = fileName;
		this.mode = mode;
		this.copyNum = new Integer(copyNum);
	}
	/**
	 * Method TestExport
	 * @param fileName
	 * @param mode
	 */
	public TestExport(String fileName,int mode){
		this.test = CommandDispatcher.getInstance().getCurrentTest();
		this.xmlFileName = fileName;
		this.mode = mode;
	}
	/**
	 * Method TestExport
	 * @param fileName
	 * @param mode
	 * @param copyNum
	 */
	public TestExport(String fileName,int mode, int copyNum){
		this.test = CommandDispatcher.getInstance().getCurrentTest();
		this.xmlFileName = fileName;
		this.mode = mode;
		this.copyNum = new Integer(copyNum);
	}
	
	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws CriticalException, RetriableException{
		if(test == null){
			throw new 
			RetriableException("Can't execute - no active test defined");			
		}
		
		try {
			test.isLegal();
		} catch (RetriableException e) {
			MedReporter.report("# " + e.getMessage());
			throw new RetriableException("<html>Can't execute - " +
			"test is in illegal state<br>(see details in Illegal States window)</html>");

		}
		
		if(xmlFileName.indexOf(".") < 0){
			xmlFileName+="." + TestParams.xmlFileExtension;
		}
		File file = new File(xmlFileName);
		if(file.exists() && mode==TestParams.ABORT_MODE){
			throw new RetriableException("File with name "+ xmlFileName + 
				" already exists, aborting command");
		}
		Element tests = new Element("tests");
		if(file.exists() && mode==TestParams.APPEND_MODE){
			
			try {
				SAXBuilder builder = GeneralParams.saxBuilder;
				Document doc = builder.build(file);
				tests = (Element) doc.getRootElement().detach();
			} catch (JDOMException e) {
				throw new 
				RetriableException("Failed reading file "+xmlFileName +
												" error:"+e.getMessage());
			} catch (IOException e) {
				throw new 
				RetriableException("Error occured while reading file "+
									xmlFileName+ " error:"+e.getMessage());
			}
		}
		Element testElem = test.toElement();
		if(copyNum != null){
			testElem.addContent((new Element("copyNumber")).
									setText(copyNum.toString()));
		}
		tests.addContent(testElem);
		
		Document doc = new Document(tests);
		Format format = Format.getPrettyFormat();
		XMLOutputter out = new XMLOutputter(format.setEncoding(GeneralParams.ENCODING));
		try {
			FileWriter fWriter = new FileWriter(file);
			out.output(doc,fWriter);
			fWriter.close();
		} catch (IOException e) {
			throw new 
				RetriableException("File export failed, error message: "+
															e.getMessage());
		}
		MedReporter.report("# test exported to XML, id = " + test.getId());
		
/*		XMLImporter importer =
			 new XMLImporter(xmlFileName, TestParams.XSD_RELATIVE_FILE_PATH);
		try{
			importer.build();
			MedReporter.report("# XML validated and found: OK!");
		}catch(RetriableException e){
			MedReporter.report("# XML validated and found: WRONG!");
			MedReporter.report("# Reason: " + e.getMessage());
		}*/
	}
	
	public String toString() {
		return "Export test - no effect for undo";
	}

}
