/*
 * Created on 21/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.test;

import java.text.DateFormat;
import java.util.Calendar;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Test;

/**
 * @author Alexander, Bennyamin, Svetlana and Valentin
 * Created in SSDL lab - Technion Israel
 *
 */
public class TestSetDate extends Command {
	private Test test = null;
	private Calendar oldDate = null;
	private Calendar newDate = null;

	/**
	 * Method TestSetDate
	 * @param test
	 * @param newDate
	 */
	public TestSetDate(Test test, Calendar newDate) {
		this.test = test;
		this.newDate = newDate;
	}

	/**
	 * Method TestSetDate
	 * @param newDate
	 */
	public TestSetDate(Calendar newDate) {
		this.test = CommandDispatcher.getInstance().getCurrentTest();
		this.newDate = newDate;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws CriticalException, RetriableException {
		if (test == null) {	throw new 
			RetriableException("Can't execute - no active test defined");
		}
		if (!test.getStatus().equals("U")) { throw new 
			RetriableException("Can't execute - test isn't under construction");
		}
		oldDate = test.getDate();
		test.setDate(newDate);
		runAndPush(new TestSetUpdatedDate(test));
		test.updateDB();
		CommandDispatcher.getInstance().checkTestLegality(test);
		MedReporter.report(
			"# test Date was changed, id = "
				+ test.getId()
				+ " new Date="
				+ DateFormat.getDateTimeInstance().format(
					test.getDate().getTime()));
	}

	/**
	 * Method undo
	 * @throws CriticalException
	 * @throws RetriableException
	 * @see Command#undo()
	 */
	public void undo() throws RetriableException, CriticalException {
		super.undo();
		test.setDate(oldDate);
		test.updateDB();
		CommandDispatcher.getInstance().checkTestLegality(test);
	}
	
	public String toString() {
		return "Set Date for test";
	}

}
