/*
 * Created on 21/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.test;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Test;
import il.ac.technion.cs.course236308.e3.med.xmlTools.TextCorrectnessChecker;

/**
 * @author Alexander, Bennyamin, Svetlana and Valentin
 * Created in SSDL lab - Technion Israel
 *
 */
public class TestSetInstructions extends Command {
	private Test test = null;
	private String oldInstructions = null;
	private String newInstructions = null;

	/**
	 * Method TestSetInstructions
	 * @param test
	 * @param newInstructions
	 */
	public TestSetInstructions(Test test, String newInstructions) {
		this.test = test;
		this.newInstructions = newInstructions;
	}

	/**
	 * Method TestSetInstructions
	 * @param newInstructions
	 */
	public TestSetInstructions(String newInstructions) {
		this.test = CommandDispatcher.getInstance().getCurrentTest();
		this.newInstructions = newInstructions;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws CriticalException, RetriableException {
		if (test == null) { throw new 
			RetriableException("Can't execute - no active test defined");
		}
		if (!test.getStatus().equals("U")) { throw new 
			RetriableException("Can't execute - test isn't under construction");
		}
		TextCorrectnessChecker tcc = new TextCorrectnessChecker(newInstructions);
		tcc.validate();
		oldInstructions = test.getInstructions();
		test.setInstructions(newInstructions);
		runAndPush(new TestSetUpdatedDate(test));
		test.updateDB();
		MedReporter.report(
			"# test Instructions was changed, id = "
				+ test.getId()
				+ " new Instructions="
				+ newInstructions);
	}

	/**
	 * Method undo
	 * @throws CriticalException
	 * @throws RetriableException
	 * @see Command#undo()
	 */
	public void undo() throws  RetriableException, CriticalException {
		super.undo();
		test.setInstructions(oldInstructions);
		test.updateDB();
	}
	
	public String toString() {
		return "Set Instructions for test";
	}

}
