/*
 * Created on 21/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.test;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Test;

/**
 * @author Alexander, Bennyamin, Svetlana and Valentin 
 * Created in SSDL lab - Technion Israel
 *
 */
public class TestSetStatus extends Command {
	private Test test = null;
	private String oldStatus = null;
	private String newStatus = null;

	/**
	 * Method TestSetStatus
	 * @param test
	 * @param newStatus
	 */
	public TestSetStatus(Test test, String newStatus) {
		this.test = test;
		this.newStatus = newStatus;
	}

	/**
	 * Method TestSetStatus
	 * @param newStatus
	 */
	public TestSetStatus(String newStatus) {
		this.test = CommandDispatcher.getInstance().getCurrentTest();
		this.newStatus = newStatus;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws CriticalException, RetriableException {
		if (test == null) { throw new 
			RetriableException("Can't execute - no active test defined");
		}
		if (!(newStatus.equals("U")
			|| newStatus.equals("P")
			|| newStatus.equals("G"))) {
			throw new RetriableException("Exam status must be U,P or G");
		}
		if (test.getStatus().equals("G") && !newStatus.equals("G")) {
			throw new 
			RetriableException("Status of Given exam can not be changed");
		}
		oldStatus = test.getStatus();
		test.setStatus(newStatus);
		runAndPush(new TestSetUpdatedDate(test));
		test.updateDB();
		CommandDispatcher.getInstance().checkTestLegality(test);
		MedReporter.report(
			"# test Status was changed, id = "
				+ test.getId()
				+ " new Status="
				+ newStatus);
	}

	/**
	 * Method undo
	 * @throws CriticalException
	 * @throws RetriableException
	 * @see Command#undo()
	 */
	public void undo() throws  RetriableException, CriticalException {
		super.undo();
		test.setStatus(oldStatus);
		test.updateDB();
		CommandDispatcher.getInstance().checkTestLegality(test);
	}
	
	public String toString() {
		return "Set Status for test";
	}

}
