/*
 * Created on 21/04/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.command.test.version;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.command.MedReporter;
import il.ac.technion.cs.course236308.e3.med.command.answer.AnswerRemoveFromTest;
import il.ac.technion.cs.course236308.e3.med.command.test.TestSetUpdatedDate;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Test;
import il.ac.technion.cs.course236308.e3.med.model.TestAnswer;
import il.ac.technion.cs.course236308.e3.med.model.TestVersion;
import il.ac.technion.cs.course236308.e3.med.model.TestVersionsFactory;
import il.ac.technion.cs.course236308.e3.med.model.TestsFactory;

/**
 * @author Alexander, Bennyamin, Svetlana and Valentin 
 * Created in SSDL lab - Technion Israel
 *
 */
public class TestVersionRemoveFromTest extends Command {
	private TestVersion testVersion;
	private Test test;

	/**
	 * Method QuestionRemoveFromTest
	 * @param test
	 * @param question
	 */
	public TestVersionRemoveFromTest(TestVersion testVersion) {
		this.testVersion = testVersion;
	}

	/**
	 * Method execute
	 * @see Command#execute()
	 */
	protected void execute() throws CriticalException, RetriableException {
		if (testVersion == null) { throw new 
			RetriableException("Can't execute - no current test version defined");
		}
		test = TestsFactory.getInstance().getTest(testVersion.getTestId());
		if (!test.getStatus().equals("U")) { throw new 
			RetriableException("Only Exams which are \"Under Constraction\"" +
				" can be edited");
		}
		while (testVersion.getTestAnswers().size() > 0) {
			TestAnswer testAns =
				(TestAnswer) testVersion.getTestAnswers().get(0);
			runAndPush(new AnswerRemoveFromTest(testAns));
		}
		TestVersionsFactory.getInstance().deleteTestVersion(testVersion);
		runAndPush(new TestSetUpdatedDate(test));
		test.updateDB();
		CommandDispatcher.getInstance().checkTestLegality(test);
		MedReporter.report("# version removed from test, test id="
				+ test.getId());
	}

	/**
	 * Method undo
	 * @see Command#undo()
	 */
	public void undo() throws RetriableException, CriticalException {
		super.undo();
		TestVersionsFactory.getInstance().storeTestVersion(testVersion);
		test.updateDB();
		CommandDispatcher.getInstance().checkTestLegality(test);
	}
	
	public String toString() {
		return "Remove Version " + testVersion.getVersion().getId()+ " from Test " + testVersion.getTestId();
	}

}
