package il.ac.technion.cs.course236308.e3.med.control;

import il.ac.technion.cs.course236308.e3.med.parameters.CategoryParams;
import il.ac.technion.cs.course236308.e3.med.parameters.GeneralParams;
import il.ac.technion.cs.course236308.e3.med.parameters.QuestionParams;
import il.ac.technion.cs.course236308.e3.med.parameters.TestParams;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * @author Shraer Alexander
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class OptionsLoader {
	public static void load(){
		File optionsFile = new File(GeneralParams.OPTIONS_FILE);
		if (!optionsFile.exists()) {
			System.out.println("# Failed locating options file: " + GeneralParams.OPTIONS_FILE);
			System.out.println("# Using system defaults");
			return;
		} 
		String newCatName = null;
		String xhtmlFileExt = null;
		String xtqFileExt = null;
		String xttFileExt = null;
		String defaultLanguage = null;
		boolean showConfirmations = false;
		boolean showIllegalStates = false;
		boolean showQuestionLinks = false;
		boolean showSubcatQuestions = false;
		boolean loadedOk = false;
		try{
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(optionsFile);
			Element root = doc.getRootElement();
			newCatName = root.getChildText("defaultNameForNewCategory").trim();
			xhtmlFileExt = root.getChildText("xhtmlFileExtension").trim();
			xtqFileExt = root.getChildText("questionExportFileExtension").trim();
			xttFileExt = root.getChildText("testExportFileExtension").trim();
			defaultLanguage = root.getChildText("defaultLanguage").trim();
			showConfirmations = Boolean.valueOf(root.getChildText("showConfirmations").trim()).booleanValue();
			showIllegalStates = Boolean.valueOf(root.getChildText("showIllegalStates").trim()).booleanValue();
			showQuestionLinks = Boolean.valueOf(root.getChildText("showQuestionLinks").trim()).booleanValue();
			showSubcatQuestions = Boolean.valueOf(root.getChildText("showSubcatQuestions").trim()).booleanValue();
			loadedOk = true;
		}catch (Throwable e){
			System.out.println("# Error loading options file. Using system defaults.");
			Store();
		}
		if (loadedOk){
			TestParams.xhtmlFileExtension = xhtmlFileExt;
			TestParams.xmlFileExtension = xttFileExt;
			QuestionParams.xmlQuestionFileExtension = xtqFileExt;
			QuestionParams.defaultQuestionLanguage = defaultLanguage;
			CategoryParams.newCategoryDefaultName = newCatName;
			GeneralParams.showConfirmDialogs = showConfirmations;
			GeneralParams.showIllegalStates = showIllegalStates;
			CategoryParams.showIncludingLinks = showQuestionLinks;
			CategoryParams.showAllSubQuestions = showSubcatQuestions;
			System.out.println("# Loaded options file. ");
		}
	}
	public static void Store(){
		Element options = new Element("options");
		Element newCatName = 
			(new Element("defaultNameForNewCategory")).setText(CategoryParams.newCategoryDefaultName);
		options.addContent(newCatName);
		Element defaultLanguage = 
			(new Element("defaultLanguage")).setText(QuestionParams.defaultQuestionLanguage);
		options.addContent(defaultLanguage);
		Element xtqFileExt = 
					(new Element("questionExportFileExtension")).setText(QuestionParams.xmlQuestionFileExtension);
		options.addContent(xtqFileExt);
		Element xttFileExt = 
					(new Element("testExportFileExtension")).setText(TestParams.xmlFileExtension);
		options.addContent(xttFileExt);
		Element xhtmlFileExt = 
					(new Element("xhtmlFileExtension")).setText(TestParams.xhtmlFileExtension);
		options.addContent(xhtmlFileExt);
		Element showConfirms = 
			(new Element("showConfirmations")).setText(String.valueOf(GeneralParams.showConfirmDialogs));
		options.addContent(showConfirms);
		Element showIllegalStates = 
			(new Element("showIllegalStates")).setText(String.valueOf(GeneralParams.showIllegalStates));
		options.addContent(showIllegalStates);
		Element showQuestionLinks = 
			(new Element("showQuestionLinks")).setText(String.valueOf(CategoryParams.showIncludingLinks));
		options.addContent(showQuestionLinks);
		Element showSubcatQuestions = 
			(new Element("showSubcatQuestions")).setText(String.valueOf(CategoryParams.showAllSubQuestions));
		options.addContent(showSubcatQuestions);
		
		
		Document doc = new Document(options);
		Format format = Format.getPrettyFormat();
		XMLOutputter out = new XMLOutputter(format.setEncoding(GeneralParams.ENCODING));
		try {
			FileWriter fWriter = new FileWriter(GeneralParams.OPTIONS_FILE);
			out.output(doc,fWriter);
			fWriter.close();
			System.out.println("# Options saved in the options file");
		} catch (IOException e) {
			System.out.println("# Error saving options to file: " + e.getMessage());
		}

		

	}
}
