/*
 * Created on Jun 16, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package il.ac.technion.cs.course236308.e3.med.control.actions;

import il.ac.technion.cs.course236308.e3.med.command.Command;
import il.ac.technion.cs.course236308.e3.med.control.GuiCommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.control.ModelChangesListener;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.gui.panels.MainWindow;
import il.ac.technion.cs.course236308.e3.med.gui.states.GuiState;
import il.ac.technion.cs.course236308.e3.med.gui.treenodes.MedNode;
import il.ac.technion.cs.course236308.e3.med.parameters.GeneralParams;

import java.awt.event.ActionEvent;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

/**
 * @author Valentin Kravtsov
 *
 * Created in DS Laboratory - Technion, Israel 2004
 */
public abstract class CommandAction extends AbstractAction {
	protected Vector listeners = new Vector();
	protected MainWindow mainWindow;
	private GuiState guiStateForUndo;
	private GuiState guiStateForRedo;
	protected Command command = null;
	
	public static boolean firstMultiSelectionExecution = true;

	public CommandAction(String name, ImageIcon image){
		super(name,image);
	}
	
	public CommandAction(String name){
		this(name,GeneralParams.ICON_NONE);
	}

	public void init(MainWindow frame){
		this.mainWindow = frame;
	}
		
	public CommandAction addListener(ModelChangesListener listener){
		listeners.add(listener);
		return this;
	}
	
	public CommandAction addAllListeners(Vector allListeners){
		listeners.addAll(allListeners);
		return this;
	}

	public Command getCommand(){
		return command;
	}

	public GuiState getGuiStateForUndo(){
		return guiStateForUndo;
	}
	
	public GuiState getGuiStateForRedo(){
		return guiStateForRedo;
	}
	
	public MainWindow getMainWindow(){
		return mainWindow;
	}
	
	public void saveGuiStateForUndo(){
		MedNode activeNode = mainWindow.getMainWindowController().getActiveNode();
		guiStateForUndo = mainWindow.getMainWindowController().getPanelForNode(activeNode).saveGuiState(activeNode);		
	}

	public void saveGuiStateForRedo(){
		MedNode activeNode = mainWindow.getMainWindowController().getActiveNode();
		guiStateForRedo = mainWindow.getMainWindowController().getPanelForNode(activeNode).saveGuiState(activeNode);		
	}

	public CommandAction toShortFormat(){
		putValue(NAME,"");
		return this;
	}
	
	public void actionPerformed(ActionEvent e){
		try {
			GuiCommandDispatcher.getInstance().run(this);
		} catch (RetriableException e1) {
//			e1.printStackTrace();
			JOptionPane.showMessageDialog(mainWindow, e1.getMessage(), "Med Error Occured", 
												JOptionPane.ERROR_MESSAGE);
						
		} catch(CriticalException e2){
			e2.printStackTrace();
			JOptionPane.showMessageDialog(mainWindow, e2.getMessage(), "Critical Error Occured", 
												JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
	}
	
	protected void runCommand(Command command) throws RetriableException, CriticalException{
		this.command = command;
		command.run();
	}
	
	abstract public void execute() throws RetriableException, CriticalException;
	
	public void undo() throws RetriableException, CriticalException{
		command.undo();
	}
	
	public String toString(){
		if(command == null){
			return "No command was set yet";
		}
		return command.toString();
	}
	
	protected void firePropertiesChanged() {
		for(Iterator it=listeners.iterator();it.hasNext();){
			((ModelChangesListener)it.next()).nodePropertiesChanged();
		}
	}
	
	protected void fireNodeAdded(MedNode newNode){
		for(Iterator it=listeners.iterator();it.hasNext();){
			((ModelChangesListener)it.next()).nodeAdded(newNode);
		}
	}
	
	protected void fireNodeRemoved(MedNode deletedNode,MedNode parentNode,int index){
		for(Iterator it=listeners.iterator();it.hasNext();){
			((ModelChangesListener)it.next()).nodeRemoved(deletedNode,parentNode,index);
		}
	}
}
