/*
 * Created on Jul 23, 2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.gui.panels;

import il.ac.technion.cs.course236308.e3.med.control.controllers.TestStatisticsController;
import il.ac.technion.cs.course236308.e3.med.gui.treenodes.TestCategoryNode;
import il.ac.technion.cs.course236308.e3.med.model.Answer;
import il.ac.technion.cs.course236308.e3.med.model.Test;
import il.ac.technion.cs.course236308.e3.med.model.TestCategoriesFactory;
import il.ac.technion.cs.course236308.e3.med.model.TestVersion;
import il.ac.technion.cs.course236308.e3.med.model.Version;
import il.ac.technion.cs.course236308.e3.med.model.VersionsFactory;
import il.ac.technion.cs.course236308.e3.med.parameters.GeneralParams;
import il.ac.technion.cs.course236308.e3.med.stats.AnswerInfo;
import il.ac.technion.cs.course236308.e3.med.stats.ComputeColumnWidthDFS;
import il.ac.technion.cs.course236308.e3.med.stats.QuestionsStats;
import il.ac.technion.cs.course236308.e3.med.stats.StatisticData;
import il.ac.technion.cs.course236308.e3.med.stats.StudentsStats;

import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultSingleSelectionModel;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableColumn;


/**
 * @author svekant
 *
 */
public class StatisticsPanel extends JPanel {
	private TestStatisticsController controller;
	
	private JTabbedPane tabbedPane;
	private JPanel statisticsPanel;
	
	
	private JPanel examStatTab = new JPanel(new GridLayout(1,1));
	private JPanel studentsStatTab = new JPanel(new GridLayout(1,1));
	private JPanel questionsStatTab = new JPanel(new GridLayout(1,1));
	private JPanel gradesStatTab = new JPanel(new GridLayout(1,1));
	
	private JTable answersStatTable = new JTable();
	private JPanel answerStatPanel = new JPanel(new GridLayout(1,1));
		
	private JPanel studStatLowRigthPane = new JPanel(new GridLayout(1,1));
	private JPanel studStatLowLeftPane = new JPanel(new GridLayout(1,1));
	private JPanel studStatHighPanel = new JPanel(new GridLayout(1,1));
	private JTable studentsStatTable = new JTable();
		
	private JPanel questStatHighPanel = new JPanel(new GridLayout(1,1));
	private JPanel questStatLowLeftPane = new JPanel(new GridLayout(1,1));
	private JPanel questStatLowRigthPane = new JPanel();
	private JTable versionStatTable = new JTable();
	
	private Integer versionSelected = null;
	
	private JSplitPane studHighLowSplitPane;
	private JSplitPane studLeftRightSplitPane;
	
	private JSplitPane questHighLowSplitPane;
	private JSplitPane questLeftRightSplitPane;
	
	private final int STUD_HIGH_LOW_DIVIDER_POSITION = (int)(Toolkit.getDefaultToolkit().getScreenSize().getHeight()/4);
	private final int STUD_LEFT_RIGHT_DIVIDER_POSITION = (int)(Toolkit.getDefaultToolkit().getScreenSize().getWidth()/3);
	private final int QUEST_HIGH_LOW_DIVIDER_POSITION = (int)(Toolkit.getDefaultToolkit().getScreenSize().getHeight()/5);
	private final int QUEST_LEFT_RIGHT_DIVIDER_POSITION = (int)(Toolkit.getDefaultToolkit().getScreenSize().getWidth()/3);
	
	public StatisticsPanel(TestStatisticsController controller){
        this.controller = controller;
    }
	  
	public void init(){
        this.setLayout(new GridLayout(1,1));	
        this.add(createMainPanel());			
        addListeners();
    }
	   
	private JComponent createMainPanel(){
		statisticsPanel = new JPanel();
		statisticsPanel.setLayout(new GridLayout(1,1));
		questStatLowRigthPane.setLayout(new BoxLayout(questStatLowRigthPane, BoxLayout.Y_AXIS));
		
		tabbedPane = new JTabbedPane();
		tabbedPane.setModel(new DefaultSingleSelectionModel());		
		tabbedPane.setTabPlacement(JTabbedPane.BOTTOM);
		
		tabbedPane.addTab("Exam Statistics",examStatTab);
		tabbedPane.addTab("Questions Statistics",questionsStatTab);
		tabbedPane.addTab("Grades Statistics",gradesStatTab);
		tabbedPane.addTab("Students Statistics",studentsStatTab);
		
		
		studLeftRightSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
    			studStatLowLeftPane, studStatLowRigthPane);			
		studHighLowSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
		   		studStatHighPanel, studLeftRightSplitPane);
		studHighLowSplitPane.setOneTouchExpandable(true);
		studHighLowSplitPane.setDividerLocation(STUD_HIGH_LOW_DIVIDER_POSITION); 		        
		studLeftRightSplitPane.setOneTouchExpandable(true);
		studLeftRightSplitPane.setDividerLocation(STUD_LEFT_RIGHT_DIVIDER_POSITION); 		        
		studentsStatTab.add(studHighLowSplitPane);
		
		questLeftRightSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
    			questStatLowLeftPane, questStatLowRigthPane);
		questHighLowSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
		   		questStatHighPanel, questLeftRightSplitPane);
		questHighLowSplitPane.setOneTouchExpandable(true);
		questHighLowSplitPane.setDividerLocation(QUEST_HIGH_LOW_DIVIDER_POSITION); 		        
		questLeftRightSplitPane.setOneTouchExpandable(true);
		questLeftRightSplitPane.setDividerLocation(QUEST_LEFT_RIGHT_DIVIDER_POSITION); 		        
		questionsStatTab.add(questHighLowSplitPane);
		
		statisticsPanel.add(tabbedPane);
		return statisticsPanel;
	}
	
		
	public void updateExamStatisticsPanel(){
		examStatTab.removeAll();
		examStatTab.add(createExamStatisticsPanel());
	}
	
	public void updateGradesStatisticsPanel(){
		gradesStatTab.removeAll();
		gradesStatTab.setLayout(new GridLayout(1,1));
		gradesStatTab.add(createGradesStatisticsPanel()); 
	}
	
	public JComponent createExamStatisticsPanel(){
		if(controller.getTest().getStatus().equals("U")){		
			return ChartPanels.getNotAvailableStatPanel();
		}
		
		QuestionsStats qs = StatisticData.getInstance().getQuestionsStats(controller.getTest().getId());
		if(qs == null)
			return ChartPanels.getNotAvailableStatPanel();
		StudentsStats ss = qs.getStudentStats();
		if(ss == null || ss.getNumStudents() == 0)
			return ChartPanels.getNotAvailableStatPanel();
		
		JPanel highPanel1;
		highPanel1 = ChartPanels.getTestStatisticPanel(controller.getTest().getId());
		highPanel1.setLayout(new GridLayout(1,1));
		JPanel highPanel2;
		highPanel2 = ChartPanels.getRoundedGradesStatPanel(controller.getTest().getId());
		highPanel2.setLayout(new GridLayout(1,1));
        JSplitPane pane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
        				highPanel1, highPanel2);
		JPanel lowPanel = ChartPanels.getTestGradesStatBarChartPanel(controller.getTest());
		lowPanel.setLayout(new GridLayout(1,1));
					
	    JSplitPane pane2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
	        		pane,lowPanel );
	    pane.setOneTouchExpandable(true);    
	    pane2.setDividerLocation(250); 
	    pane.setDividerLocation(400);     
		return pane2;		

		
	}
	
	private JComponent createGradesStatisticsPanel(){
		
		if(controller.getTest().getStatus().equals("U")){		
			return ChartPanels.getNotAvailableStatPanel();
		}
		QuestionsStats qs = StatisticData.getInstance().getQuestionsStats(controller.getTest().getId());
		if(qs == null)
			return ChartPanels.getNotAvailableStatPanel();
		StudentsStats ss = qs.getStudentStats();
		if(ss == null || ss.getNumStudents() == 0)
			return ChartPanels.getNotAvailableStatPanel();
		
		return ChartPanels.getTestGradesStatisticPanel(controller.getTest().getId());			
	}
	
	public void updateQuestionsStatisticsPanel(){
		questStatLowRigthPane.removeAll();
		questStatLowLeftPane.removeAll();
		questStatHighPanel.removeAll();

		if(controller.getTest().getStatus().equals("U")){		
			questStatHighPanel.add(ChartPanels.getNotAvailableStatPanel());
			return;
		}
		QuestionsStats qs = StatisticData.getInstance().getQuestionsStats(controller.getTest().getId());
		if(qs == null){
			questStatHighPanel.add(ChartPanels.getNotAvailableStatPanel());
			return;
		}
		StudentsStats ss = qs.getStudentStats();
		if(ss == null || ss.getNumStudents() == 0){
			questStatHighPanel.add(ChartPanels.getNotAvailableStatPanel());
			return;
		}
		
		Test test = controller.getTest();
		questStatHighPanel.add(getVersionInfoTablePanel(test));		
		questStatLowLeftPane.add(ChartPanels.getQuestionStatGeneralInfo(test));		
		versionStatTable.clearSelection();
		versionStatTable.setRowSelectionInterval(0,0);
		answersStatTable.clearSelection();
		answersStatTable.setRowSelectionInterval(0,0);
	}
	
	public void updateStudentsStatisticsPanel(){
		studStatLowRigthPane.removeAll();
		studStatLowLeftPane.removeAll();
		studStatHighPanel.removeAll();
		
		if(controller.getTest().getStatus().equals("U")){		
			studStatHighPanel.add(ChartPanels.getNotAvailableStatPanel());
			return;
		}		
		QuestionsStats qs = StatisticData.getInstance().getQuestionsStats(controller.getTest().getId());
		if(qs == null){			
			studStatHighPanel.add(ChartPanels.getNotAvailableStatPanel());
			return;
		}
		StudentsStats ss = qs.getStudentStats();
		if(ss == null || ss.getNumStudents() == 0){
			studStatHighPanel.add(ChartPanels.getNotAvailableStatPanel());
			return;
		}
		studStatHighPanel.add(getTestStudentsTableStatisticPanel(controller.getTest()));		
		studStatLowLeftPane.add(ChartPanels.getTestQuestionsPointsPiePanel(controller.getTest()));
		studentsStatTable.clearSelection();
		studentsStatTable.setRowSelectionInterval(0, 0);
	}
	
	private JPanel getTestStudentsTableStatisticPanel(Test test){
		
		JPanel panel = new JPanel(new GridLayout(1,1));
		QuestionsStats qs = StatisticData.getInstance().getQuestionsStats(test.getId());
		if(qs == null)
			return new JPanel();
		StudentsStats ss = qs.getStudentStats();
		
		if(ss == null)
			return new JPanel();
			
		int studentsNum = ss.getNumStudents();
		ArrayList nodesDfsOrdered = new ArrayList();
		ComputeColumnWidthDFS cwDfs = new ComputeColumnWidthDFS(ss, nodesDfsOrdered);
		cwDfs.dfs(TestCategoriesFactory.getInstance().getTestCategoryNode("/",test));
		
		ArrayList tableHeader = new ArrayList();
		tableHeader.add("Student ID");
		tableHeader.add("Grade");
		
		for(Iterator it = nodesDfsOrdered.iterator();it.hasNext();){
			TestCategoryNode node = (TestCategoryNode)it.next();
			String catName = node.getTestCategory().getLongName();
			tableHeader.add(catName);
			if (!node.isLeaf()){
				tableHeader.add(catName + " + sub-categories");
			}	
		}
		

		String[] columnNames = new String[tableHeader.size()];
		tableHeader.toArray(columnNames);

		Object[][] data = new Object[studentsNum][columnNames.length];
		TreeSet studIds = ss.getStudentIds();
		int i = 0;
		for(Iterator it = studIds.iterator();it.hasNext();){
			String studId = (String)it.next();
			data[i][0] = studId;
			data[i][1] = ChartPanels.doubleParser(ss.getStudentGrade(studId));
			
			List studStats = ss.getStudentCategoriesRow(studId,nodesDfsOrdered);
			
			int j = 2;
			for(Iterator it2 = studStats.iterator();it2.hasNext();){
				Double value = (Double)it2.next();
				if(value == null){
					data[i][j] = new String("N/A");
				}else{
					data[i][j] = ChartPanels.doubleParser(value.doubleValue());
				}
				j++;
			}
			i++;
		}				
		studentsStatTable.setModel(new ChartPanels.MyTableModel(data, columnNames));
		studentsStatTable.setBackground(new Color(255,255,255));
		studentsStatTable.getTableHeader().setReorderingAllowed(false);
		studentsStatTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		////////
		studentsStatTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		studentsStatTable.setPreferredScrollableViewportSize(new Dimension(350, 350));

		//studentsStatTable.setColumnSelectionAllowed(false);
		TableColumn column = null;
		for (int j = 0; j < studentsStatTable.getColumnCount(); j++) {
			column = studentsStatTable.getColumnModel().getColumn(j);
			column.setMinWidth(70);
			column.setPreferredWidth(80);
		}
		JScrollPane jScrollPane = new JScrollPane(studentsStatTable);
		panel.add(jScrollPane);
		jScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		return panel;
		
	}
	
	private JPanel getVersionInfoTablePanel(Test test){
		JPanel panel = new JPanel(new GridLayout(1,1));
		
		panel.setBorder(BorderFactory.createCompoundBorder
                (BorderFactory.createTitledBorder("Version Info : "), 
                 BorderFactory.createEmptyBorder(5,5,5,5)));

		QuestionsStats qs;
		qs = StatisticData.getInstance().getQuestionsStats(test.getId());
		if(qs == null)
			return new JPanel();
				
		List versions = test.getTestVersions();			
		String[] columnNames = {"Question Id",
					"Version Id",
					"Version Body",
					"Test Category",
					"Difficulty",
					"Validity ",
					};
		//JTable versionStatTable = new JTable(); 
		Object[][] data = new Object[versions.size()][columnNames.length];
		int i = 0;
		for(Iterator it = versions.iterator();it.hasNext();){
			TestVersion ver = (TestVersion)it.next();
			Integer id = new  Integer(ver.getVersion().getId());
			data[i][0] = new Integer(ver.getVersion().getQuestionId());
			data[i][1] = id;
			data[i][2] = ver.getVersion().getBody();
			data[i][3] = ver.getFullCategoryName();
			Double diff = qs.getDifficulty(id.intValue());
			if(diff == null || diff.isNaN()){
				data[i][4] = "N/A";
			}else{
				data[i][4] = ChartPanels.doubleParser(diff.doubleValue());
			}
				
			Double valid = qs.getValidity(id.intValue());
			if(valid == null || valid.isNaN()){
				data[i][5] = "N/A";
			}else{
				data[i][5] = ChartPanels.doubleParser(valid.doubleValue());
			}
			i++;
		}
		versionStatTable.setModel(new ChartPanels.MyTableModel(data, columnNames));
		versionStatTable.setBackground(new Color(255,255,255));
		versionStatTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		versionStatTable.getTableHeader().setReorderingAllowed(false);

		versionStatTable.getColumnModel().getColumn(0).setPreferredWidth(50);
        versionStatTable.getColumnModel().getColumn(1).setPreferredWidth(50);
        versionStatTable.getColumnModel().getColumn(2).setPreferredWidth(350);
        versionStatTable.getColumnModel().getColumn(3).setPreferredWidth(200);
        versionStatTable.getColumnModel().getColumn(4).setPreferredWidth(50);
        versionStatTable.getColumnModel().getColumn(5).setPreferredWidth(50);

		
		JScrollPane scrollPane = new JScrollPane(versionStatTable);
		panel.add(scrollPane);				
		return panel;
		
	}	
	
	public JPanel getAnswersPanel(int testId,int verId){
		
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.setBorder(BorderFactory.createCompoundBorder
				(BorderFactory.createTitledBorder("Answers Info : "), 
				 BorderFactory.createEmptyBorder(5,5,5,5)));
		
		QuestionsStats qs;
		qs = StatisticData.getInstance().getQuestionsStats(testId);
		if(qs == null)
			return new JPanel();
		HashMap answersHash = qs.getAnswersInfo(verId);	
		
		String[] columnNames = {
				"Ordinal",
				"Correct",
				"N",
				"N(%)",
				"AVG(grades)",
				"CORR"};
		

		Object[][] data = new Object[answersHash.keySet().size()][columnNames.length];
	
		int i = 0;	
		
		TreeSet ts = new TreeSet(answersHash.values());
		for (Iterator it = ts.iterator(); it.hasNext();){
			AnswerInfo aInfo =  (AnswerInfo) it.next();
			
			data[i][0] = new Integer(aInfo.getOrdinal());			
			
			if(aInfo.getIsCorrect()){
				data[i][1] = "yes";
			}else{
				data[i][1] = "no";
			}
			data[i][2] = new Integer(aInfo.getNumAnsweringStudents());
			data[i][3] = ChartPanels.doubleParser(aInfo.getPercentAnsweringStudents().doubleValue());
			
			Double norm = aInfo.getNormGradeAvg();
			if(norm == null || norm.isNaN()){
				data[i][4] = "N/A";
			}else{
				data[i][4] = ChartPanels.doubleParser(norm.doubleValue());
			}
			
			Double corr = aInfo.getCorelation();
			if(corr == null || corr.isNaN()){
				data[i][5] = "N/A";
			}else{
				data[i][5] = ChartPanels.doubleParser(corr.doubleValue());
			}
			
			i++;
		}
		answersStatTable.setModel(new ChartPanels.MyTableModel(data, columnNames));
		answersStatTable.setBackground(new Color(255,255,255));
		answersStatTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		answersStatTable.getTableHeader().setReorderingAllowed(false);
		JScrollPane scrollPane = new JScrollPane(answersStatTable);
		panel.add(scrollPane);
		
		JLabel comment1 = new JLabel(" *  N - num. of students that chose the answer.");
		Font font = new Font("DIALOG", Font.ITALIC, 12);
		comment1.setFont(font);
		JLabel comment2 = new JLabel("    N(%) - num. of students that chose the answer out of students that took the test.");
		comment2.setFont(font);
		JLabel comment3 = new JLabel("    AVG(grades) - norm.grades average of students that chose the answer.");
		comment3.setFont(font);
		JLabel comment4 = new JLabel("    CORR - correlation between the answer choosing and the test success");
		comment4.setFont(font);
		
		panel.add(comment1);
		panel.add(comment2);
		panel.add(comment3);
		panel.add(comment4);
						
		return panel;
	}
	
	public static JPanel getAnswerTextPanel(Answer ans){
			JPanel panel = new JPanel(new GridLayout(1,1));
			panel.setBorder(BorderFactory.createCompoundBorder
					(BorderFactory.createTitledBorder("Answer text :"), 
					 BorderFactory.createEmptyBorder(5,5,5,5)));		
			JTextArea ta = new JTextArea(ans.getText());
			ta.setRows(4);
			ta.setEditable(false);
			ta.setBackground(new Color(255,255,255));
			if(GeneralParams.isRtlLanguage(ans.getLanguage())){
				ta.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
			}else{
				ta.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
			}
			panel.add(new JScrollPane(ta));
		
			return panel;
		}
		
	
	private void addListeners(){
		ListSelectionModel rowSMStudents = studentsStatTable.getSelectionModel();
		ListSelectionModel rowSMQuestions  = versionStatTable.getSelectionModel();
		ListSelectionModel rowSMAnswers  = answersStatTable.getSelectionModel();
		
		rowSMStudents.addListSelectionListener(new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e) {
		        //Ignore extra messages.
		        if (e.getValueIsAdjusting()) return;

		        ListSelectionModel lsm = (ListSelectionModel)e.getSource();
		        if(studentsStatTable.getModel().getRowCount() == 0){
		        	return;
		        }
		        if (lsm.isSelectionEmpty()) {
		        	studentsStatTable.clearSelection();
		        	studentsStatTable.setRowSelectionInterval(0, 0);
		        	return;
		        }
		        int selectedRow = lsm.getMinSelectionIndex(); 
		        String tmp = (String)studentsStatTable.getValueAt(selectedRow,0);	            			        
		        studStatLowRigthPane.removeAll();
				studStatLowRigthPane.add(ChartPanels.getTestStudentBarChartPanel(controller.getTest(),tmp));
				studLeftRightSplitPane.getRightComponent().setVisible(false);
				studLeftRightSplitPane.getRightComponent().setVisible(true);
		    }
			
		});
		
		rowSMQuestions.addListSelectionListener(new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e) {
		        //Ignore extra messages.
		        if (e.getValueIsAdjusting()) return;

		        ListSelectionModel lsm =(ListSelectionModel)e.getSource();
		        if(versionStatTable.getModel().getRowCount() == 0){
		        	return;
		        }
		        if (lsm.isSelectionEmpty()) {
		        	versionStatTable.clearSelection();
		        	versionStatTable.setRowSelectionInterval(0, 0);
		        	return;
		        } 
		        int selectedRow = lsm.getMinSelectionIndex(); 
				versionSelected = (Integer)versionStatTable.getValueAt(selectedRow,1);
	           	int testId = controller.getTest().getId();
	           	
	           	questStatLowRigthPane.removeAll();
	           	questStatLowRigthPane.add(ChartPanels.getVersionHeuristicPanel(testId,versionSelected.intValue()));	
	           	questStatLowRigthPane.add(getAnswersPanel(testId,versionSelected.intValue()));
				questStatLowRigthPane.add(answerStatPanel);
	           	
	           	questLeftRightSplitPane.getRightComponent().setVisible(false);
	           	questLeftRightSplitPane.getRightComponent().setVisible(true);
		    }
		});
		
		rowSMAnswers.addListSelectionListener(new ListSelectionListener() {
					public void valueChanged(ListSelectionEvent e) {
						//Ignore extra messages.
						if (e.getValueIsAdjusting()) return;

						ListSelectionModel lsm =(ListSelectionModel)e.getSource();
						if(answersStatTable.getModel().getRowCount() == 0){
							return;
						}
						if (lsm.isSelectionEmpty()) {
							answersStatTable.clearSelection();
							answersStatTable.setRowSelectionInterval(0, 0);
							return;
						} 
						int selectedRow = lsm.getMinSelectionIndex(); 
						Integer selectedOrdinal = (Integer)answersStatTable.getValueAt(selectedRow,0);
						
						// find answer
						Version version;
						try {
							version =
								VersionsFactory.getInstance().getVersion(versionSelected.intValue());
						
							List answersList = version.getAnswers();
						
							Answer ans = null;
						
							for (Iterator it = answersList.iterator(); it.hasNext();){
								Answer curr = (Answer)it.next();
								if(curr.getOrdinal() == selectedOrdinal.intValue()){
									ans = curr;
								}
							}
						
							answerStatPanel.removeAll();
							answerStatPanel.add(getAnswerTextPanel(ans));
												
							questStatLowRigthPane.remove(answerStatPanel);
							questStatLowRigthPane.add(answerStatPanel);
							
							questLeftRightSplitPane.getRightComponent().setVisible(false);
							questLeftRightSplitPane.getRightComponent().setVisible(true);
							
						} catch (Throwable e1) {
													//e1.printStackTrace();
						} 
												
					}
				});
				

	}
}
