/*
 * Created on 17/04/2004
 *
 * Created by: svali
 *  */
package il.ac.technion.cs.course236308.e3.med.model;

import java.io.IOException;
import java.io.StringReader;

import javax.swing.ImageIcon;

import il.ac.technion.cs.course236308.e3.med.command.CommandDispatcher;
import il.ac.technion.cs.course236308.e3.med.db.DBConnect;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.parameters.GeneralParams;
import il.ac.technion.cs.course236308.e3.med.xmlTools.FindAnsRefs;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.xml.sax.InputSource;


/**
 * @author svali
 *
 */
public class Answer implements Comparable,LegalObject{
	private RetriableException isSelfLegal = null; 
	protected int id;
	protected Double avgPercentage;
	protected Double avgGrade;
	protected Double avgCorrelation;
	protected String notes;
	protected int versionId;
	protected boolean mixable = true;
	protected byte ordinal;
	protected String text;
	protected String language;
	
	public Object clone(){
		Answer newAns = new Answer();
		newAns.id = getId();
		newAns.avgPercentage = avgPercentage==null?null:new Double(avgPercentage.doubleValue());
		newAns.avgGrade = avgGrade==null?null:new Double(avgGrade.doubleValue());
		newAns.avgCorrelation = avgCorrelation==null?null:new Double(avgCorrelation.doubleValue());
		newAns.notes = notes==null?null:new String(notes);
		newAns.versionId = versionId;
		newAns.mixable = mixable;
		newAns.ordinal = ordinal;
		newAns.text = text==null?null:new String(text);
		newAns.language = language==null?null:new String(language);
		return newAns;
	}
	
	/**
	 * Method getAvgPercentage
	 * @return Double
	 */
	public Double getAvgPercentage() {
		return avgPercentage;
	}

	/**
	 * Method setAvgPercentage
	 * @param value
	 */
	public void setAvgPercentage(Double value) {
		avgPercentage = value;
	}

	/**
	 * Method getAvgGrade
	 * @return Double
	 */
	public Double getAvgGrade() {
		return avgGrade;
	}

	/**
	 * Method setAvgGrade
	 * @param value
	 */
	public void setAvgGrade(Double value) {
		avgGrade = value;
	}

	/**
	 * Method getAvgCorrelation
	 * @return Double
	 */
	public Double getAvgCorrelation() {
		return avgCorrelation;
	}

	/**
	 * Method setAvgCorrelation
	 * @param value
	 */
	public void setAvgCorrelation(Double value) {
		avgCorrelation = value;
	}

	/**
	 * Method getNotes
	 * @return String
	 */
	public String getNotes() {
		return notes;
	}

	/**
	 * Method setNotes
	 * @param value
	 */
	public void setNotes(String value) {
		notes = value;
	}

	/**
	 * Method getVersionId
	 * @return int
	 */
	public int getVersionId() {
		return versionId;
	}

	/**
	 * Method setVersionId
	 * @param value
	 */
	public void setVersionId(int value) {
		versionId = value;
	}

	/**
	 * Method isMixable
	 * @return boolean
	 */
	public boolean isMixable() {
		return mixable;
	}

	/**
	 * Method setMixable
	 * @param value
	 */
	public void setMixable(boolean value) {
		mixable = value;
	}

	/**
	 * Method getOrdinal
	 * @return byte
	 */
	public byte getOrdinal() {
		return ordinal;
	}

	/**
	 * Method setOrdinal
	 * @param value
	 */
	public void setOrdinal(byte value) {
		ordinal = value;
	}

	/**
	 * Method getText
	 * @return String
	 */
	public String getText() {
		return text;
	}

	/**
	 * Method setText
	 * @param value
	 */
	public void setText(String value) {
		text = value;
	}

	/**
	 * Method getLanguage
	 * @return String
	 */
	public String getLanguage() {
		return language;
	}

	/**
	 * Method setLanguage
	 * @param value
	 */
	public void setLanguage(String value) {
		language = value;
	}

	/**
	 * Method getId
	 * @return int
	 */
	public int getId() {
		return id;
	}

	/**
	 * Method setId
	 * @param value
	 */
	public void setId(int value) {
		id = value;
	}
	
	
	/**
	 * 
	 * @return how many DB records were effected by this command 
	 * @throws CriticalException
	 */
	
	public int updateDB() throws CriticalException{
		return DBConnect.updateAnswer(this);
	}
	
	/**
	 * Method hashCode
	 * @see Object#hashCode()
	 */
	public int hashCode(){
		return getId();
	}
	
	/**
	 * Method equals
	 * @see Object#equals(Object)
	 */
	public boolean equals(Object obj){
		if(!(obj instanceof Answer)){
			return false;
		}
		return ((Answer)obj).getId() == getId();
	}

	/**
	 * Method compareTo
	 * @see Comparable#compareTo(Object)
	 */
	public int compareTo(Object obj) {
		Answer ans = (Answer)obj;
		return (new Integer(getOrdinal())).
				compareTo(new Integer(ans.getOrdinal()));	
	}
	
	/**
	 * Method toElement
	 * @return Element
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public Element toElement() throws RetriableException, CriticalException{
		Element answer = new Element("answer");
		answer.setAttribute("mixable",String.valueOf(isMixable()));
		Version ver = VersionsFactory.getInstance().getVersion(getVersionId());
		answer.setAttribute
			("correct",String.valueOf(ver.getCorrectAnswerId()==getId()));
		answer.setAttribute("ordinal",String.valueOf(getOrdinal()));
		
		InputSource source =
		 new InputSource(new StringReader("<text>"+getText()+"</text>"));
		Document doc;
		try {
			SAXBuilder builder = GeneralParams.saxBuilder;
			doc = builder.build(source);
		} catch (JDOMException e1) {
			throw 
				new RetriableException("Converting text to XML element failed");			
		} catch (IOException e1) {
			throw 
				new RetriableException("Converting text to XML element failed");
		}
		Element mytext = (Element) doc.getRootElement().detach();
		mytext.setAttribute("lang",getLanguage());
		answer.addContent(mytext);
		return answer;
	}

	public ImageIcon getIcon() {
		try{
			isLegal();
			return GeneralParams.ICON_ANSWER;	
		}catch(RetriableException e){
			return GeneralParams.ICON_ANSWER_ILLEGAL;
		}
	}

	public boolean isLegal() throws RetriableException {
		return isSelfLegal();
	}

	public boolean isSelfLegal() throws RetriableException {
		if(isSelfLegal != null){
			throw isSelfLegal;
		}
		return true;
	}

	public void recalculateLegality() throws CriticalException {
		recalculateSelfLegality();
	}

	public void recalculateSelfLegality() throws CriticalException {
		isSelfLegal = null;
		String[] ansRefs;
		int ansCount;
		try {
			ansRefs = FindAnsRefs.findAnsRefs(getText());
			ansCount = VersionsFactory.getInstance().getVersion(getVersionId()).getAnswers().size();
		} catch (Throwable e) {
			return;
		}
		try{
			for(int i=0;i<ansRefs.length;++i){
				if(Integer.parseInt(ansRefs[i]) < 1 || Integer.parseInt(ansRefs[i]) > ansCount){
					CommandDispatcher.getIllegalObjects().add(this);
					isSelfLegal = new RetriableException("Reference to non-existent answer");			
				}
			}
		}catch(NumberFormatException nfe){
			CommandDispatcher.getIllegalObjects().add(this);
			isSelfLegal = new RetriableException("Illegal reference to answer");
		}
		if(isSelfLegal == null){
			CommandDispatcher.getInstance().removeIllegalObject(this);
		}
	}
	
	public String toString(){
		return "Answer " + getOrdinal() + " of Version " + getVersionId();
	}
	
}
