/*
 * Created on 17/04/2004
 *
 * Created by: svali
 *  */
package il.ac.technion.cs.course236308.e3.med.model;

import java.util.HashMap;

import il.ac.technion.cs.course236308.e3.med.db.DBConnect;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.MedException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
/**
 * @author svali
 *
 */
public class AnswersFactory{
	
	private static AnswersFactory instance = new AnswersFactory();
	private HashMap answers_ = new HashMap();
	private int maxAnsId = 0;
	
	private AnswersFactory(){}
	
	/**
	 * Method getInstance
	 * @return AnswersFactory
	 */
	public static synchronized AnswersFactory getInstance(){
		return instance; 
	}
	
	/**
	 * Method getAnswer
	 * @param id
	 * @return Answer
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Answer getAnswer(int id) 
		throws RetriableException,CriticalException {
		Answer ans = null;
		if((ans =(Answer)answers_.get(new Integer(id))) != null){
			return ans;
		}
		ans = DBConnect.getAnswer(id);
		answers_.put(new Integer(id),ans);	
		if(ans.getId()>maxAnsId){
			maxAnsId = ans.getId();		
		}
		return ans;	
	}
	
	public int getMaxAnsId(){
		return maxAnsId;
	}
	
	public void resetMaxId(){
		maxAnsId = 0;
	}
	
	private Answer createEmptyAnswer(Version ver){
		++maxAnsId;
		Answer ans = new Answer();
		ans.setId(maxAnsId);
		ans.setVersionId(ver.getId());
		ans.setText("text was not set yet");
		ans.setLanguage(ver.getLanguage());
		ans.setOrdinal((byte)(ver.getAnswers().size()+1));	
		ans.setMixable(true);
		return ans;
	}
	
	/**
	 * Method createAnswer
	 * @param ver
	 * @return Answer
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Answer createAnswer(Version ver) 
		throws CriticalException{
		Answer ans =  createEmptyAnswer(ver);		
		storeAnswer(ver,ans);
		return ans;
	}

	/**
	 * Method cloneAnswer
	 * @param ver
	 * @param ans
	 * @return Answer
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Answer cloneAnswer(Version ver,Answer ans) 
		throws CriticalException{
		Answer newAns =  createEmptyAnswer(ver);
		newAns.setLanguage(ans.getLanguage());
		newAns.setMixable(ans.isMixable());
		newAns.setNotes(ans.getNotes());
		newAns.setText(ans.getText());
		storeAnswer(ver,newAns);
		return newAns;
	}


	/**
	 * Method storeAnswer
	 * @param ver
	 * @param ans
	 * @return int
	 * @throws CriticalException
	 */
	public synchronized int storeAnswer(Version ver, Answer ans) 
		throws CriticalException{
		int affectedRows = DBConnect.storeAnswer(ver,ans);
		if(ans.getId()>maxAnsId){
			maxAnsId = ans.getId();
		}
		answers_.put(new Integer(ans.getId()),ans);
		if(!ver.getAnswers().contains(ans)){
			ver.getAnswers().add(ans);
		}
		return affectedRows;
	}

	/**
	 * 
	 * @param answer
	 * @return number of deleted rows 
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized int deleteAnswer(Answer answer) 
		throws RetriableException,CriticalException {
		
		if(answer.getId()==maxAnsId){
			--maxAnsId;
		}
		VersionsFactory.getInstance().
			getVersion(answer.getVersionId()).getAnswers().remove(answer);
		answers_.remove(new Integer(answer.getId()));
		return DBConnect.deleteAnswer(answer.getId());
	}
	
	/**
	 * Method exists
	 * @param ans
	 * @return boolean
	 */
	public synchronized boolean exists(Answer ans){
		return exists(ans.getId());
	}
	
	/**
	 * Method exists
	 * @param ansId
	 * @return boolean
	 */
	public synchronized boolean exists(int ansId){
		try{
			getAnswer(ansId);
			return true;
		}catch(MedException e){
			return false;
		}
	}

	
}
