/*
 * Created on 17/04/2004
 *
 * Created by: svali
 *  */
package il.ac.technion.cs.course236308.e3.med.model;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;


import il.ac.technion.cs.course236308.e3.med.db.DBConnect;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.MedException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.gui.treenodes.CategoryNode;

/**
 * @author svali
 *
 */
public class CategoriesFactory{	
	private static CategoriesFactory instance = new CategoriesFactory();
	private HashMap categories_ = new HashMap();
	
	private CategoriesFactory(){}
	
	private int maxCategoryId = 0;
	private int rootCategoryId = -1;
	/**
	 * Method getInstance
	 * @return CategoriesFactory
	 */
	public static synchronized CategoriesFactory getInstance(){
		return instance; 
	}
	
	public int getMaxCategoryId(){
		return maxCategoryId;
	}
	
	public void resetMaxId(){
		maxCategoryId = 1;
	}
	
	/**
	 * Method getRootCategory
	 * @return Category
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Category getRootCategory() 
				throws RetriableException,CriticalException{
		if(rootCategoryId == -1){
			rootCategoryId = DBConnect.getRootCategoryId();
		}
		return getCategory(rootCategoryId);
	}
	
	/**
	 * Method getCategory
	 * @param id
	 * @return Category
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Category getCategory(int id) 
		throws RetriableException,CriticalException {
		Category category = null;
		if((category =(Category)categories_.get(new Integer(id))) != null){
			return category;
		}
		category = DBConnect.getCategory(id);
		if(category.getId() > maxCategoryId){
			maxCategoryId = category.getId();
		}
		categories_.put(new Integer(id),category);	
		for(Iterator it=DBConnect.getQuestionIdsOfCategory(category).iterator();
				it.hasNext();){
			int questId = ((Integer)it.next()).intValue();
			category.getQuestions().
				add(QuestionsFactory.getInstance().getQuestion(questId));		
		}
		for(Iterator it=DBConnect.getCategoryIdsOfCategory(category).iterator();
				it.hasNext();){
			int catId = ((Integer)it.next()).intValue();
			category.getCategories().
				add(CategoriesFactory.getInstance().getCategory(catId));		
		}	
		for(Iterator it=DBConnect.getLinksToQuestionsOfCategory(category)
				.iterator();it.hasNext();){
			Integer linkId = (Integer)it.next();
			category.getLinksToQuestionIds().add(linkId);
		}
		return category;	
	}
	
	private Category createEmptyCategory(Category parent,String name){
		++maxCategoryId;
		Category cat = new Category();
		cat.setId(maxCategoryId);
		cat.setName(name);
		cat.setOrdinal(parent.getCategories().size() + 1);
		cat.setParentID(new Integer(parent.getId()));		
		return cat;
	}
	
	/**
	 * Method createChildCategoryByName
	 * @param parent
	 * @param childName
	 * @return
	 * @throws RetriableException
	 * @throws CriticalException
	 */
	public Category createChildCategoryByName(Category parent,String childName)
	 throws CriticalException{
		Category child = parent.getChild(childName); 
		if(child != null){
			return child;
		}
		return createCategory(parent,childName);
	}
	
	/**
	 * 
	 * @param fullName
	 * @return
	 * @throws RetriableException
	 * @throws CriticalException
	 */	
	public synchronized int createCategoriesFromFullName(String fullName)
		 throws RetriableException, CriticalException{
		if (!fullName.startsWith("/")) {
			throw new RetriableException("Category name \"" + fullName + "\"" +
				" is illegal - doesn't start with \'/\'");
		}
		if (fullName.endsWith("/") && !fullName.equals("/")){
				fullName=fullName.substring(0,fullName.lastIndexOf("/"));	
		}
		Category parent = CategoriesFactory.getInstance().getRootCategory();
		String[] catNames = fullName.split("/");
		for(int i=0;i<catNames.length;++i){
			if(catNames[i].equals("")){
				continue;
			}
			parent = CategoriesFactory.getInstance().
			createChildCategoryByName(parent,catNames[i]);
		}
		return parent.getId();
	}
	
	/**
	 * Method createCategory
	 * @param parentCategory
	 * @param name
	 * @return Category
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Category createCategory(Category parentCategory,
		String name) throws CriticalException{
		Category cat = createEmptyCategory(parentCategory,parentCategory.newSubCatUniqueName(name)); 
		storeCategory(parentCategory,cat);
		return cat;
	}
	
	/**
	 * Method cloneCategory
	 * @param cat
	 * @param destCategory
	 * @return Category
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Category cloneCategory(Category cat,
		Category destCategory) throws CriticalException{
		String newCategoryName = destCategory.
									newSubCatUniqueName(cat.getName());
		Category newCategory = createEmptyCategory(destCategory,newCategoryName);
		newCategory.setComment(cat.getComment());
		storeCategory(destCategory,newCategory);
		return newCategory;
	}
	
	/**
	 * Method storeCategory
	 * @param parentCategory
	 * @param cat
	 * @return int
	 * @throws CriticalException
	 */
	public synchronized int storeCategory(Category parentCategory,Category cat)
				 throws CriticalException{
		int affectedRows = DBConnect.storeCategory(parentCategory,cat);
		if(cat.getId() > maxCategoryId){
			maxCategoryId = cat.getId();
		}
		categories_.put(new Integer(cat.getId()),cat);
		parentCategory.getCategories().add(cat);
		return affectedRows;
	}

	/**
	 * 
	 * @param cat
	 * @return number of deleted rows 
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized int deleteCategory(Category cat) 
		throws RetriableException,CriticalException {
		CategoriesFactory.getInstance().getCategory(
					cat.getParentID().intValue()).getCategories().remove(cat);
		if(cat.getId() == maxCategoryId){
			--maxCategoryId;
		}
		categories_.remove(new Integer(cat.getId()));
		return DBConnect.deleteCategory(cat.getId());
	}
	
	/**
	 * Method exists
	 * @param cat
	 * @return boolean
	 */
	public synchronized boolean exists(Category cat){
		return exists(cat.getId());
	}
	
	/**
	 * Method exists
	 * @param catId
	 * @return boolean
	 */
	public synchronized boolean exists(int catId){
		try{
			getCategory(catId);
			return true;
		}catch(MedException e){
			return false;
		}
	}	
	
	public CategoryNode buildTree(Category cat){
		Collections.sort(cat.getCategories());
		CategoryNode node = new CategoryNode(cat);
		for(Iterator it=cat.getCategories().iterator();it.hasNext();){
			Category subCat = (Category)it.next();
			node.add(buildTree(subCat));
		}
		return node;
	}
}
