/*
 * Created on 17/04/2004
 *
 * Created by: svali
 *  */
package il.ac.technion.cs.course236308.e3.med.model;

import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import il.ac.technion.cs.course236308.e3.med.db.DBConnect;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.MedException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;

/**
 * @author svali
 *
 */
public class QuestionsFactory{	
	private static QuestionsFactory instance = new QuestionsFactory();
	private HashMap questions_ = new HashMap();
	private int maxQuestionId = 0;
	
	private QuestionsFactory(){}
	
	/**
	 * Method getInstance
	 * @return QuestionsFactory
	 */
	public static synchronized QuestionsFactory getInstance(){
		return instance; 
	}
	
	public int getMaxQuestionId(){
		return maxQuestionId;
	}
	
	public void resetMaxId(){
		maxQuestionId = 0;
	}
	
	/**
	 * Method getQuestion
	 * @param id
	 * @return Question
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Question getQuestion(int id) 
		throws RetriableException,CriticalException {
		Question quest = null;
		if((quest =(Question)questions_.get(new Integer(id))) != null){
			return quest;
		}
		quest = DBConnect.getQuestion(id);
		if(quest.getId() > maxQuestionId){
			maxQuestionId = quest.getId();
		}
		questions_.put(new Integer(id),quest);			
		for(Iterator it=DBConnect.getVersionIdsOfQuestion(quest).iterator();
				it.hasNext();){
			int verId = ((Integer)it.next()).intValue();
			quest.getVersions().
				add(VersionsFactory.getInstance().getVersion(verId));		
		}
		for(Iterator it=DBConnect.getCategoryIdsOfQuestion(quest).iterator();
				it.hasNext();){
			Integer catId = (Integer)it.next();
			quest.getSecondaryCategoryIds().add(catId);
		}
		return quest;	
	}
	
	public Question createEmptyQuestion(Category cat){
		++maxQuestionId;
		Question quest = new Question();
		quest.setId(maxQuestionId);
		quest.setMainCategoryId(cat.getId());
		quest.setOrdinal(cat.getQuestions().size()+1);
		return quest;
	}
	
	/**
	 * Method createQuestion
	 * @param cat
	 * @return Question
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Question createQuestion(Category cat) 
		throws CriticalException{
		Question quest = createEmptyQuestion(cat);  
		storeQuestion(cat,quest);
		return quest;
	}

	/**
	 * Method storeQuestion
	 * @param cat
	 * @param quest
	 * @return int
	 * @throws CriticalException
	 */
	public synchronized int storeQuestion(Category cat,Question quest)
		 throws CriticalException{
		int affectedRows = DBConnect.storeQuestion(cat,quest);
		if(quest.getId() > maxQuestionId){
			maxQuestionId = quest.getId();
		}
		questions_.put(new Integer(quest.getId()),quest);
		if(!cat.getQuestions().contains(quest)){
			cat.getQuestions().add(quest);
		}
		return affectedRows;
	}
	
	/**
	 * Method cloneQuestion
	 * @param destCategory
	 * @param quest
	 * @return Question
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Question cloneQuestion(Category destCategory,Question quest) throws CriticalException{
		Question newQuestion = createEmptyQuestion(destCategory);
		newQuestion.setActiveVersionId(quest.getActiveVersionId());
		newQuestion.setCancelled(quest.isCancelled());
		storeQuestion(destCategory,newQuestion);
		return newQuestion;
	}
	
	/**
	 * 
	 * @param quest
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized int deleteQuestion(Question quest)
		 throws RetriableException,CriticalException {
		if(quest.getId() == maxQuestionId){
			maxQuestionId--;
		}
		CategoriesFactory.getInstance().
			getCategory(quest.getMainCategoryId()).getQuestions().remove(quest);
		questions_.remove(new Integer(quest.getId()));
		return DBConnect.deleteQuestion(quest.getId());
	}
	
	/**
	 * Method importQuestions
	 * @param questions
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public void importQuestions(List questions)
		 throws RetriableException, CriticalException{
		int maxQuestId = QuestionsFactory.getInstance().getMaxQuestionId();
		int maxVerId = VersionsFactory.getInstance().getMaxVersionId();
		int maxAnsId = AnswersFactory.getInstance().getMaxAnsId();
		for(Iterator it = questions.iterator();it.hasNext();){
			Question quest = (Question)it.next();
			quest.setId(maxQuestId + quest.getId());
			Category cat =
			 CategoriesFactory.getInstance().
			 	getCategory(quest.getMainCategoryId());
			quest.setOrdinal(cat.getQuestions().size() + 1);
			if(quest.getActiveVersionId()!=null ){
				quest.
				setActiveVersionId(new 
					Integer(maxVerId+quest.getActiveVersionId().intValue()));
			}
			for(Iterator it2 = quest.getVersions().iterator();it2.hasNext();){
				Version ver = (Version)it2.next();
				ver.setId(maxVerId + ver.getId());
				ver.setQuestionId(quest.getId());
				ver.setCreated(new GregorianCalendar());
				ver.setUpdated(new GregorianCalendar());
				if(ver.getBody().equals("")){
					ver.setBody(null);
				}
				if(ver.getCorrectAnswerId()!=0){
					ver.setCorrectAnswerId(maxAnsId+ver.getCorrectAnswerId());
				}
				for(Iterator it3 = ver.getAnswers().iterator();it3.hasNext();){
					Answer ans = (Answer)it3.next();
					ans.setId(maxAnsId + ans.getId());
					ans.setVersionId(ver.getId());
					AnswersFactory.getInstance().storeAnswer(ver,ans);
				}
				maxAnsId+= ver.getAnswers().size();
				VersionsFactory.getInstance().storeVersion(quest,ver);
			}
			maxVerId+=quest.getVersions().size();
			QuestionsFactory.getInstance().storeQuestion(cat,quest);
		}
	}
	
	/**
	 * Method exists
	 * @param quest
	 * @return boolean
	 */
	public synchronized boolean exists(Question quest){
		return exists(quest.getId());
	}
	
	/**
	 * Method exists
	 * @param questId
	 * @return boolean
	 */
	public synchronized boolean exists(int questId){
		try{
			getQuestion(questId);
			return true;
		}catch(MedException e){
			return false;
		}
	}		
}
