/*
 * Created on 17/04/2004
 *
 * Created by: svali
 *  */
package il.ac.technion.cs.course236308.e3.med.model;

import il.ac.technion.cs.course236308.e3.med.db.DBConnect;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.MedException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * @author svali
 *
 */
public class TestAnswersFactory{	
	private static TestAnswersFactory instance = new TestAnswersFactory();
	private HashMap testAnswers_ = new HashMap();
	
	private TestAnswersFactory(){}
	
	/**
	 * Method getInstance
	 * @return TestAnswersFactory
	 */
	public static synchronized TestAnswersFactory getInstance(){
		return instance; 
	}
	
	/**
	 * Method getTestAnswer
	 * @param testId
	 * @param answerId
	 * @return TestAnswer
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized TestAnswer getTestAnswer(int testId,int answerId)
			 throws RetriableException,CriticalException {
		TestAnswer testAnswer = null;
		if((testAnswer =(TestAnswer)testAnswers_.get(new Key(testId,answerId))) 
					!= null){
			return testAnswer;
		}
		testAnswer = DBConnect.getTestAnswer(testId,answerId);
		try{
			AnswersFactory.getInstance().getAnswer(answerId);
		}catch(RetriableException e2){ throw new 
			RetriableException("Referential integrity violation -" +
				" record in table ExamAnswer with AnswerID=" 
				+answerId + " without such answer in Answer Table");
		}
		testAnswers_.put(new Key(testId,answerId),testAnswer);		
		return testAnswer;	
	}
	
	public synchronized List getTestAnswers(int answerId){
		List list = new LinkedList();
		for(Iterator it=testAnswers_.values().iterator();it.hasNext();){
			TestAnswer ta = (TestAnswer)it.next();
			if(ta.getAnswerId() == answerId){
				list.add(ta);
			}
		}
		return list;
	}
	
	/**
	 * Method createTestAnswer
	 * @param test
	 * @param testVersion
	 * @param answer
	 * @return TestAnswer
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized TestAnswer createTestAnswer(Test test,Answer answer) 
		 throws RetriableException, CriticalException{
		final int max_weight = 100;
		int correctAnsId = VersionsFactory.getInstance().getVersion(answer.getVersionId()).getCorrectAnswerId();
		byte weight = (byte) (correctAnsId==answer.getId()?max_weight:0);
		TestAnswer ta = new TestAnswer();
		ta.setAnswerId(answer.getId());
		ta.setTestId(test.getId());
		ta.setWeight(weight);
		storeTestAnswer(ta);
		return ta;
	}

	/**
	 * Method cloneTestAnswer
	 * @param newTestVersion
	 * @param testAnswer
	 * @return TestAnswer
	 * @throws CriticalException
	 * @throws RetriableException
	 * @throws RetriableException
	 */
	public synchronized TestAnswer cloneTestAnswer(TestVersion newTestVersion,
		TestAnswer testAnswer) throws  RetriableException, CriticalException{
		TestAnswer newTestAnswer = new TestAnswer(); 
		newTestAnswer.setAnswerId(testAnswer.getAnswerId());
		newTestAnswer.setTestId(newTestVersion.getTestId());
		newTestAnswer.setWeight(testAnswer.getWeight());
		storeTestAnswer(newTestAnswer);
		return newTestAnswer;
	}
	
	/**
	 * Method storeTestAnswer
	 * @param testAnswer
	 * @return int
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized int storeTestAnswer(TestAnswer testAnswer) 
					throws RetriableException,CriticalException{
		int affectedRows;
		affectedRows = DBConnect.storeTestAnswer(testAnswer);
		testAnswers_.put(new Key(testAnswer.getTestId(),
						testAnswer.getAnswerId()),testAnswer);
		int versionId = AnswersFactory.getInstance().
							getAnswer(testAnswer.getAnswerId()).getVersionId();
		TestVersionsFactory.getInstance().
			getTestVersion(testAnswer.getTestId(),versionId).getTestAnswers().
					add(testAnswer);
		return affectedRows;
	}
	
	/**
	 * Method deleteTestAnswer
	 * @param testAnswer
	 * @return int
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized int deleteTestAnswer(TestAnswer testAnswer)
		 throws RetriableException,CriticalException {
		testAnswers_.remove(new Key(testAnswer.getTestId(),
								testAnswer.getAnswerId()));
		int versionId = AnswersFactory.getInstance().
			getAnswer(testAnswer.getAnswerId()).getVersionId();		
		TestVersionsFactory.getInstance().
			getTestVersion(testAnswer.getTestId(),versionId).getTestAnswers().
				remove(testAnswer);
		return DBConnect.deleteTestAnswer(testAnswer);
	}
	
	/**
	 * Method exists
	 * @param testAns
	 * @return boolean
	 */
	public synchronized boolean exists(TestAnswer testAns){
		return exists(testAns.getTestId(),testAns.getAnswerId());
	}
	
	/**
	 * Method exists
	 * @param testId
	 * @param answerId
	 * @return boolean
	 */
	public synchronized boolean exists(int testId,int answerId){
		try{
			getTestAnswer(testId,answerId);
			return true;
		}catch(MedException e){
			return false;
		}
	}
	
	private class Key{
		int testId;
		int answerId;
		public Key(int testId,int answerId){
			this.testId = testId;
			this.answerId = answerId;
		}
		
		public boolean equals(Object obj) {
			if(!(obj instanceof Key)){
				return false;
			}
			Key key = (Key)obj;
			return key.testId == testId && key.answerId == answerId;
		}
		
		public int hashCode() {
			return (String.valueOf(testId) + String.valueOf(answerId)).hashCode(); 
		}

	}
}
