/*
 * Created on 17/04/2004
 *
 * Created by: svali
 *  */
package il.ac.technion.cs.course236308.e3.med.model;

import il.ac.technion.cs.course236308.e3.med.db.DBConnect;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.MedException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.parameters.GeneralParams;
import il.ac.technion.cs.course236308.e3.med.stats.StatisticData;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * @author svali
 *
 */
public class TestRepliesFactory{
	
	private static TestRepliesFactory instance = new TestRepliesFactory();
	private HashMap testReplies_ = new HashMap();
	
	private TestRepliesFactory(){}
		
	/**
	 * Method getInstance
	 * @return TestRepliesFactory
	 */
	public static synchronized TestRepliesFactory getInstance(){
		return instance; 
	}
	
	
	public synchronized void loadAllTestReplies() throws CriticalException{
		for(Iterator it=DBConnect.getAllTestReplies().iterator();it.hasNext();){
			TestReply tr = (TestReply)it.next();
			testReplies_.put(new Key(tr.getTestTakerId(),tr.getTestId(),tr.getAnswerId()),tr);
		}
	}
	/**
	 * Method getTestReply
	 * @param testTakerId
	 * @param testId
	 * @param answerId
	 * @return TestReply
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized TestReply getTestReply(String testTakerId,int testId,
					int answerId) throws RetriableException,CriticalException {
		TestReply testReply = null;
		if((testReply =(TestReply)testReplies_.
				get(new Key(testTakerId,testId,answerId))) != null){
			return testReply;
		}
		testReply = DBConnect.getTestReply(testTakerId,testId,answerId);
		testReplies_.put(new Key(testTakerId,testId,answerId),testReply);			
		return testReply;	
	}
	
	public synchronized List getTestRepliesList(String testTakerId,int testId){
		List list = new LinkedList();
		for(Iterator it=testReplies_.values().iterator();it.hasNext();){
			TestReply tr = (TestReply)it.next();
			if(testId == tr.getTestId() && GeneralParams.equalNullableStrings(testTakerId,tr.getTestTakerId())){
				list.add(tr);
			}
		}
		return list;
	}
    
    public synchronized List getTestRepliesList(int testId){
		List list = new LinkedList();
		for(Iterator it=testReplies_.values().iterator();it.hasNext();){
			TestReply tr = (TestReply)it.next();
			if(testId == tr.getTestId() ){
				list.add(tr);
			}
		}
		return list;
    }
    
    public synchronized List getTestRepliesByVersion(String testTakerId, int versionId,int testId) throws RetriableException, CriticalException{
		List list = new LinkedList();
		for(Iterator it=testReplies_.values().iterator();it.hasNext();){
			TestReply tr = (TestReply)it.next();
			int verId = AnswersFactory.getInstance().getAnswer(tr.answerId).getVersionId();
			if(testId == tr.getTestId() && verId == versionId && GeneralParams.equalNullableStrings(testTakerId,tr.getTestTakerId())){
				list.add(tr);
			}
		}
		return list;
    }
	
	/**
	 * Method createTestReply
	 * @param tr
	 * @throws CriticalException
	 */
	public synchronized void createTestReply(TestReply tr)
		 throws  RetriableException, CriticalException{
		DBConnect.storeTestReply(tr);
		testReplies_.put(new Key(tr.getTestTakerId(),tr.getTestId(),
								tr.getAnswerId()),tr);
		StatisticData.getInstance().updateQuestionsStatistics(tr.getTestId());				
	}

	/**
	 * Method deleteTestReply
	 * @param tr
	 * @return int
	 * @throws CriticalException
	 */
	public synchronized int deleteTestReply(TestReply tr)
					 throws RetriableException, CriticalException {
		testReplies_.remove(new Key(tr.getTestTakerId(),tr.getTestId(),
							tr.getAnswerId()));
		int res = DBConnect.deleteTestReply(tr);
		StatisticData.getInstance().updateQuestionsStatistics(tr.getTestId());
		return res;
	}
	
	/**
	 * Method exists
	 * @param tr
	 * @return boolean
	 */
	public synchronized boolean exists(TestReply tr){
		return exists(tr.getTestTakerId(),tr.getTestId(),tr.getAnswerId());
	}
	
	/**
	 * Method exists
	 * @param testTakerId
	 * @param testId
	 * @param answerId
	 * @return boolean
	 */
	public synchronized boolean exists(String testTakerId,int testId,
										int answerId){
		try{
			getTestReply(testTakerId,testId,answerId);
			return true;
		}catch(MedException e){
			return false;
		}
	}

	private class Key{
		private int testId;
		private int answerId;
		private String testTakerId;
		public Key(String testTakerId,int testId,int answerId){
			this.testTakerId = testTakerId;
			this.testId = testId;
			this.answerId = answerId;
		}
		
		public boolean equals(Object obj) {
			if(!(obj instanceof Key)){
				return false;
			}
			Key key = (Key)obj;
			return key.testTakerId.equals(testTakerId) && key.testId == testId && key.answerId == answerId;
		}
		
		public int hashCode() {
			return (testTakerId + String.valueOf(testId) + String.valueOf(answerId)).hashCode(); 
		}

	}
	
}
