/*
 * Created on 17/04/2004
 *
 * Created by: svali
 *  */
package il.ac.technion.cs.course236308.e3.med.model;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import il.ac.technion.cs.course236308.e3.med.db.DBConnect;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.MedException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.stats.StatisticData;

/**
 * @author svali
 *
 */
public class TestTakersFactory{
	
	private static TestTakersFactory instance = new TestTakersFactory();
	private HashMap testTakers_ = new HashMap();
	
	private TestTakersFactory(){}
	
	private String hash(String id,int testId){
		return new String(id+testId);	
	}

	/**
	 * Method getInstance
	 * @return TestTakersFactory
	 */
	public static synchronized TestTakersFactory getInstance(){
		return instance; 
	}
	
	public synchronized void loadAllTestTakers() throws CriticalException{
		for(Iterator it=DBConnect.getAllTestTakers().iterator();it.hasNext();){
			TestTaker tt = (TestTaker)it.next();
			testTakers_.put(hash(tt.getId(),tt.getTestId()),tt);
		}
	}
	
	/**
	 * Method getTestTaker
	 * @param id
	 * @param testId
	 * @return TestTaker
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized TestTaker getTestTaker(String id,int testId) 
		throws RetriableException,CriticalException {
		TestTaker testTaker = null;
		if((testTaker =(TestTaker)testTakers_.get(hash(id,testId))) != null){
			return testTaker;
		}
		testTaker = DBConnect.getTestTaker(id,testId);
		testTakers_.put(hash(id,testId),testTaker);			
		return testTaker;	
	}
	
	/**
	 * Method getTestTakersOfTest
	 * @param test
	 * @return List
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized List getTestTakersOfTest(Test test){
		List list = new LinkedList();
		int testId = test.getId();
		for(Iterator it = testTakers_.values().iterator();it.hasNext();){
			TestTaker tt = (TestTaker)it.next();
			if(tt.getTestId() == testId){
				list.add(tt);
			}
		}
		return list;
	}
	
	/**
	 * Method createTestTaker
	 * @param tt
	 * @throws CriticalException
	 */
	public synchronized void createTestTaker(TestTaker tt) 
		throws RetriableException, CriticalException{
		DBConnect.storeTestTaker(tt);
		testTakers_.put(hash(tt.getId(),tt.getTestId()),tt);
		StatisticData.getInstance().updateQuestionsStatistics(tt.getTestId());
	}

	/**
	 * Method storeTestTaker
	 * @param tt
	 * @return int
	 * @throws CriticalException
	 
	public synchronized int storeTestTaker(TestTaker tt) 
		throws CriticalException{
		int affectedRows = DBConnect.storeTestTaker(tt);
		testTakers_.put(hash(tt.getId(),tt.getTestId()),tt);
		return affectedRows;
	}
    */

	/**
	 * Method deleteTestTaker
	 * @param tt
	 * @return int
	 * @throws CriticalException
	 */
	public synchronized int deleteTestTaker(TestTaker tt) 
		throws RetriableException, CriticalException {
		testTakers_.remove(hash(tt.getId(),tt.getTestId()));
		int res = DBConnect.deleteTestTaker(tt);
		StatisticData.getInstance().updateQuestionsStatistics(tt.getTestId());
		return res;
	}
	
	/**
	 * Method exists
	 * @param tt
	 * @return boolean
	 */
	public synchronized boolean exists(TestTaker tt){
		return exists(tt.getId(),tt.getTestId());
	}
	
	/**
	 * Method exists
	 * @param id
	 * @param testId
	 * @return boolean
	 */
	public synchronized boolean exists(String id,int testId){
		try{
			getTestTaker(id,testId);
			return true;
		}catch(MedException e){
			return false;
		}
	}

	
	public synchronized int countTestTakers(Test test){
		int count = 0;
		for(Iterator it=testTakers_.values().iterator();it.hasNext();){
			TestTaker tt = (TestTaker)it.next();
			if(tt.getTestId() == test.getId()){
				++count;
			}
		}
		return count;
	}
	
}
