/*
 * Created on 17/04/2004
 *
 * Created by: svali
 *  */
package il.ac.technion.cs.course236308.e3.med.model;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import il.ac.technion.cs.course236308.e3.med.db.DBConnect;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.MedException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;

/**
 * @author svali
 *
 */
public class TestVersionsFactory{	
	private static TestVersionsFactory instance = new TestVersionsFactory();
	private HashMap testVersions_ = new HashMap();
	
	private TestVersionsFactory(){}
	
	/**
	 * Method getInstance
	 * @return TestVersionsFactory
	 */
	public static synchronized TestVersionsFactory getInstance(){
		return instance; 
	}
	
	/**
	 * Method getTestVersion
	 * @param testId
	 * @param versionId
	 * @return TestVersion
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized TestVersion getTestVersion(int testId,int versionId) 
		throws RetriableException,CriticalException {
		TestVersion testVersion = null;
		if((testVersion =(TestVersion)testVersions_.get(new Key(testId,versionId)))
				 != null){
			return testVersion;
		}
		testVersion = DBConnect.getTestVersion(testId,versionId);
		try{
			VersionsFactory.getInstance().getVersion(versionId);
		}catch(RetriableException e2){ throw new 
			RetriableException("Referential integrity violation - " +
				"record in table ExamVersion with VersionID=" +versionId +
				 " without such answer in Version Table");
		}
		for(Iterator it=DBConnect.getTestAnswerIdsOfVersion(testVersion).iterator();it.hasNext();){
			int taId = ((Integer)it.next()).intValue();
			testVersion.getTestAnswers().
				add(TestAnswersFactory.getInstance().
					getTestAnswer(testId,taId));
		}	
		testVersions_.put(new Key(testVersion.getTestId(),testVersion.getVersion().getId()),testVersion);					
		return testVersion;	
	}
	
    
    public synchronized List getTestVersions(Test test){
        List list = new LinkedList();
        for(Iterator it=testVersions_.values().iterator();it.hasNext();){
        	TestVersion tv = (TestVersion)it.next();
        	if(tv.getTestId() == test.getId()){
        		list.add(tv);
        	}
        }
        return list;
    }
    
    public synchronized List getTestVersions(int verId){
        List list = new LinkedList();
        for(Iterator it=testVersions_.values().iterator();it.hasNext();){
        	TestVersion tv = (TestVersion)it.next();
        	if(tv.getVersion().getId() == verId){
        		list.add(tv);
        	}
        }
        return list;    	
    }

	/**
	 * Method createTestVersion
	 * @param test
	 * @param version
	 * @return TestVersion
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized TestVersion createTestVersion(Test test,Version version)
		 throws RetriableException, CriticalException{
		int mainCatId = QuestionsFactory.getInstance().getQuestion(version.getQuestionId()).getMainCategoryId();
		String mainCatName = CategoriesFactory.getInstance().getCategory(mainCatId).getFullName();
		TestVersion tv = new TestVersion();
		tv.setFullCategoryName(mainCatName);
		tv.setTestId(test.getId());
		tv.setVersion(version);
		tv.setWeight(new Double(1));
		storeTestVersion(tv);
        return tv;
	}

	/**
	 * Method cloneTestVersion
	 * @param newTest
	 * @param testVersion
	 * @return TestVersion
	 * @throws CriticalException
	 * @throws RetriableException
	 * @throws RetriableException
	 */
	public synchronized TestVersion cloneTestVersion(Test newTest,TestVersion testVersion) 
				throws RetriableException, CriticalException{
		TestVersion newTestVersion = new TestVersion();
		newTestVersion.setFullCategoryName(testVersion.getFullCategoryName());
		newTestVersion.setTestId(newTest.getId());
		newTestVersion.setVersion(testVersion.getVersion());
		newTestVersion.setWeight(testVersion.getWeight());
		storeTestVersion(newTestVersion);
		return newTestVersion;
	}

	/**
	 * Method storeTestVersion
	 * @param testVersion
	 * @return int
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized int storeTestVersion(TestVersion testVersion) 
		throws RetriableException,CriticalException{
		int affectedRows = DBConnect.storeTestVersion(testVersion);
		TestsFactory.getInstance().getTest(testVersion.getTestId()).getTestVersions().add(testVersion);
		testVersions_.put(new Key(testVersion.getTestId(),testVersion.getVersion().getId()),testVersion);		
		return affectedRows;
	}
	
	/**
	 * Method deleteTestVersion
	 * @param testVersion
	 * @return int
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized int deleteTestVersion(TestVersion testVersion)
		throws RetriableException, CriticalException{
		TestsFactory.getInstance().getTest(testVersion.getTestId()).
										getTestVersions().remove(testVersion);
		testVersions_.remove(testVersion);
		return DBConnect.deleteTestVersion(testVersion);
	}
	
	/**
	 * Method exists
	 * @param testVer
	 * @return boolean
	 */
	public synchronized boolean exists(TestVersion testVer){
		return exists(testVer.getTestId(),testVer.getVersion().getId());
	}
	
	/**
	 * Method exists
	 * @param testId
	 * @param versionId
	 * @return boolean
	 */
	public synchronized boolean exists(int testId,int versionId){
		try{
			getTestVersion(testId,versionId);
			return true;
		}catch(MedException e){
			return false;
		}
	}	
	
	private class Key{
		int testId;
		int versionId;
		public Key(int testId,int versionId){
			this.testId = testId;
			this.versionId = versionId;
		}
		
		public boolean equals(Object obj) {
			if(!(obj instanceof Key)){
				return false;
			}
			Key key = (Key)obj;
			return key.testId == testId && key.versionId == versionId;
		}
		
		public int hashCode() {
			return (String.valueOf(testId) + String.valueOf(versionId)).hashCode(); 
		}
	}
}
