/*
 * Created on 17/04/2004
 *
 * Created by: svali
 *  */
package il.ac.technion.cs.course236308.e3.med.model;

import il.ac.technion.cs.course236308.e3.med.db.DBConnect;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.MedException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.gui.treenodes.TestsTreeRootNode;

import java.util.Collections;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * @author svali
 *
 */
public class TestsFactory{	
	private static TestsFactory instance = new TestsFactory();
	private HashMap tests_ = new HashMap();
	private int maxTestId = 0;
	
	private TestsFactory(){}
	
	/**
	 * Method getInstance
	 * @return TestsFactory
	 */
	public static synchronized TestsFactory getInstance(){
		return instance; 
	}
	
	public int getMaxTestId(){
		return maxTestId;
	}
	
	public void resetMaxId(){
		maxTestId = 0;
	}
	
	public synchronized Test createEmptyTest(){
		++maxTestId; 
		Test test = new Test();
		GregorianCalendar date = new GregorianCalendar();
		date.add(GregorianCalendar.MONTH,1);
		test.setDate(date);
		test.setId(maxTestId);
		return test;
	}
	
	/**
	 * Method getTest
	 * @param id
	 * @return Test
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Test getTest(int id) 
		throws RetriableException,CriticalException {
		Test test = null;
		if((test =(Test)tests_.get(new Integer(id))) != null){
			return test;
		}
		test = DBConnect.getTest(id);
		if(test.getId() > maxTestId){
			maxTestId = test.getId();
		}
		tests_.put(new Integer(id),test);
		for(Iterator it=DBConnect.getTestVersionIdsOfTest(test).iterator();	it.hasNext();){
			int tvId = ((Integer)it.next()).intValue();
			test.getTestVersions().add(TestVersionsFactory.getInstance().getTestVersion(id,tvId));
		}
		return test;	
	}
	
	public synchronized Set getAllTestIds(){
		return tests_.keySet();
	}
	
	/**
	 * Method loadAllTests
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized void loadAllTests() throws CriticalException, RetriableException{
		for(Iterator it=DBConnect.getAllTestIds().iterator();it.hasNext();){
			getTest(((Integer)it.next()).intValue());			
		}
	}
	/**
	 * Method getCopyOfTest
	 * @param testId
	 * @return
	 * @throws RetriableException
	 * @throws CriticalException
	 */
	public synchronized Test getCopyOfTest(int testId) throws RetriableException, CriticalException{
		return (Test)TestsFactory.getInstance().getTest(testId).clone();
	}

	
	/**
	 * Method createTest
	 * @return Test
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Test createTest() throws CriticalException{
		Test test = createEmptyTest(); 
		storeTest(test);
		return test;
	}

	/**
	 * Method cloneTest
	 * @param test
	 * @return Test
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Test cloneTest(Test test) throws CriticalException{
		Test newTest = createEmptyTest();
		newTest.setCourseName(test.getCourseName());
		newTest.setDate(test.getDate());
		newTest.setDescription1(test.getDescription1());
		newTest.setDescription2(test.getDescription2());
		newTest.setFactorConstA(test.getFactorConstA());
		newTest.setFactorConstB(test.getFactorConstB());
		newTest.setInstructions(test.getInstructions());
		newTest.setMaxGrade(test.getMaxGrade());
		newTest.setMinGrade(test.getMinGrade());
		newTest.setPassingGrade(test.getPassingGrade()); 
		storeTest(newTest);
		return newTest;
	}
		
	/**
	 * Method storeTest
	 * @param test
	 * @return int
	 * @throws CriticalException
	 */
	public synchronized int storeTest(Test test) throws CriticalException{
		int affectedRows = DBConnect.storeTest(test);
		if(test.getId() > maxTestId){
			maxTestId = test.getId();
		}
		tests_.put(new Integer(test.getId()),test);
		return affectedRows;
	}
	
	/**
	 * Method deleteTest
	 * @param test
	 * @return int
	 * @throws CriticalException
	 */
	public synchronized int deleteTest(Test test) 
		throws CriticalException {
		if(test.getId() == maxTestId){
			--maxTestId;
		}
		tests_.remove(new Integer(test.getId()));
		return DBConnect.deleteTest(test.getId());
	}
	
	public boolean isVersionLocked(Version ver){
		for(Iterator it=tests_.values().iterator();it.hasNext();){
			Test test = (Test)it.next();
			if(test.getStatus().equals("U")){
				continue;
			}
			for(Iterator it2=test.getTestVersions().iterator();it2.hasNext();){
				TestVersion tv = (TestVersion)it2.next();
				if(tv.getVersion().equals(ver)){
					return true;
				}
			}
		}
		return false;
	}
	
	public List getTestsOfVersion(Version version){
		List list = new LinkedList();
		for(Iterator it=tests_.values().iterator();it.hasNext();){
			Test test = (Test)it.next();
			for(Iterator it2=test.getTestVersions().iterator();it2.hasNext();){
				TestVersion tv = (TestVersion)it2.next();
				if(tv.getVersion().equals(version)){
					list.add(test);
					break;
				}
			}
		}
		return list;
	}
	
	public boolean isVersionAppearInTests(Version ver){
		for(Iterator it=tests_.values().iterator();it.hasNext();){
			Test test = (Test)it.next();
			for(Iterator it2=test.getTestVersions().iterator();it2.hasNext();){
				TestVersion tv = (TestVersion)it2.next();
				if(tv.getVersion().equals(ver)){
					return true;
				}
			}
		}
		return false;		
	}
	
	/**
	 * Method exists
	 * @param test
	 * @return boolean
	 */
	public synchronized boolean exists(Test test){
		return exists(test.getId());
	}
	
	/**
	 * Method exists
	 * @param testId
	 * @return boolean
	 */
	public synchronized boolean exists(int testId){
		try{
			getTest(testId);
			return true;
		}catch(MedException e){
			return false;
		}
	}	
	
	public TestsTreeRootNode buildTree(){
		TestsTreeRootNode node = new TestsTreeRootNode("Tests Root");
		LinkedList tests = new LinkedList();
		tests.addAll(tests_.values());
		Collections.sort(tests);
		for(Iterator it=tests.iterator();it.hasNext();){
			Test test = (Test)it.next();
			node.add(test.toTreeNode());
		}
		return node;
	}
}
