/*
 * Created on 17/04/2004
 *
 * Created by: svali
 *  */
package il.ac.technion.cs.course236308.e3.med.model;

import il.ac.technion.cs.course236308.e3.med.db.DBConnect;
import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.MedException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;

import java.util.HashMap;
import java.util.Iterator;

/**
 * @author svali
 *
 */
public class VersionsFactory{	
	private static VersionsFactory instance = new VersionsFactory();
	private HashMap versions_ = new HashMap();
	int maxVerId = 0;
	
	private VersionsFactory(){}
	
	/**
	 * Method getInstance
	 * @return VersionsFactory
	 */
	public static synchronized VersionsFactory getInstance(){
		return instance; 
	}
	
	public int getMaxVersionId(){
		return maxVerId;
	}
	
	public void resetMaxId(){
		maxVerId = 0;
	}

	
	/**
	 * Method getVersion
	 * @param id
	 * @return Version
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Version getVersion(int id)
		 throws RetriableException,CriticalException {
		Version ver = null;
		if((ver =(Version)versions_.get(new Integer(id))) != null){
			return ver;
		}
		ver = DBConnect.getVersion(id);
		if(ver.getId() > maxVerId){
			maxVerId = ver.getId();
		}
		versions_.put(new Integer(id),ver);	
		for(Iterator it=DBConnect.getAnswerIdsOfVersion(ver).iterator();
				it.hasNext();){
			int ansId = ((Integer)it.next()).intValue();
			ver.getAnswers().add(AnswersFactory.getInstance().getAnswer(ansId));		
		}
		return ver;	
	}
		
		
	private Version createEmptyVersion(Question quest){
		++maxVerId;
		Version ver = new Version();
		ver.setId(maxVerId);
		ver.setQuestionId(quest.getId());
		return ver; 
	}
	/**
	 * Method createVersion
	 * @param quest
	 * @return Version
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Version createVersion(Question quest)throws CriticalException{
		Version ver = createEmptyVersion(quest);
		storeVersion(quest,ver);
		return ver;
	}

	/**
	 * Method cloneVersion
	 * @param quest
	 * @param ver
	 * @return Version
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized Version cloneVersion(Question quest,Version ver)throws CriticalException{
		Version newVer = createEmptyVersion(quest);
		newVer.setBody(ver.getBody());
		newVer.setCorrectAnswerId(ver.getCorrectAnswerId());
		newVer.setLanguage(ver.getLanguage());
		newVer.setNotes(ver.getNotes());
		storeVersion(quest,newVer);
		return newVer;
	}
	
	/**
	 * Method storeVersion
	 * @param quest
	 * @param ver
	 * @return int
	 * @throws CriticalException
	 */
	public synchronized int storeVersion(Question quest,Version ver)
		throws CriticalException{
		int affectedRows = DBConnect.storeVersion(quest,ver);
		if(ver.getId() > maxVerId){
			maxVerId = ver.getId();
		}
		versions_.put(new Integer(ver.getId()),ver);
		if(!quest.getVersions().contains(ver)){
			quest.getVersions().add(ver);
		}
		return affectedRows;
	}
	
	/**
	 * Method deleteVersion
	 * @param ver
	 * @return int
	 * @throws CriticalException
	 * @throws RetriableException
	 */
	public synchronized int deleteVersion(Version ver)
		throws RetriableException,CriticalException {
		if(ver.getId() == maxVerId){
			--maxVerId;
		}
		QuestionsFactory.getInstance().getQuestion(ver.getQuestionId()).
													getVersions().remove(ver);
		versions_.remove(new Integer(ver.getId()));
		return DBConnect.deleteVersion(ver.getId());
	}
	
	/**
	 * Method exists
	 * @param ver
	 * @return boolean
	 */
	public synchronized boolean exists(Version ver){
		return exists(ver.getId());
	}
	
	/**
	 * Method exists
	 * @param verId
	 * @return boolean
	 */
	public synchronized boolean exists(int verId){
		try{
			getVersion(verId);
			return true;
		}catch(MedException e){
			return false;
		}
	}	
}
