/*
 * Created on 04/05/2004
 *
 */
package il.ac.technion.cs.course236308.e3.med.model.testscrambler;

import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Answer;
import il.ac.technion.cs.course236308.e3.med.model.Test;
import il.ac.technion.cs.course236308.e3.med.model.TestVersion;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

/**
 * @author Alexander, Bennyamin, Svetlana and Valentin 
 * Created in SSDL lab - Technion Israel
 *
 */
public class TestScrambler {
	Test test;
	HashMap versionHash;
	Random randGen;
	
	/**
	 * Method TestScrambler
	 * @param t
	 */
	public TestScrambler(Test t) {
		this.test = t;
	}
	
	//returns a string with the name of the parent category
	private String getParentCatName(String catName){
		if (catName.equals("/")) return null;
		if (catName.lastIndexOf("/")==0) return "/";
		return catName.substring(0,catName.lastIndexOf("/"));
	}
	
	//returns the first free index in an array
	private int getFirstFreeIndex(Object[] array){
		for(int i=0; i<array.length;i++){
			if (array[i]==null) return i;
		}
		return array.length;
	}
	
//scrambles input list, changes it to be scrambled and also returns the result
	private List scrambleArray(List srcList){
		Object[] dstList = new Object[srcList.size()];
		
		for (int i=0; i<dstList.length;i++){
			Object o = srcList.get(i);
			if (o instanceof Answer && !((Answer)o).isMixable()){
				dstList[i] = o;
			}else {
				dstList[i]=null;
			}
		}
		for (int i=0; i<dstList.length;i++){
			if (dstList[i] != null) srcList.remove(dstList[i]);
		}
		while (srcList.size()>0) {
			int curRand = randGen.nextInt(srcList.size());	
			Object o = srcList.remove(curRand);
			int destIndex = getFirstFreeIndex(dstList);
			dstList[destIndex] = o;
		}
		
	//This changes the input list and also returns it, for convinience of use
		for(int i=0; i<dstList.length; i++){
			srcList.add(dstList[i]);
		}
		return srcList;
	}
	
	private void scrambleVersionHash (List l, String catig){
		ScrambleHashEntry entry = ((ScrambleHashEntry)versionHash.get(catig));
		//scramble sub-catigories of catig
		scrambleArray(entry.getChildren());
		//scramble questions in this catig
		scrambleArray(entry.getVersions());
		//return the testversions to the list l
		for (Iterator i=entry.getVersions().iterator();i.hasNext();){
			l.add(i.next());
		}
		//recursivly visit sub-catigories in the scrambled order
		for (Iterator i=entry.getChildren().iterator();i.hasNext();){
			String childCat = (String)i.next();
			scrambleVersionHash(l,childCat);	
		}
	}
	private void buildVersionHash(List l) throws RetriableException{
		for(Iterator it = l.iterator();it.hasNext();){
			TestVersion tv = (TestVersion)it.next();
			String catName = tv.getFullCategoryName();
			
			if (!catName.startsWith("/")) 
				throw new RetriableException("Category name \"" + catName
					 + "\"" +" is illegal - doesn't start with \'/\'");
			
			if (catName.endsWith("/") && !catName.equals("/")){
					catName=catName.substring(0,catName.lastIndexOf("/"));	
			}
			
			String parentCat = getParentCatName(catName);
			
			if (versionHash.get(catName)==null){
				versionHash.put(catName, new ScrambleHashEntry(parentCat));				
			}
			((ScrambleHashEntry)versionHash.get(catName)).addVersion(tv);
			
			while (parentCat!=null){
				String pp = getParentCatName(parentCat);
				if (versionHash.get(parentCat)==null){
						versionHash.put(parentCat, new ScrambleHashEntry(pp));				
				}
				((ScrambleHashEntry)versionHash.get(parentCat)).
											addChildCategory(catName);
				catName = parentCat;
				parentCat = pp;
			}
		}
	}
	/**
	 * Method scramble
	 * @param copyNum
	 * @throws RetriableException
	 * @throws CriticalException
	 */
	public void scramble(int copyNum) throws RetriableException{
		if (test.getTestVersions()==null || 
			test.getTestVersions().size()==0) return;
		
		//use copyNum as seed to random number generator
		randGen = new Random(copyNum);
		versionHash  = new HashMap();
		List origVersionList = test.getTestVersions();
		
		//build the hashtable of categories and questions in them
		buildVersionHash(origVersionList);

		//earase old version list
		origVersionList.clear();
		
		//Go over hash, scramble categories and questions 
		//and rebuild origVersionList
		scrambleVersionHash(origVersionList, "/");
		
		//scramble answers in each version
		for(Iterator it = origVersionList.iterator();it.hasNext();){
			TestVersion tv = (TestVersion)it.next();
			scrambleArray(tv.getVersion().getAnswers());
		}
	}
}
