/*
 * Created on 23/04/2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package il.ac.technion.cs.course236308.e3.med.script.command;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

import il.ac.technion.cs.course236308.e3.med.exception.MedException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;

/**
 * Type ScriptCommand
 * @author sybenny
 * @version 1.0
 */
public abstract class ScriptCommand {
	/** Field STRING */
	public static final int STRING = 0;
	/** Field INTEGER */
	public static final int INTEGER = 1;
	/** Field BOOL */
	public static final int BOOL = 2;
	/** Field DOUBLE */
	public static final int DOUBLE = 3;
	/** Field TEXT */
	public static final int TEXT = 0;
	
	protected Vector params = new Vector();
	protected String usage = new String();
	
	/**
	 * Method setParams
	 * @param paramStrs
	 */
	public void setParams(String[] paramStrs) { 
		for(int i=0; i<paramStrs.length;i++){
			if(!paramStrs[i].equals("")) params.add(paramStrs[i]); 
		}
	}
	/**
	 * Method execute
	 * @throws IOException
	 * @throws MedException
	 */
	public abstract void execute() throws MedException, IOException;
	
	/**
	 * Method extractString
	 * @param myText
	 * @param paramName
	 * @return String
	 * @throws RetriableException
	 */
	public String extractString(String myText, String paramName)
		 throws RetriableException{	
		if (myText.equals("")) 
			throw new RetriableException(usage);
		if (!(myText.startsWith("\"") && myText.endsWith("\""))) 
			throw new RetriableException(paramName + 
				" should be a string surrounded by double-qoutes");
		return myText.substring(1, myText.length()-1);
	}
	/**
	 * Method extractDate
	 * @param myText
	 * @param paramName
	 * @return Date
	 * @throws RetriableException
	 */
	public Date extractDate(String myText, String paramName) 
			throws RetriableException{	
			if (myText.equals("")) 
				throw new RetriableException(usage);
			if (!(myText.startsWith("\"") && myText.endsWith("\""))) 
				throw new RetriableException(paramName + 
					" should be a string surrounded by double-qoutes\n" +
					" in the format: \"yyyy-MM-dd HH:mm:ss\"");				
			String dateString = myText.substring(1, myText.length()-1);
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");	
			Date date = null;
			try{
				date = sdf.parse(dateString);
			}catch(ParseException e){
				throw new RetriableException(paramName +
				 " should be a string surrounded by double-qoutes\n   " +
				 "in the format: \"yyyy-MM-dd HH:mm:ss\"");				
			}
			return date;
		}
	/**
	 * Method paramExists
	 * @param index
	 * @return boolean
	 */
	public boolean paramExists(int index){
		if (index<0 || index>(params.size()-1)) return false;
		return true;
	}
	
	/**
	 * Method extractInt
	 * @param myText
	 * @param paramName
	 * @return int
	 * @throws RetriableException
	 */
	public int extractInt(String myText, String paramName)  
			throws RetriableException{	
		if (myText.equals("")) 
			throw new RetriableException(usage);
		int result;
		try {
			result = (new Integer(myText)).intValue();
		} catch (NumberFormatException e){
			throw new RetriableException(paramName + " should be an integer");
		}
		return result;
	}
	/**
	 * Method extractDouble
	 * @param myText
	 * @param paramName
	 * @return double
	 * @throws RetriableException
	 */
	public double extractDouble(String myText, String paramName) 
		 throws RetriableException{	
			if (myText.equals("")) 
				throw new RetriableException(usage);
			double result;
			try {
				result = (new Double(myText)).doubleValue();
			} catch (NumberFormatException e){
				throw new RetriableException(paramName + " should be a double");
			}
			return result;
	}
		
	/**
	 * Method extractBool
	 * @param myText
	 * @param paramName
	 * @return boolean
	 * @throws RetriableException
	 */
	public boolean extractBool(String myText, String paramName)
	 throws RetriableException {
		if (myText.equals(""))
			throw new RetriableException(usage);
		
		if (myText.equalsIgnoreCase("true"))  return true;
		if (myText.equalsIgnoreCase("false")) return false;

		throw new RetriableException(paramName + " should be true or false");
	}
	
	/**
	 * Method extractByList
	 * @param myText
	 * @param options
	 * @param paramName
	 * @return int
	 * @throws RetriableException
	 */
	public int extractByList(String myText, String[] options, String paramName) 
		throws RetriableException {
		if (myText.equals(""))
			throw new RetriableException(usage);
		String optionsStr = new String();
		
		for (int i=0; i<options.length; i++){
			if (options[i].equalsIgnoreCase(myText)) return i;
			optionsStr = optionsStr.concat(options[i]);
			if (i!=(options.length-1)) optionsStr = optionsStr.concat(" or ");
		}
		
		throw new RetriableException(paramName + " should be " + optionsStr);
	}
	
	/**
	 * Method paramOfType
	 * @param index
	 * @param type
	 * @return boolean
	 */
	public boolean paramOfType(int index, int type){
		if (!paramExists(index)) return false;
		String myText = (String)params.get(index);
		switch (type) {
			case STRING: return (myText.startsWith("\"") 
							&& myText.endsWith("\""));
			case INTEGER: try {
							(new Integer(myText)).intValue();
						  }catch (NumberFormatException e){
						  	return false;
						  }
							return true;
			case DOUBLE: try{
							(new Double(myText)).doubleValue();
						}catch (NumberFormatException e){
							return false;
						}
						return true;
		 	case BOOL: return (myText.equalsIgnoreCase("true") ||
		 				 myText.equalsIgnoreCase("false"));
		 	
		 	default: return false;
		}
	}
	/**
	 * Method assertMaxParams
	 * @param num
	 * @throws RetriableException
	 */
	public void assertMaxParams(int num) throws RetriableException{
		if (params.size()>num)
				throw new RetriableException(usage);
	}
	
}
