package il.ac.technion.cs.course236308.e3.med.stats;

import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.AnswersFactory;

/**
 * @author shralex
 *
 */
public class AnswerInfo implements Comparable{
	private int answerId;
	private byte answerOrdinal;
	private int numStudentsAnswered = 0;
	private double gradeSumAnswered = 0;
	private double normGradeSumAnswered = 0;
	private int normGradeNum =0;
	private boolean isCorrect;
	private int totalNumStudents;
	private Double totalGradesSum;
	private Double totalGradesStd;
	
	public AnswerInfo(Integer answerId) throws CriticalException, RetriableException{
		this.answerId = answerId.intValue();
		answerOrdinal = AnswersFactory.getInstance().getAnswer(answerId.intValue()).getOrdinal();	
	}
	public byte getOrdinal(){
		return answerOrdinal;
	}
	public AnswerInfo(Integer answerId, boolean b) throws RetriableException, CriticalException{
		this.answerId = answerId.intValue();
		this.isCorrect = b;
		answerOrdinal = AnswersFactory.getInstance().getAnswer(answerId.intValue()).getOrdinal();
	}
	public void setIsCorrect(boolean b){
		isCorrect = b;
	}
	public int getNumAnsweringStudents(){
		return numStudentsAnswered;
	}
	public Double getPercentAnsweringStudents(){
		if (totalNumStudents == 0) return null;
		return new Double(100*(((double)numStudentsAnswered))/totalNumStudents);
	}
	public void setTotalNumStudents(int tot){
		this.totalNumStudents = tot;
	}
	public void setTotalGradesAverage(Double totAvg){
		if (totAvg==null) this.totalGradesSum = null;
		else this.totalGradesSum = new Double(totAvg.doubleValue() * totalNumStudents);
	}
	public void setTotalGradesStd(Double totStd){
		this.totalGradesStd = totStd;
	}
	/**
	 * 
	 * @return average of the normalized grades of students that chose this answer
	 */
	public Double getNormGradeAvg(){
		if (normGradeNum == 0) return null;
		return new Double(normGradeSumAnswered/normGradeNum);
	}
	public Double getCorelation(){
		if (numStudentsAnswered == 0 || totalGradesStd == null || totalGradesStd.doubleValue()==0|| totalGradesSum==null || (totalNumStudents - numStudentsAnswered) == 0)return null;
		double Mp = gradeSumAnswered/numStudentsAnswered;
		double Mq = (totalGradesSum.doubleValue() - gradeSumAnswered)/(totalNumStudents - numStudentsAnswered);
		double p = getPercentAnsweringStudents().intValue()/100.0;
		double q = 1-p;
		double corel = ((Mp-Mq)/totalGradesStd.doubleValue())*Math.sqrt(p*q); 
		return new Double(corel);
	}
	public boolean getIsCorrect(){
		return isCorrect;
	}
	
	public void addStudentInfo(double grade, Double normGrade){
		numStudentsAnswered++;
		gradeSumAnswered += grade;
		if (normGrade !=null ){
			normGradeSumAnswered += normGrade.doubleValue();
			normGradeNum++;
		}
	}
	
	public int getAnswerId(){
		return answerId;
	}
	public int compareTo(Object obj) {
		AnswerInfo ans = (AnswerInfo)obj;
		return (new Integer(getOrdinal())).
				compareTo(new Integer(ans.getOrdinal()));	
	}
}
