package il.ac.technion.cs.course236308.e3.med.stats;

import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.TestVersion;

import java.util.HashMap;
import java.util.Iterator;

/**
 * @author shralex
 *
 */
public class QuestionInfo implements Comparable{
	private TestVersion tv;
	private Double difficulty = null;
	private Double validity = null;
	private HashMap answersHash;
	private String remarks;
	private int numCorrectAnswers=0;
	private int numRemarks;
	
	public QuestionInfo(GradingTableItem gti, int numCorrectAnswers, HashMap allAnswersInfo) throws RetriableException, CriticalException{
		tv = gti.getTestVersion();
		this.numCorrectAnswers = numCorrectAnswers;
		answersHash = new HashMap();
		collectData(gti.getAnswerHash(), allAnswersInfo);
	}
	public void setRemarks(String rem){
		this.remarks = rem;
		char[] letters = remarks.toCharArray();
		numRemarks = 0;
		for (int i = 0; i< letters.length; i++){
			if (letters[i]=='.') numRemarks++;
		}
	}
	public int getNumRemarks(){
		return numRemarks;
	}
	public String getRemarks(){
		return remarks;
	}
	public void setDifficulty(Double dif){
		this.difficulty = dif;
	}
	public Double getDifficulty(){
		return difficulty;
	}
	public void setValidity(Double val){
		this.validity = val;
	}
	public Double getValidity(){
		return validity;
	}
	public int getCorrectAnswerId(){
		return tv.getVersion().getCorrectAnswerId();
	}
	private void collectData(HashMap aHash, HashMap allAnswersInfo) throws RetriableException, CriticalException{
		//aHash has all answers
		//allAnswersInfo has all answers that were answered by students
//		System.out.println("VerId: " + tv.getVersion().getId());
		for (Iterator it = aHash.keySet().iterator(); it.hasNext();){
			Integer ansId = (Integer)it.next();
			if (allAnswersInfo.containsKey(ansId)){
				AnswerInfo aInfo = (AnswerInfo)allAnswersInfo.get(ansId);
//				System.out.println("VerId: " + tv.getVersion().getId()+ " AnsId: " + ansId);
				answersHash.put(ansId, aInfo);
				aInfo.setIsCorrect(getCorrectAnswerId()==ansId.intValue());
			}
			else {
//				System.out.println("VerId: " + tv.getVersion().getId()+ " AnsId: " + ansId);
				answersHash.put(ansId, new AnswerInfo(ansId, getCorrectAnswerId()==ansId.intValue()));
			}
		}
	}
	public HashMap getAnswersHash(){
		return answersHash;
	}
	public int getNumCorrectAnswers(){
		return numCorrectAnswers;
	}
	public TestVersion getTestVersion(){
		return tv;
	}
	public int getTestVersionId(){
		return tv.getVersion().getId();
	}
	public int compareTo(Object o)
	{
		QuestionInfo other = (QuestionInfo)o;	
		if (numCorrectAnswers<other.getNumCorrectAnswers()) return -1;
		else if (numCorrectAnswers>other.getNumCorrectAnswers()) return 1;
		int verId = tv.getVersion().getId();
		int otherVerId = other.getTestVersion().getVersion().getId();
		if (verId > otherVerId) return 1;
		if (verId < otherVerId) return -1;
		return 0;
	}
}
