package il.ac.technion.cs.course236308.e3.med.stats;

import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

/**
 * @author shralex
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class QuestionsStats {
	private StudentsStats studentStats;
	private HashMap questions; //versionId -> QuestionInfo
	private int testId;
	private int numQuestions;
	private int correctAnswersSum;
	private int correctAnswersSum2;
	//private Double difficultyAvg = null;
	//private Double difficultyStd = null;
	private TreeSet sortedQinfo;
	private int totalNumRemarks;
	private int numQuestionsWithoutRemarks;
	
	public QuestionsStats(int testId, StudentsStats studStats) throws RetriableException, CriticalException{
		this.testId = testId;
		studentStats = studStats;
		collectData();
	}
	public int getTestId(){
		return testId;
	}
	public StudentsStats getStudentStats(){
		return studentStats;
	}
	
	public void collectData() throws RetriableException, CriticalException {
		GradingTable gt = studentStats.getGradingTable();
		HashMap qHash = gt.getQuestionHash();
		numQuestions = qHash.size();
		//difficultyAvg = null;
		//difficultyStd = null;
		correctAnswersSum = 0;
		correctAnswersSum2 = 0;
		double difficultySum = 0;
		double difficultySum2 = 0;
		HashMap correctAnswersHash = studentStats.getCorrectAnswersPerQuestion();
		questions = new HashMap();
		for (Iterator it = qHash.keySet().iterator();it.hasNext();){
			Integer versionId = (Integer)it.next();
			GradingTableItem gti = (GradingTableItem)qHash.get(versionId);
			int numCorrectAnswers = ((Integer)correctAnswersHash.get(versionId)).intValue();
			questions.put(versionId, new QuestionInfo(gti, numCorrectAnswers, studentStats.getAnswersHash()));
			correctAnswersSum += numCorrectAnswers;
			correctAnswersSum2 += numCorrectAnswers * numCorrectAnswers;
		}
	
		sortedQinfo = new TreeSet(questions.values());
		
		Double avgRightReplies = getAvgNumCorrectRepliesPerQuestion();
		Double stdRightReplies = getStdNumCorrectRepliesPerQuestion();
		for (Iterator it = sortedQinfo.iterator();it.hasNext();){
			QuestionInfo qInfo = (QuestionInfo)it.next();
			Double difficulty = null; 
			if (!(avgRightReplies == null || stdRightReplies==null || stdRightReplies.doubleValue()==0))
			 	difficulty = new Double((avgRightReplies.doubleValue() - qInfo.getNumCorrectAnswers())/stdRightReplies.doubleValue());
			if (difficulty!=null){
				difficultySum += difficulty.doubleValue();
				difficultySum2 += difficulty.doubleValue() * difficulty.doubleValue();
		    }
			qInfo.setDifficulty(difficulty);
			qInfo.setValidity(calcValidity(qInfo));
		}
		/*if (numQuestions>0) difficultyAvg = new Double(difficultySum/numQuestions);
		double m0 = numQuestions;
		double m1 = difficultySum;
		double m2 = difficultySum2;
		if (m0 > 1) difficultyStd = new Double(Math.sqrt((m0*m2-m1*m1)/(m0*(m0-1))));
		*/
		totalNumRemarks = 0;
		numQuestionsWithoutRemarks = 0;
		for (Iterator it = sortedQinfo.iterator();it.hasNext();){
			QuestionInfo qInfo = (QuestionInfo)it.next();
			qInfo.setRemarks(calcRemarks(qInfo));
			int numRemarks = qInfo.getNumRemarks();
			totalNumRemarks += numRemarks;
			if (numRemarks==0) numQuestionsWithoutRemarks++;
		}
	}
	public int getNumQuestionsWithoutRemarks(){
		return numQuestionsWithoutRemarks;
	}
	public int getTotalNumRemarks(){
		return totalNumRemarks;
	}
	//public Double getAvgDifficulty(){
	//	return difficultyAvg;
	//}
	//public Double getStdDifficulty(){
	//	return difficultyStd;
	//}
	public Double getNormDifficulty(int verId){
		QuestionInfo qInfo = (QuestionInfo)questions.get(new Integer(verId));
		return calcNormDifficulty(qInfo);
	}
	private Double calcNormDifficulty(QuestionInfo qInfo){
		//if (difficultyAvg==null || difficultyStd == null || difficultyStd.doubleValue() == 0) return null;
		//return new Double ((qInfo.getDifficulty().doubleValue() - difficultyAvg.doubleValue())/difficultyStd.doubleValue());
		return qInfo.getDifficulty();
	}
	public String getRemarks(int verId){
		QuestionInfo qInfo = (QuestionInfo)questions.get(new Integer(verId));
		return qInfo.getRemarks();
	}
	public double getSumQuestinsWeights(){
		GradingTable gt = studentStats.getGradingTable();
		return gt.getSumQuestionsWeight();
	}
	private String calcRemarks(QuestionInfo qInfo){
		String remark = new String();
		if (studentStats.getNumStudents()>=10){	
			Double normDifficulty = calcNormDifficulty(qInfo);
			if (qInfo.getNumCorrectAnswers()==0) remark = "Unfair Question. ";
			else if (normDifficulty!=null && normDifficulty.doubleValue()>2) remark = "Very difficult question. ";
			else if (normDifficulty!=null && normDifficulty.doubleValue()>1) remark = "Difficult question. ";
			else if (qInfo.getNumCorrectAnswers()==studentStats.getNumStudents())
				remark = "Trivial question. ";
			else if (normDifficulty!=null && normDifficulty.doubleValue()<-2) remark = "Very easy question. ";
			else if (normDifficulty!=null && normDifficulty.doubleValue()<-1) remark = "Easy question. ";			
			if (qInfo.getValidity()!=null && qInfo.getValidity().doubleValue()<0)
				remark = remark.concat("Very low validity to this question. ");
			else if (qInfo.getValidity()!=null && qInfo.getValidity().doubleValue()<0.1)
				remark = remark.concat("Low validity to this question. ");
		}
		HashMap answerHash = getAnswersInfo(qInfo.getTestVersionId());
		int numNotCorrectAnswersNotChosen = 0;
		for (Iterator it = answerHash.keySet().iterator(); it.hasNext();){
			Integer ansId = (Integer)it.next();
			AnswerInfo aInfo = (AnswerInfo)answerHash.get(ansId);
			if (!aInfo.getIsCorrect()){
				if (aInfo.getNumAnsweringStudents()==0) numNotCorrectAnswersNotChosen++;
				else if (aInfo.getNumAnsweringStudents()>qInfo.getNumCorrectAnswers())
					remark = remark.concat("The incorrect answer " + aInfo.getOrdinal() + " was chosen more times than the correct answer. ");
				if (aInfo.getNumAnsweringStudents()>=5 &&aInfo.getCorelation()!=null && aInfo.getCorelation().doubleValue()>0.1)
					remark = remark.concat("High correlation to choosing wrong answer " + aInfo.getOrdinal() + " and succeess in the test. ");
			} else if (aInfo.getNumAnsweringStudents()>=5 && aInfo.getCorelation()!=null &&aInfo.getCorelation().doubleValue()<-0.1){
				remark = remark.concat("Low correlation to choosing right answer and succeess in the test. ");
			}
		}
		if (numNotCorrectAnswersNotChosen>1)
			remark = remark.concat(numNotCorrectAnswersNotChosen + " incorrect answers weren't chosen at all. ");
		else if (numNotCorrectAnswersNotChosen>0)
			remark = remark.concat(numNotCorrectAnswersNotChosen + " incorrect answer wasn't chosen at all. ");
	

		return remark;
	}
	
	private int calcNumCorrectReplies(List studentList, QuestionInfo qInfo, double borderMark){
		int numCorrect = 0;
		int verId = qInfo.getTestVersionId();
		for (Iterator it = studentList.iterator(); it.hasNext();){
			StudentAnswers sa = (StudentAnswers)it.next();
			if (sa.getCorrectlyAnsweredVersions().contains(new Integer(verId))){
				numCorrect++;
				if (sa.getGrade()==borderMark) numCorrect -=0.5;
			}
				
		}
		return numCorrect;
	}
	private Double calcValidity(QuestionInfo qInfo){
		if (studentStats.getNumStudents()==0) return null;
		List lowThirdList = studentStats.getStudentsInLowThirdGrades();
		List highThirdList = studentStats.getStudentsInHighThirdGrades();
		double lowThirdMark = studentStats.Med(1.0/3);
		double highThirdMark = studentStats.Med(2.0/3);
		int t1 = calcNumCorrectReplies(lowThirdList, qInfo, lowThirdMark);
		int t2 = calcNumCorrectReplies(highThirdList, qInfo, highThirdMark);
		double validity = (3.0*(t2-t1))/studentStats.getNumStudents();
		return new Double(validity);
	}
	public Double getAvgRightAnswersPerStudent(){
		return studentStats.getAvgRightAnswersPerStudent();
	}
	public Double getStdRightAnswersPerStudent(){
		return studentStats.getStdRightAnswersPerStudent();
	}
	public double getLowThirdRightAnswersPerStudent(){
		return studentStats.getLowThirdRightAnswersPerStudent();
	}
	public double getHighThirdRightAnswersPerStudent(){
		return studentStats.getHighThirdRightAnswersPerStudent();
	}
	
	public Integer getNumAnswersToQuestion(int verId){
		return studentStats.getNumAnswersToQuestion(verId);
	}
	public Double getAvgNumCorrectRepliesPerQuestion(){
		if (numQuestions<1) return null;
		return new Double(((double)correctAnswersSum)/numQuestions);
	}

	public Double getStdNumCorrectRepliesPerQuestion(){
		double m0 = numQuestions;
		double m1 = correctAnswersSum;
		double m2 = correctAnswersSum2;
		if (m0<2) return null;
		double stdCorrectAnswers = Math.sqrt((m0*m2-m1*m1)/(m0*(m0-1)));
		return new Double(stdCorrectAnswers);
	}
	public double getLowThirdNumCorrectRepliesPerQuestion(){
		return MedCorrectRepliesPerQuestion(1.0/3);
	}
	public double getHighThirdNumCorrectRepliesPerQuestion(){
		return MedCorrectRepliesPerQuestion(2.0/3);
	}
	private double MedCorrectRepliesPerQuestion(double fract){
		int low = (int)Math.floor(fract * (numQuestions- 1));
		int high =(int)Math.round(fract * (numQuestions- 1));
		QuestionInfo[] arr = new QuestionInfo[sortedQinfo.size()];
		sortedQinfo.toArray(arr);
		return fract * (arr[low]).getNumCorrectAnswers() + 
		  		(1 - fract) * (arr[high]).getNumCorrectAnswers();
	}
	public int getNumCorrectReplies(int verId){
		return ((QuestionInfo)questions.get(new Integer(verId))).getNumCorrectAnswers();
	}
	public Double getDifficulty(int verId){
        return ((QuestionInfo)questions.get(new Integer(verId))).getDifficulty();
    }
	public Double getValidity(int verId){
        return ((QuestionInfo)questions.get(new Integer(verId))).getValidity();
    }
	
	public HashMap getAnswersInfo(int verId){
		QuestionInfo qInfo = (QuestionInfo)questions.get(new Integer(verId));
		HashMap answersHash = qInfo.getAnswersHash(); 
		for (Iterator it = answersHash.keySet().iterator(); it.hasNext();){
			Integer ansId =  (Integer) it.next();
			AnswerInfo aInfo = (AnswerInfo)answersHash.get(ansId);
			aInfo.setTotalNumStudents(studentStats.getNumStudents());
			aInfo.setTotalGradesAverage(studentStats.getAverage());
			aInfo.setTotalGradesStd(studentStats.getStd());
		}
		return answersHash;
	}
	
    public String calcP1(int verId, int answerOrdinal){
        HashMap ansMap = getAnswersInfo(verId);
        AnswerInfo aInfo = null;
        for (Iterator it = ansMap.keySet().iterator();it.hasNext();){
            Integer ansId = (Integer)it.next();
            AnswerInfo ansInfo = (AnswerInfo)ansMap.get(ansId);
            if (ansInfo.getOrdinal() == answerOrdinal) {
                aInfo = ansInfo;
                break;
            } 
        }
        int numStudentsThatChoseThisAnswer = aInfo==null? 0 : aInfo.getNumAnsweringStudents();
        int numStudentsThatTookTheTest = studentStats.getNumStudents();
        if (numStudentsThatTookTheTest == 0) return "N/A";
        Double percent = new Double(100*numStudentsThatChoseThisAnswer/((double)numStudentsThatTookTheTest));
        String percentStr = percent.toString();
        if (percentStr.indexOf(".")>=0) percentStr = percentStr.substring(0,percentStr.indexOf("."));
        return percentStr.concat("%");
    }
    
    public String calcP2(int verId, int answerOrdinal){
        HashMap ansMap = getAnswersInfo(verId);
        AnswerInfo aInfo = null;
        for (Iterator it = ansMap.keySet().iterator();it.hasNext();){
            Integer ansId = (Integer)it.next();
            AnswerInfo ansInfo = (AnswerInfo)ansMap.get(ansId);
            if (ansInfo.getOrdinal() == answerOrdinal) {
                aInfo = ansInfo;
                break;
            } 
        }
        int numStudentsThatChoseThisAnswer = aInfo==null? 0 : aInfo.getNumAnsweringStudents();
        int numStudentsThatAnsweredThisQuestion = getNumAnswersToQuestion(verId).intValue();
        if (numStudentsThatAnsweredThisQuestion == 0) return "N/A";
        Double percent = new Double(100*numStudentsThatChoseThisAnswer/((double)numStudentsThatAnsweredThisQuestion));
        String percentStr = percent.toString();
        if (percentStr.indexOf(".")>=0) percentStr = percentStr.substring(0,percentStr.indexOf("."));
        return percentStr.concat("%");
    }
    
	//write here functions that get verId, ansId and return stuff about questions
	
}
