package il.ac.technion.cs.course236308.e3.med.stats;

import il.ac.technion.cs.course236308.e3.med.exception.CriticalException;
import il.ac.technion.cs.course236308.e3.med.exception.RetriableException;
import il.ac.technion.cs.course236308.e3.med.model.Answer;
import il.ac.technion.cs.course236308.e3.med.model.AnswersFactory;
import il.ac.technion.cs.course236308.e3.med.model.CategoriesFactory;
import il.ac.technion.cs.course236308.e3.med.model.Category;
import il.ac.technion.cs.course236308.e3.med.model.Question;
import il.ac.technion.cs.course236308.e3.med.model.QuestionsFactory;
import il.ac.technion.cs.course236308.e3.med.model.Test;
import il.ac.technion.cs.course236308.e3.med.model.TestAnswer;
import il.ac.technion.cs.course236308.e3.med.model.TestAnswersFactory;
import il.ac.technion.cs.course236308.e3.med.model.TestCategoriesFactory;
import il.ac.technion.cs.course236308.e3.med.model.TestVersion;
import il.ac.technion.cs.course236308.e3.med.model.TestVersionsFactory;
import il.ac.technion.cs.course236308.e3.med.model.TestsFactory;
import il.ac.technion.cs.course236308.e3.med.model.Version;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

/**
 * @author Shraer Alexander
 */
public class StatsUpdateDB {
	
	public static StatsUpdateDB instance = new StatsUpdateDB();
	private StatsUpdateDB() {}
	public static StatsUpdateDB getInstance(){
		return instance;
	}
	public void update(int testId) throws RetriableException, CriticalException{
		Test test = TestsFactory.getInstance().getTest(testId);
		QuestionsStats qStats = StatisticData.getInstance().getQuestionsStats(testId);
		
		for (Iterator it = test.getTestVersions().iterator(); it.hasNext();){
			TestVersion tv = (TestVersion)it.next();
			int verId = tv.getVersion().getId();
			tv.setDifficulty(qStats.getNormDifficulty(verId));
			tv.setDI(qStats.getValidity(verId));
			Version ver = tv.getVersion();
			updateVersionStatistics(ver);
			Question question = QuestionsFactory.getInstance().getQuestion(ver.getQuestionId());
			updateQuestionStatistics(question);
			Category category = CategoriesFactory.getInstance().getCategory(question.getMainCategoryId());
			updateCategoryStatistics(category);
			HashMap answersMap = qStats.getAnswersInfo(tv.getVersion().getId());
			for (Iterator it2 = tv.getTestAnswers().iterator();it2.hasNext(); ){
				TestAnswer ta = (TestAnswer)it2.next();
				AnswerInfo aInfo = (AnswerInfo)answersMap.get(new Integer(ta.getAnswerId()));
				ta.setAvgGrade(aInfo.getNormGradeAvg());
				ta.setCorrelation(aInfo.getCorelation());
				ta.setPercentage(aInfo.getPercentAnsweringStudents());
				Answer answer = AnswersFactory.getInstance().getAnswer(ta.getAnswerId());
				updateAnswerStatistics(answer);
			}	
		}
       
        ComputeTestStatisticsDFS cdfs = new ComputeTestStatisticsDFS(qStats.getSumQuestinsWeights());
        cdfs.dfs(TestCategoriesFactory.getInstance().getTestCategoryNode("/",test));
	}
	
	private void updateAnswerStatistics(Answer answer){
		List tasList = TestAnswersFactory.getInstance().getTestAnswers(answer.getId());
		Counter gradeCounter = new Counter();
		Counter correlationCounter = new Counter();
		Counter percentageCouner = new Counter();
		for (Iterator it = tasList.iterator();it.hasNext();){
			TestAnswer ta = (TestAnswer)it.next();
			QuestionsStats qs = StatisticData.getInstance().getQuestionsStats(ta.getTestId());
			int numStudents = qs.getStudentStats().getNumStudents();
			gradeCounter.count(ta.getAvgGrade(), numStudents);
			correlationCounter.count(ta.getCorrelation(), numStudents);
			percentageCouner.count(ta.getPercentage(), numStudents);
		}
		answer.setAvgGrade(gradeCounter.getAvg());
		answer.setAvgCorrelation(correlationCounter.getAvg());
		answer.setAvgPercentage(percentageCouner.getAvg());
	}
	private void updateVersionStatistics(Version ver){
		List tvsList = TestVersionsFactory.getInstance().getTestVersions(ver.getId());
		Counter diCounter = new Counter();
		Counter difficultyCounter = new Counter();
		for (Iterator it = tvsList.iterator();it.hasNext();){
			TestVersion tv = (TestVersion)it.next();
			QuestionsStats qs = StatisticData.getInstance().getQuestionsStats(tv.getTestId());
			int numStudents = qs.getStudentStats().getNumStudents();
			diCounter.count(tv.getDI(), numStudents);
			difficultyCounter.count(tv.getDifficulty(), numStudents);
		}
		ver.setAvgDI(diCounter.getAvg());
		ver.setNumStudentsDI(diCounter.getNumSpeciments());
		
		ver.setAvgDifficulty(difficultyCounter.getAvg());
		ver.setNumStudentsDifficulty(difficultyCounter.getNumSpeciments());
	}
	
	private void updateQuestionStatistics(Question question){
		List versList = question.getVersions();
		Counter diCounter = new Counter();
		Counter difficultyCounter = new Counter();
		for (Iterator it = versList.iterator(); it.hasNext();){
			Version ver = (Version)it.next();
			diCounter.count(ver.getAvgDI(), ver.getNumStudentsDI());
			difficultyCounter.count(ver.getAvgDifficulty(), ver.getNumStudentsDifficulty());
		}
		question.setAvgDI(diCounter.getAvg());
		question.setNumStudentsDI(diCounter.getNumSpeciments());
		question.setAvgDifficulty(difficultyCounter.getAvg());
		question.setNumStudentsDifficulty(difficultyCounter.getNumSpeciments());
	}
	
	public void updateCategoryStatistics(Category cat) throws RetriableException, CriticalException{
		List questionsList = cat.getQuestions();
		
		Counter diCounter = new Counter();
		Counter difficultyCounter = new Counter();
	
		for (Iterator it = questionsList.iterator();it.hasNext();){
			Question question = (Question)it.next();
			diCounter.count(question.getAvgDI(), question.getNumStudentsDI());
			difficultyCounter.count(question.getAvgDifficulty(), question.getNumStudentsDifficulty());
		}
		cat.setAvgDICatOnly(diCounter.getAvg());
		cat.setNumStudentsDICatOnly(diCounter.getNumSpeciments());
		cat.setAvgDifficultyCatOnly(difficultyCounter.getAvg());
		cat.setNumStudentsDifficultyCatOnly(difficultyCounter.getNumSpeciments());
		
		Integer curId = new Integer(cat.getId());
		while(curId!=null){
			cat = CategoriesFactory.getInstance().getCategory(curId.intValue());
			Counter diCounterCatAndBellow = new Counter();
			Counter difficultyCounterCatAndBellow = new Counter();
			diCounterCatAndBellow.count(cat.getAvgDICatOnly(), cat.getNumStudentsDICatOnly());
			difficultyCounterCatAndBellow.count(cat.getAvgDifficultyCatOnly(), cat.getNumStudentsDifficultyCatOnly());

			for (Iterator it = cat.getCategories().iterator();it.hasNext();){
				Category subCat = (Category)it.next();
				diCounterCatAndBellow.count(subCat.getAvgDICatAndBellow(),subCat.getNumStudentsDICatAndBellow());
				difficultyCounterCatAndBellow.count(subCat.getAvgDifficultyCatAndBellow(), subCat.getNumStudentsDifficultyCatAndBellow());
			}
			cat.setAvgDICatAndBellow(diCounterCatAndBellow.getAvg());
			cat.setNumStudentsDICatAndBellow(diCounterCatAndBellow.getNumSpeciments());
			cat.setAvgDifficultyCatAndBellow(difficultyCounterCatAndBellow.getAvg());
			cat.setNumStudentsDifficultyCatAndBellow(difficultyCounterCatAndBellow.getNumSpeciments());
			curId = cat.getParentID();
		}
	}
}
